/*
;$Id: primes.pro,v 1.11 2003/02/03 18:13:23 scottm Exp $
;
; Copyright (c) 1994-2003, Research Systems, Inc.  All rights reserved.
;       Unauthorized reproduction prohibited.
;+
; NAME:
;       PRIMES
;
; PURPOSE:
;       This function computes the first K prime numbers. The result is a 
;       K-element vector of type long integer.
;
; CATEGORY:
;       Statistics.
;
; CALLING SEQUENCE:
;       Result = Primes(K)
;
; INPUTS:
;       K:    A scalar of type integer or long integer that specifies the 
;             number of primes to be computed.
;
; EXAMPLE:
;       Compute the first 25 prime numbers.
;         result = primes(25)
;
;       The result should be:
;         [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, $
;          53, 59, 61, 67, 71, 73, 79, 83, 89, 97]
;
; REFERENCE:
;       PROBABILITY and STATISTICS for ENGINEERS and SCIENTISTS (3rd edition)
;       Ronald E. Walpole & Raymond H. Myers
;       ISBN 0-02-424170-9
;
; MODIFICATION HISTORY:
;       Written by:  GGS, RSI, November 1994
;                    S. Lett, RSI, December 1997, added support for k=1
;-
*/
function primes, k {
  oldbase = indexbase()
// Compute the first k prime numbers.

  if (k == 1) {
    primes = [2]
	break
  }
  prm = longarray(k)

  cnotation

  prm[0] = 2
  n = 3
  count = 1
  prm[count] = 3

  count++

  while(count < k) {
    n = n + 2

    for ip = 1, count {
      q = n / prm[ip]
      r = n % prm[ip]
      if (r == 0) {
		ip = count + 1    // n is not prime.
      } else {
	    if (q <= prm[ip]) {  // n is prime.
          prm[count] = n
          count++               // compute next prime.
		  ip = count + 1
		}
      }

    }

  }

  primes = prm

  setindexbase(oldbase)
}
