C*GRSY00 -- initialize font definition
C+
      SUBROUTINE GRSY00
C
C This routine must be called once in order to initialize the tables
C defining the symbol numbers to be used for ASCII characters in each
C font, and to read the character digitization from a file.
C
C Arguments: none.
C
C Implicit input:
C  The file with name specified in environment variable PGPLOT_FONT
C  is read, if it is available.
C  This is a binary file containing two arrays INDEX and BUFFER.
C  The digitization of each symbol occupies a number of words in
C  the INTEGER*2 array BUFFER; the start of the digitization
C  for symbol number N is in BUFFER(INDEX(N)), where INDEX is an
C  integer array of 3000 elements. Not all symbols 1...3000 have
C  a representation; if INDEX(N) = 0, the symbol is undefined.
C
*  PGPLOT uses the Hershey symbols for two `primitive' operations:
*  graph markers and text.  The Hershey symbol set includes several
*  hundred different symbols in a digitized form that allows them to
*  be drawn with a series of vectors (polylines).
*
*  The digital representation of all the symbols is stored in common
*  block /GRSYMB/.  This is read from a disk file at run time. The
*  name of the disk file is specified in environment variable
*  PGPLOT_FONT.
*
* Modules:
*
* GRSY00 -- initialize font definition
* GRSYDS -- decode character string into list of symbol numbers
* GRSYMK -- convert marker number into symbol number
* GRSYXD -- obtain the polyline representation of a given symbol
*
* PGPLOT calls these routines as follows:
*
* Routine          Called by
*
* GRSY00          GROPEN
* GRSYDS          GRTEXT, GRLEN
* GRSYMK          GRMKER,
* GRSYXD          GRTEXT, GRLEN, GRMKER
***********************************************************************
C--
C (2-Jan-1984)
C 22-Jul-1984 - revise to use DATA statements [TJP].
C  5-Jan-1985 - make missing font file non-fatal [TJP].
C  9-Feb-1988 - change default file name to Unix name; overridden
C               by environment variable PGPLOT_FONT [TJP].
C 29-Nov-1990 - move font assignment to GRSYMK.
C  7-Nov-1994 - look for font file in PGPLOT_DIR if PGPLOT_FONT is
C               undefined [TJP].
C-----------------------------------------------------------------------
      INTEGER*2  BUFFER(27000)
      INTEGER    FNTFIL, IER, INDEX(3000), NC1, NC2, NC3
      INTEGER    L, GRTRIM
      COMMON     /GRSYMB/ NC1, NC2, INDEX, BUFFER
      CHARACTER*128 FF

      DATA INDEX/
     c    1,   13,   32,   46,   61,   76,   88,  106,  118,  124
     c,  134,  146,  155,  170,  182,  199,  212,  232,  248,  264
     c,  273,  285,  294,  309,  318,  328,  340,  352,  371,  380
     c,  392,  407,  419,  431,  454,  460,  472,  481,  496,  508
     c,  523,  540,  552,  565,  578,  587,  605,  621,  630,  645
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,  661,  666,  671,  676
     c,  691,  698,  710,  728,  738,  755,  772,  781,  803,  820
     c,  829,  839,  854,  870,  885,  905,  911,  920,  933,  952
     c,  958,  968,  978,  984,  990,  999, 1008, 1017, 1029, 1038
     c, 1048, 1058, 1068, 1083, 1107, 1130,    0, 1154, 1169, 1204
     c,    0, 1210,    0,    0,    0,    0,    0, 1219,    0, 1266
     c,    0, 1297,    0, 1308,    0, 1317,    0, 1349, 1355,    0
     c, 1364, 1415, 1470, 1522, 1589, 1669, 1765, 1855, 1934, 2009
     c, 2038, 2096, 2138, 2200, 2249, 2264,    0, 2285, 2311, 2337
     c, 2356, 2375,    0, 2394,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c, 2406, 2418, 2445, 2467, 2486, 2501, 2513, 2539, 2551, 2557
     c, 2571, 2583, 2592, 2607, 2619, 2644, 2661, 2689, 2709, 2733
     c, 2742, 2756, 2765, 2780, 2789, 2799, 2811, 2823, 2850, 2859
     c, 2871, 2886, 2898, 2910, 2938, 2944, 2956, 2965, 2980, 2992
     c, 3004, 3029, 3041, 3058, 3071, 3080, 3102, 3126, 3135, 3156
     c, 3176, 3199, 3243, 3270, 3308, 3339, 3370, 3402, 3443, 3471
     c, 3499, 3541, 3573, 3621, 3656, 3688, 3722, 3757, 3798, 3829
     c, 3857, 3893, 3928, 3956, 3995, 4036,    0,    0,    0,    0
     c,    0,    0, 4079,    0,    0,    0,    0,    0,    0, 4091
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c, 4097, 4118, 4139, 4157, 4178, 4199, 4211, 4237, 4251, 4263
     c, 4278, 4290, 4296, 4318, 4332, 4353, 4374, 4395, 4407, 4428
     c, 4440, 4454, 4463, 4478, 4487, 4500, 4512, 4539, 4573, 4593
     c, 4620, 4642, 4668, 4690, 4720, 4732, 4754, 4766, 4790, 4807
     c, 4839, 4860, 4876, 4898, 4919, 4930, 4949, 4973, 4990, 5013
     c, 5039, 5064, 5090, 5107, 5134, 5154, 5181, 5212, 5244, 5263
     c, 5286, 5322, 5343, 5379, 5405, 5431, 5458, 5488, 5506, 5525
     c, 5544, 5566, 5586, 5614, 5637, 5663, 5689,    0,    0,    0
     c,    0,    0, 5714, 5743, 5762, 5788, 5812,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0, 5831, 5836, 5841, 5846
     c, 5867, 5875, 5893, 5912, 5922, 5943, 5970, 5979, 6012, 6039
     c, 6048, 6060, 6075, 6093, 6105, 6129, 6135, 6144, 6161, 6191
     c, 6197, 6211, 6225, 6231, 6237, 6246, 6255, 6264, 6276, 6285
     c, 6296, 6307, 6319, 6334, 6372,    0, 6403, 6412, 6421, 6428
     c, 6449, 6485, 6514, 6537, 6588, 6646,    0,    0,    0, 6704
     c, 6725, 6744, 6756, 6787, 6816, 6833, 6846, 6860, 6876, 6890
     c, 6900, 6909, 6919, 6929, 6953, 6977, 7011, 7036,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0, 7069, 7075, 7081, 7087, 7093
     c, 7099, 7105, 7111, 7117, 7123, 7129, 7135, 7141, 7147, 7153
     c, 7164, 7175, 7186, 7197, 7209, 7221, 7233, 7245, 7253, 7261
     c, 7269, 7277, 7303, 7329, 7355, 7381, 7405, 7427, 7456, 7470
     c, 7482, 7490, 7499, 7511,    0,    0,    0,    0,    0, 7523
     c, 7544, 7553, 7561, 7570, 7585, 7594, 7603,    0,    0, 7615
     c, 7653, 7683, 7703, 7723, 7743, 7763, 7788,    0,    0, 7802
     c, 7818, 7832, 7849, 7870, 7894, 7908, 7925, 7960, 7973, 8010
     c, 8077, 8174, 8217, 8252,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0, 8270, 8279
     c, 8292, 8309, 8330, 8351, 8376, 8413, 8450, 8503, 8540, 8576
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c, 8610, 8631, 8679, 8714, 8747, 8772, 8795, 8838, 8868, 8883
     c, 8906, 8936, 8953, 8986, 9010, 9057, 9089, 9156, 9204, 9241
     c, 9260, 9286, 9304, 9331, 9355, 9378, 9397, 9418, 9466, 9483
     c, 9501, 9526, 9545, 9575, 9634, 9649, 9679, 9697, 9730, 9754
     c, 9793, 9840, 9864, 9896, 9919, 9938, 9974,10025,10049,10093
     c,10139,10160,10204,10241,10274,10299,10322,10367,10397,10412
     c,10436,10466,10483,10516,10540,10585,10615,10679,10724,10762
     c,10781,10809,10827,10854,10878,10901,10920,10969,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,11004,11046,11082,11113,11152,11186,11211,11274,11305,11326
     c,11354,11384,11399,11446,11477,11516,11555,11591,11617,11652
     c,11671,11702,11720,11747,11771,11796,11815,11858,11918,11949
     c,11996,12031,12065,12100,12147,12165,12197,12223,12254,12281
     c,12329,12364,12389,12423,12461,12480,12514,12554,12580,12617
     c,12664,12706,12745,12773,12818,12847,12885,12931,12965,12994
     c,13029,13066,13087,13142,13182,13217,13262,13298,13327,13358
     c,13379,13419,13448,13492,13537,13581,13615,13661,13697,13735
     c,13792,13851,    0,13866,13894,13940,13985,    0,    0,14015
     c,14063,14135,14190,14247,14336,14427,14450,14455,14460,14465
     c,14508,14522,14570,14620,14636,14678,14729,14763,14829,14880
     c,14889,14900,14915,14932,14950,14985,14994,15009,15026,15038
     c,15044,15067,15090,15105,15120,15149,15178,15185,15192,15198
     c,15207,15213,15222,15234,15246,15255,15264,15282,15291,15303
     c,15315,15322,15329,15342,15355,15379,15406,15417,15427,15437
     c,15458,15469,15480,15491,15502,15515,15531,15547,15563,15579
     c,15598,15612,15626,15640,15654,15701,15719,15732,15767,15820
     c,15849,15884,15936,15995,16040,16055,16104,16137,16196,    0
     c,16211,16252,16300,16330,16361,16395,16426,16459,16506,16546
     c,16580,16607,16637,16652,16703,16754,16770,16786,16802,    0
     c,16818,16861,16915,16948,16999,17050,17101,17145,17178,17214
     c,17250,17289,    0,    0,    0,    0,17340,17355,17376,17397
     c,17436,17475,17512,17533,17554,17582,17603,17624,17676,17705
     c,17794,17900,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,17933,17948,17969,17990
     c,18025,18064,18101,18122,18143,18171,18192,18213,18252,18281
     c,18370,18462,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,18550,18574,18600,18631,18662,18679,18696,18737,18778,18813
     c,18848,18865,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,18925,18948,18995,19036,19071,19101,19125,19172,19197,19208
     c,19231,19256,19273,19302,19325,19368,19398,19449,19486,19532
     c,19550,19577,19594,19623,19642,19662,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,19685,19726,19799,19843,19889,19941,19990,20046,20104,20140
     c,20177,20239,20282,20334,20375,20412,20468,20514,20582,20621
     c,20662,20704,20743,20787,20842,20887,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,20934,20973,21012,21047,21086,21125,21152,21203,21231,21258
     c,21285,21310,21321,21366,21394,21433,21472,21511,21535,21588
     c,21607,21635,21652,21681,21700,21720,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,21743,21779,21805,21830,21866,21892,21922,21963,21996,22018
     c,22045,22079,22099,22147,22182,22214,22247,22285,22312,22337
     c,22360,22390,22416,22455,22500,22535,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,22576,22581,22586,22591
     c,22636,22651,22688,22739,22760,22816,22881,22896,22967,23032
     c,23051,23078,23113,23156,23183,23244,23271,23298,23372,23426
     c,23437,23464,23491,23533,23544,23563,23582,23596,23621,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,23654,23659,23664,23669
     c,23714,23732,23777,23830,23843,23885,23934,23967,24033,24082
     c,24091,24102,24116,24133,24153,24190,24201,24212,24270,24314
     c,24320,24343,24366,24378,24384,24393,24402,24411,24426,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,24443,24464,24498,24546,24563,24597,24622,24696,24744,24774
     c,24814,24860,24888,24921,24951,24998,25022,25054,25089,25108
     c,25135,25186,25210,25240,25271,25304,25343,25377,25421,25453
     c,25495,25557,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,25605,25647,25698,25738,25755,25780,25814,25876,25921,25951
     c,25991,26032,26057,26090,26120,26159,26183,26222,26253,26272
     c,26297,26352,26376,26402,26433,26466,26501,26531,26571,26599
     c,26636,26690,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0/

      DATA BUFFER/
     c   -4, 7749, 7621, 8261, 7740,   64, 8261, 8764,   64, 7999
     c, 8511,    0,   -4, 7749, 7621, 7877, 7868,   64, 7877, 8389
     c, 8644, 8642, 8385,   64, 7873, 8385, 8640, 8637, 8380, 7868
     c,    0,   -4, 7749, 7622, 8772, 8517, 8261, 8004, 7874, 7871
     c, 7997, 8252, 8508, 8765,    0,   -4, 7749, 7621, 7877, 7868
     c,   64, 7877, 8261, 8516, 8642, 8639, 8509, 8252, 7868,    0
     c,   -4, 7749, 7621, 7877, 7868,   64, 7877, 8645,   64, 7873
     c, 8385,   64, 7868, 8636,    0,   -4, 7749, 7620, 7877, 7868
     c,   64, 7877, 8645,   64, 7873, 8385,    0,   -4, 7749, 7622
     c, 8772, 8517, 8261, 8004, 7874, 7871, 7997, 8252, 8508, 8765
     c, 8768,   64, 8384, 8768,    0,   -4, 7749, 7621, 7877, 7868
     c,   64, 8645, 8636,   64, 7873, 8641,    0,   -4, 7749, 8002
     c, 8261, 8252,    0,   -4, 7749, 7747, 8389, 8382, 8252, 7996
     c, 7870, 7871,    0,   -4, 7749, 7621, 7877, 7868,   64, 8645
     c, 7871,   64, 8129, 8636,    0,   -4, 7749, 7620, 7877, 7868
     c,   64, 7868, 8636,    0,   -4, 7749, 7494, 7749, 7740,   64
     c, 7749, 8252,   64, 8773, 8252,   64, 8773, 8764,    0,   -4
     c, 7749, 7621, 7877, 7868,   64, 7877, 8636,   64, 8645, 8636
     c,    0,   -4, 7749, 7622, 8261, 8004, 7874, 7871, 7997, 8252
     c, 8380, 8637, 8767, 8770, 8644, 8389, 8261,    0,   -4, 7749
     c, 7621, 7877, 7868,   64, 7877, 8389, 8644, 8641, 8384, 7872
     c,    0,   -4, 7749, 7622, 8261, 8004, 7874, 7871, 7997, 8252
     c, 8380, 8637, 8767, 8770, 8644, 8389, 8261,   64, 8382, 8763
     c,    0,   -4, 7749, 7621, 7877, 7868,   64, 7877, 8389, 8644
     c, 8641, 8384, 7872,   64, 8256, 8636,    0,   -4, 7749, 7621
     c, 8644, 8389, 8133, 7876, 7875, 8002, 8512, 8639, 8637, 8380
     c, 8124, 7869,    0,   -4, 7749, 7621, 8261, 8252,   64, 7749
     c, 8773,    0,   -4, 7749, 7622, 7877, 7871, 7997, 8252, 8380
     c, 8637, 8767, 8773,    0,   -4, 7749, 7621, 7749, 8252,   64
     c, 8773, 8252,    0,   -4, 7749, 7494, 7749, 7996,   64, 8261
     c, 7996,   64, 8261, 8508,   64, 8773, 8508,    0,   -4, 7749
     c, 7621, 7877, 8636,   64, 8645, 7868,    0,   -4, 7749, 7621
     c, 7749, 8257, 8252,   64, 8773, 8257,    0,   -4, 7749, 7621
     c, 8645, 7868,   64, 7877, 8645,   64, 7868, 8636,    0,   -4
     c, 7749, 7621, 8261, 7740,   64, 8261, 8764,   64, 7999, 8511
     c,    0,   -4, 7749, 7621, 7877, 7868,   64, 7877, 8389, 8644
     c, 8642, 8385,   64, 7873, 8385, 8640, 8637, 8380, 7868,    0
     c,   -4, 7749, 7620, 7877, 7868,   64, 7877, 8645,    0,   -4
     c, 7749, 7621, 8261, 7740,   64, 8261, 8764,   64, 7740, 8764
     c,    0,   -4, 7749, 7621, 7877, 7868,   64, 7877, 8645,   64
     c, 7873, 8385,   64, 7868, 8636,    0,   -4, 7749, 7621, 8645
     c, 7868,   64, 7877, 8645,   64, 7868, 8636,    0,   -4, 7749
     c, 7621, 7877, 7868,   64, 8645, 8636,   64, 7873, 8641,    0
     c,   -4, 7749, 7622, 8261, 8004, 7874, 7871, 7997, 8252, 8380
     c, 8637, 8767, 8770, 8644, 8389, 8261,   64, 8129, 8512,   64
     c, 8513, 8128,    0,   -4, 7749, 8002, 8261, 8252,    0,   -4
     c, 7749, 7621, 7877, 7868,   64, 8645, 7871,   64, 8129, 8636
     c,    0,   -4, 7749, 7621, 8261, 7740,   64, 8261, 8764,    0
     c,   -4, 7749, 7494, 7749, 7740,   64, 7749, 8252,   64, 8773
     c, 8252,   64, 8773, 8764,    0,   -4, 7749, 7621, 7877, 7868
     c,   64, 7877, 8636,   64, 8645, 8636,    0,   -4, 7749, 7621
     c, 7877, 8645,   64, 8001, 8512,   64, 8513, 8000,   64, 7868
     c, 8636,    0,   -4, 7749, 7622, 8261, 8004, 7874, 7871, 7997
     c, 8252, 8380, 8637, 8767, 8770, 8644, 8389, 8261,    0,   -4
     c, 7749, 7621, 7877, 7868,   64, 8645, 8636,   64, 7877, 8645
     c,    0,   -4, 7749, 7621, 7877, 7868,   64, 7877, 8389, 8644
     c, 8641, 8384, 7872,    0,   -4, 7749, 7621, 7877, 8257, 7868
     c,   64, 7877, 8645,   64, 7868, 8636,    0,   -4, 7749, 7621
     c, 8261, 8252,   64, 7749, 8773,    0,   -4, 7749, 7621, 7747
     c, 7748, 7877, 8005, 8132, 8258, 8252,   64, 8771, 8772, 8645
     c, 8517, 8388, 8258,    0,   -4, 7749, 7494, 8261, 8252,   64
     c, 8003, 7746, 7743, 7998, 8510, 8767, 8770, 8515, 8003,    0
     c,   -4, 7749, 7621, 7877, 8636,   64, 8645, 7868,    0,   -4
     c, 7749, 7494, 8261, 8252,   64, 7747, 7874, 7871, 8126, 8382
     c, 8639, 8642, 8771,    0,   -4, 7749, 7622, 7868, 8124, 7872
     c, 7874, 8004, 8261, 8389, 8644, 8770, 8768, 8508, 8764,    0
     c,   -4, 7749, 8256,   64,    0,   -4, 7749, 8002,   64,    0
     c,   -4, 7749, 7748,   64,    0,   -4, 7749, 7621, 8261, 8004
     c, 7874, 7871, 7997, 8252, 8509, 8639, 8642, 8516, 8261,    0
     c,   -4, 7749, 7621, 8003, 8261, 8252,    0,   -4, 7749, 7621
     c, 7876, 8133, 8389, 8644, 8642, 8512, 7868, 8636,    0,   -4
     c, 7749, 7621, 7876, 8133, 8389, 8644, 8642, 8385,   64, 8257
     c, 8385, 8640, 8637, 8380, 8124, 7869,    0,   -4, 7749, 7621
     c, 8389, 8380,   64, 8389, 7743, 8767,    0,   -4, 7749, 7621
     c, 8005, 7873, 8130, 8258, 8513, 8639, 8509, 8252, 8124, 7869
     c,   64, 8005, 8517,    0,   -4, 7749, 7621, 8517, 8261, 8004
     c, 7874, 7871, 7997, 8252, 8509, 8639, 8513, 8258, 8001, 7871
     c,    0,   -4, 7749, 7621, 8645, 8124,   64, 7877, 8645,    0
     c,   -4, 7749, 7621, 8133, 7876, 7874, 8129, 8385, 8642, 8644
     c, 8389, 8133,   64, 8129, 7872, 7869, 8124, 8380, 8637, 8640
     c, 8385,    0,   -4, 7749, 7621, 8642, 8512, 8255, 8000, 7874
     c, 8004, 8261, 8516, 8642, 8639, 8509, 8252, 7996,    0,   -4
     c, 7749, 8003, 8253, 8252, 8380, 8381, 8253,    0,   -4, 7749
     c, 8003, 8380, 8252, 8253, 8381, 8379, 8249,    0,   -4, 7749
     c, 8003, 8258, 8257, 8385, 8386, 8258,   64, 8253, 8252, 8380
     c, 8381, 8253,    0,   -4, 7749, 8003, 8258, 8257, 8385, 8386
     c, 8258,   64, 8380, 8252, 8253, 8381, 8379, 8249,    0,   -4
     c, 7749, 8003, 8261, 8256,   64, 8389, 8384,   64, 8253, 8252
     c, 8380, 8381, 8253,    0,   -4, 7749, 7749, 8004, 8261, 8389
     c, 8644, 8642, 8257, 8256, 8384, 8385, 8642,   64, 8253, 8252
     c, 8380, 8381, 8253,    0,   -4, 7749, 8002, 8261, 8257,    0
     c,   -4, 7749, 7748, 8005, 8001,   64, 8517, 8513,    0,   -4
     c, 7749, 7748, 8133, 8004, 8002, 8129, 8385, 8514, 8516, 8389
     c, 8133,    0,   -4, 7749, 7621, 8644, 8389, 8133, 7876, 7874
     c, 8129, 8512, 8639, 8637, 8380, 8124, 7869,   64, 8262, 8251
     c,    0,   -4, 7749, 7621, 8774, 7739,    0,   -4, 7749, 7876
     c, 8518, 8260, 8130, 8127, 8253, 8507,    0,   -4, 7749, 7747
     c, 8006, 8260, 8386, 8383, 8253, 7995,    0,   -4, 7749, 8002
     c, 8262, 8251,    0,   -4, 7749, 7494, 7744, 8768,    0,   -4
     c, 7749, 7494, 8260, 8252,   64, 7744, 8768,    0,   -4, 7749
     c, 7494, 7746, 8770,   64, 7742, 8766,    0,   -4, 7749, 7621
     c, 7875, 8637,   64, 8643, 7869,    0,   -4, 7749, 7621, 8259
     c, 8253,   64, 7874, 8638,   64, 8642, 7870,    0,   -4, 7749
     c, 8003, 8257, 8256, 8384, 8385, 8257,    0,   -4, 7749, 8003
     c, 8389, 8259, 8257, 8385, 8386, 8258,    0,   -4, 7749, 8003
     c, 8388, 8260, 8261, 8389, 8387, 8257,    0,   -4, 7749, 7494
     c, 8387, 8768, 8381,   64, 7744, 8768,    0,   -4, 7749, 7622
     c, 8262, 7995,   64, 8646, 8379,   64, 7874, 8770,   64, 7871
     c, 8767,    0,   -4, 7749, 7494, 8768, 8640, 8511, 8381, 8252
     c, 7868, 7741, 7743, 7872, 8257, 8386, 8388, 8261, 8005, 7876
     c, 7874, 8127, 8381, 8636, 8764,    0,   -4, 7749, 7494, 7748
     c, 7873, 7871, 7740,   64, 8772, 8641, 8639, 8764,   64, 7748
     c, 8131, 8387, 8772,   64, 7740, 8125, 8381, 8764,    0,   -4
     c, 7749, 7621, 8382, 8386, 8258, 8130, 8001, 7999, 8126, 8510
     c, 8768, 8769, 8643, 8388, 8132, 7875, 7745, 7743, 7869, 8124
     c, 8380, 8637,    0,  -16, 7116, 6986, 7744, 7617, 7488, 7615
     c, 7744,   64, 8768, 8897, 9024, 8895, 8768,    0,  -16, 7116
     c, 6986, 7618, 7489, 7488, 7615, 7743, 7872, 7873, 7746, 7618
     c,   64, 7617, 7616, 7744, 7745, 7617,   64, 8898, 8769, 8768
     c, 8895, 9023, 9152, 9153, 9026, 8898,   64, 8897, 8896, 9024
     c, 9025, 8897,    0,  -16, 7116, 7113, 7360, 9152,    0,  -16
     c, 7116, 6986, 8253, 7234, 8254, 9282, 8253,    0,  -16, 7116
     c, 6732, 7103, 7233, 7490, 7746, 8001, 8510, 8765, 9021, 9278
     c, 9409,   64, 7103, 7234, 7491, 7747, 8002, 8511, 8766, 9022
     c, 9279, 9409,   64, 7097, 7227, 7484, 7740, 7995, 8504, 8759
     c, 9015, 9272, 9403,   64, 7097, 7228, 7485, 7741, 7996, 8505
     c, 8760, 9016, 9273, 9403,    0,  -16, 7116, 6732, 7103, 7233
     c, 7490, 7746, 8001, 8510, 8765, 9021, 9278, 9409,   64, 7103
     c, 7234, 7491, 7747, 8002, 8511, 8766, 9022, 9279, 9409,   64
     c, 7099, 9403,   64, 7095, 9399,    0,  -16, 7116, 6351, 6851
     c, 9667, 9033,   64, 7479, 6845, 9661,    0,  -16, 7116, 6478
     c, 6708, 9780, 9804, 6732, 6708,    0,  -16, 7116, 7367, 8124
     c, 7869, 7743, 7745, 7875, 8132, 8388, 8643, 8769, 8767, 8637
     c, 8380, 8124,   64, 8380, 8767,   64, 8124, 8769,   64, 7869
     c, 8643,   64, 7743, 8388,   64, 7745, 8132,    0,  -16, 7116
     c, 6986, 7104, 9408,    0,  -16, 7116, 6986, 7104, 9408,   64
     c, 7105, 9409,    0,  -16, 7116, 7495, 8012, 8399, 8386,   64
     c, 8270, 8258,   64, 7874, 8770,   64, 7616, 9024,   64, 7868
     c, 7867, 7739, 7740, 7869, 8126, 8510, 8765, 8891, 8761, 8504
     c, 8119, 7862, 7732, 7729,   64, 8510, 8637, 8763, 8633, 8504
     c,   64, 7730, 7859, 7987, 8370, 8754, 8883,   64, 7987, 8369
     c, 8753, 8883, 8884,    0,  -16, 7116, 7495, 8012, 8399, 8386
     c,   64, 8270, 8258,   64, 7874, 8770,   64, 7616, 9024,   64
     c, 7868, 7867, 7739, 7740, 7869, 8126, 8510, 8765, 8891, 8761
     c, 8504,   64, 8510, 8637, 8763, 8633, 8504,   64, 8248, 8504
     c, 8759, 8885, 8884, 8754, 8497, 8113, 7858, 7731, 7732, 7860
     c, 7859,   64, 8504, 8631, 8757, 8756, 8626, 8497,    0,  -16
     c, 7116, 7495, 8012, 8399, 8386,   64, 8270, 8258,   64, 7874
     c, 8770,   64, 7616, 9024,   64, 8764, 8763, 8891, 8892, 8765
     c, 8510, 8254, 7997, 7868, 7737, 7732, 7858, 8113, 8497, 8754
     c, 8884, 8886, 8760, 8505, 8121, 7735,   64, 8254, 7996, 7865
     c, 7860, 7986, 8113,   64, 8497, 8626, 8756, 8758, 8632, 8505
     c,    0,  -16, 7116, 7495, 8012, 8399, 8386,   64, 8270, 8258
     c,   64, 7874, 8770,   64, 7616, 9024,   64, 8126, 7869, 7739
     c, 7865, 8120, 8504, 8761, 8891, 8765, 8510, 8126,   64, 8126
     c, 7997, 7867, 7993, 8120,   64, 8504, 8633, 8763, 8637, 8510
     c,   64, 8120, 7863, 7733, 7732, 7858, 8113, 8497, 8754, 8884
     c, 8885, 8759, 8504,   64, 8120, 7991, 7861, 7860, 7986, 8113
     c,   64, 8497, 8626, 8756, 8757, 8631, 8504,    0,  -16, 7116
     c, 7495, 7885, 7884, 7756, 7757, 7886, 8143, 8527, 8782, 8908
     c, 8778, 8521, 8136, 7879, 7749, 7746,   64, 8527, 8654, 8780
     c, 8650, 8521,   64, 7747, 7876, 8004, 8387, 8771, 8900,   64
     c, 8004, 8386, 8770, 8900, 8901,   64, 7616, 9024,   64, 7868
     c, 7867, 7739, 7740, 7869, 8126, 8510, 8765, 8891, 8761, 8504
     c,   64, 8510, 8637, 8763, 8633, 8504,   64, 8248, 8504, 8759
     c, 8885, 8884, 8754, 8497, 8113, 7858, 7731, 7732, 7860, 7859
     c,   64, 8504, 8631, 8757, 8756, 8626, 8497,    0,  -16, 7116
     c, 7495, 7885, 7884, 7756, 7757, 7886, 8143, 8527, 8782, 8908
     c, 8778, 8521,   64, 8527, 8654, 8780, 8650, 8521,   64, 8265
     c, 8521, 8776, 8902, 8901, 8771, 8514, 8130, 7875, 7748, 7749
     c, 7877, 7876,   64, 8521, 8648, 8774, 8773, 8643, 8514,   64
     c, 7616, 9024,   64, 8126, 7869, 7739, 7865, 8120, 8504, 8761
     c, 8891, 8765, 8510, 8126,   64, 8126, 7997, 7867, 7993, 8120
     c,   64, 8504, 8633, 8763, 8637, 8510,   64, 8120, 7863, 7733
     c, 7732, 7858, 8113, 8497, 8754, 8884, 8885, 8759, 8504,   64
     c, 8120, 7991, 7861, 7860, 7986, 8113,   64, 8497, 8626, 8756
     c, 8757, 8631, 8504,    0,  -16, 7116, 7495, 7887, 7753,   64
     c, 7887, 8783,   64, 7886, 8398, 8783,   64, 7753, 7882, 8139
     c, 8523, 8778, 8904, 8901, 8771, 8514, 8130, 7875, 7748, 7749
     c, 7877, 7876,   64, 8523, 8650, 8776, 8773, 8643, 8514,   64
     c, 7616, 9024,   64, 8126, 7869, 7739, 7865, 8120, 8504, 8761
     c, 8891, 8765, 8510, 8126,   64, 8126, 7997, 7867, 7993, 8120
     c,   64, 8504, 8633, 8763, 8637, 8510,   64, 8120, 7863, 7733
     c, 7732, 7858, 8113, 8497, 8754, 8884, 8885, 8759, 8504,   64
     c, 8120, 7991, 7861, 7860, 7986, 8113,   64, 8497, 8626, 8756
     c, 8757, 8631, 8504,    0,  -16, 7116, 7495, 7759, 7755,   64
     c, 8781, 8262, 8002,   64, 8911, 8521, 8130,   64, 7757, 8015
     c, 8271, 8653,   64, 7757, 8014, 8270, 8653, 8781,   64, 7616
     c, 9024,   64, 8126, 7869, 7739, 7865, 8120, 8504, 8761, 8891
     c, 8765, 8510, 8126,   64, 8126, 7997, 7867, 7993, 8120,   64
     c, 8504, 8633, 8763, 8637, 8510,   64, 8120, 7863, 7733, 7732
     c, 7858, 8113, 8497, 8754, 8884, 8885, 8759, 8504,   64, 8120
     c, 7991, 7861, 7860, 7986, 8113,   64, 8497, 8626, 8756, 8757
     c, 8631, 8504,    0,  -16, 7116, 7495, 7887, 7753,   64, 7887
     c, 8783,   64, 7886, 8398, 8783,   64, 7753, 7882, 8139, 8523
     c, 8778, 8904, 8901, 8771, 8514, 8130, 7875, 7748, 7749, 7877
     c, 7876,   64, 8523, 8650, 8776, 8773, 8643, 8514,   64, 7616
     c, 9024,   64, 8764, 8763, 8891, 8892, 8765, 8510, 8254, 7997
     c, 7868, 7737, 7732, 7858, 8113, 8497, 8754, 8884, 8886, 8760
     c, 8505, 8121, 7735,   64, 8254, 7996, 7865, 7860, 7986, 8113
     c,   64, 8497, 8626, 8756, 8758, 8632, 8505,    0,  -16, 7116
     c, 7495, 8012, 8399, 8386,   64, 8270, 8258,   64, 7874, 8770
     c,   64, 7616, 9024,   64, 8380, 8369,   64, 8510, 8497,   64
     c, 8510, 7605, 9013,   64, 8113, 8753,    0,  -16, 7116, 7495
     c, 7885, 7884, 7756, 7757, 7886, 8143, 8527, 8782, 8908, 8778
     c, 8521,   64, 8527, 8654, 8780, 8650, 8521,   64, 8265, 8521
     c, 8776, 8902, 8901, 8771, 8514, 8130, 7875, 7748, 7749, 7877
     c, 7876,   64, 8521, 8648, 8774, 8773, 8643, 8514,   64, 7616
     c, 9024,   64, 8380, 8369,   64, 8510, 8497,   64, 8510, 7605
     c, 9013,   64, 8113, 8753,    0,  -16, 7116, 6732, 9146, 8888
     c, 8631, 8247, 7992, 7610, 7355, 7099, 6970, 6968, 7095, 7351
     c, 7608, 7867, 7997, 8257, 8519, 8907, 9164, 9292, 9419, 9546
     c, 9417, 9290, 9419, 9420, 9292, 9164, 8907, 8778, 8518, 8258
     c, 7996, 7736, 7608,   64, 7874, 8642,    0,  -16, 7116, 6604
     c, 7991, 7608, 7226, 6973, 6848, 6851, 6982, 7241, 7627, 8012
     c, 8524, 8907, 9161, 9414, 9539, 9536, 9405, 9146, 8760, 8375
     c, 7991,   64, 7752, 7739,   64, 7880, 7867,   64, 7496, 8392
     c, 8775, 8901, 8900, 8770, 8385, 7873,   64, 8392, 8647, 8773
     c, 8772, 8642, 8385,   64, 7483, 8123,   64, 8385, 8512, 8636
     c, 8763, 8891, 9020,   64, 8385, 8640, 8764, 8891,    0,  -16
     c, 7116, 6604, 7991, 7608, 7226, 6973, 6848, 6851, 6982, 7241
     c, 7627, 8012, 8524, 8907, 9161, 9414, 9539, 9536, 9405, 9146
     c, 8760, 8375, 7991,   64, 8647, 8776, 8772, 8647, 8392, 8008
     c, 7751, 7622, 7491, 7488, 7613, 7740, 7995, 8379, 8636, 8766
     c,   64, 8008, 7750, 7619, 7616, 7741, 7995,    0,  -16, 7116
     c, 6605, 7109, 9413,   64, 7104, 9408,   64, 7099, 9403,   64
     c, 7351, 9161,    0,  -16, 7116, 6351, 6977, 6848, 6975, 7104
     c, 6977,   64, 8257, 8128, 8255, 8384, 8257,   64, 9537, 9408
     c, 9535, 9664, 9537,    0,  -16, 7116, 6605, 7739, 7486, 7104
     c, 7490, 7749,   64, 7490, 7232, 7486,   64, 7232, 9280,   64
     c, 8763, 9022, 9408, 9026, 8773,   64, 9026, 9280, 9022,    0
     c,  -16, 7116, 7240, 8900, 8518, 8265, 8006, 7620,   64, 8006
     c, 8264, 8518,   64, 8264, 8248,   64, 8892, 8506, 8247, 7994
     c, 7612,   64, 7994, 8248, 8506,    0,  -16, 7116, 6732, 9288
     c, 8392, 7879, 7622, 7364, 7233, 7231, 7356, 7610, 7865, 8376
     c, 9272,   64, 8396, 8372,    0,  -16, 7116, 6732, 7240, 8136
     c, 8647, 8902, 9156, 9281, 9279, 9148, 8890, 8633, 8120, 7224
     c,   64, 8140, 8116,    0,  -16, 7116, 6732, 7240, 8136, 8647
     c, 8902, 9156, 9281, 9279, 9148, 8890, 8633, 8120, 7224,   64
     c, 7872, 9280,    0,  -16, 7116, 6986, 7504, 7479,   64, 8272
     c, 8247,   64, 9040, 9015,    0,  -16, 7116, 7113, 8268, 7223
     c,   64, 8268, 9271,   64, 7614, 8894,    0,  -16, 7116, 6858
     c, 7372, 7351,   64, 7372, 8524, 8907, 9034, 9160, 9158, 9028
     c, 8899, 8514,   64, 7362, 8514, 8897, 9024, 9150, 9147, 9017
     c, 8888, 8503, 7351,    0,  -16, 7116, 6987, 9287, 9161, 8907
     c, 8652, 8140, 7883, 7625, 7495, 7364, 7359, 7484, 7610, 7864
     c, 8119, 8631, 8888, 9146, 9276,    0,  -16, 7116, 6858, 7372
     c, 7351,   64, 7372, 8268, 8651, 8905, 9031, 9156, 9151, 9020
     c, 8890, 8632, 8247, 7351,    0,  -16, 7116, 6985, 7500, 7479
     c,   64, 7500, 9164,   64, 7490, 8514,   64, 7479, 9143,    0
     c,  -16, 7116, 6984, 7500, 7479,   64, 7500, 9164,   64, 7490
     c, 8514,    0,  -16, 7116, 6987, 9287, 9161, 8907, 8652, 8140
     c, 7883, 7625, 7495, 7364, 7359, 7484, 7610, 7864, 8119, 8631
     c, 8888, 9146, 9276, 9279,   64, 8639, 9279,    0,  -16, 7116
     c, 6859, 7372, 7351,   64, 9164, 9143,   64, 7362, 9154,    0
     c,  -16, 7116, 7748, 8268, 8247,    0,  -16, 7116, 7240, 8780
     c, 8764, 8633, 8504, 8247, 7991, 7736, 7609, 7484, 7486,    0
     c,  -16, 7116, 6858, 7372, 7351,   64, 9164, 7358,   64, 8003
     c, 9143,    0,  -16, 7116, 6983, 7500, 7479,   64, 7479, 9015
     c,    0,  -16, 7116, 6732, 7244, 7223,   64, 7244, 8247,   64
     c, 9292, 8247,   64, 9292, 9271,    0,  -16, 7116, 6859, 7372
     c, 7351,   64, 7372, 9143,   64, 9164, 9143,    0,  -16, 7116
     c, 6859, 8012, 7755, 7497, 7367, 7236, 7231, 7356, 7482, 7736
     c, 7991, 8503, 8760, 9018, 9148, 9279, 9284, 9159, 9033, 8779
     c, 8524, 8012,    0,  -16, 7116, 6858, 7372, 7351,   64, 7372
     c, 8524, 8907, 9034, 9160, 9157, 9027, 8898, 8513, 7361,    0
     c,  -16, 7116, 6859, 8012, 7755, 7497, 7367, 7236, 7231, 7356
     c, 7482, 7736, 7991, 8503, 8760, 9018, 9148, 9279, 9284, 9159
     c, 9033, 8779, 8524, 8012,   64, 8379, 9141,    0,  -16, 7116
     c, 6858, 7372, 7351,   64, 7372, 8524, 8907, 9034, 9160, 9158
     c, 9028, 8899, 8514, 7362,   64, 8258, 9143,    0,  -16, 7116
     c, 6986, 9161, 8907, 8524, 8012, 7627, 7369, 7367, 7493, 7620
     c, 7875, 8641, 8896, 9023, 9149, 9146, 8888, 8503, 7991, 7608
     c, 7354,    0,  -16, 7116, 7240, 8268, 8247,   64, 7372, 9164
     c,    0,  -16, 7116, 6859, 7372, 7357, 7482, 7736, 8119, 8375
     c, 8760, 9018, 9149, 9164,    0,  -16, 7116, 7113, 7244, 8247
     c,   64, 9292, 8247,    0,  -16, 7116, 6732, 6988, 7607,   64
     c, 8268, 7607,   64, 8268, 8887,   64, 9548, 8887,    0,  -16
     c, 7116, 6986, 7372, 9143,   64, 9164, 7351,    0,  -16, 7116
     c, 7113, 7244, 8258, 8247,   64, 9292, 8258,    0,  -16, 7116
     c, 6986, 9164, 7351,   64, 7372, 9164,   64, 7351, 9143,    0
     c,  -16, 7116, 7113, 8268, 7223,   64, 8268, 9271,   64, 7614
     c, 8894,    0,  -16, 7116, 6858, 7372, 7351,   64, 7372, 8524
     c, 8907, 9034, 9160, 9158, 9028, 8899, 8514,   64, 7362, 8514
     c, 8897, 9024, 9150, 9147, 9017, 8888, 8503, 7351,    0,  -16
     c, 7116, 6983, 7500, 7479,   64, 7500, 9036,    0,  -16, 7116
     c, 7113, 8268, 7223,   64, 8268, 9271,   64, 7223, 9271,    0
     c,  -16, 7116, 6985, 7500, 7479,   64, 7500, 9164,   64, 7490
     c, 8514,   64, 7479, 9143,    0,  -16, 7116, 6986, 9164, 7351
     c,   64, 7372, 9164,   64, 7351, 9143,    0,  -16, 7116, 6859
     c, 7372, 7351,   64, 9164, 9143,   64, 7362, 9154,    0,  -16
     c, 7116, 6859, 8012, 7755, 7497, 7367, 7236, 7231, 7356, 7482
     c, 7736, 7991, 8503, 8760, 9018, 9148, 9279, 9284, 9159, 9033
     c, 8779, 8524, 8012,   64, 7874, 8642,    0,  -16, 7116, 7748
     c, 8268, 8247,    0,  -16, 7116, 6858, 7372, 7351,   64, 9164
     c, 7358,   64, 8003, 9143,    0,  -16, 7116, 7113, 8268, 7223
     c,   64, 8268, 9271,    0,  -16, 7116, 6732, 7244, 7223,   64
     c, 7244, 8247,   64, 9292, 8247,   64, 9292, 9271,    0,  -16
     c, 7116, 6859, 7372, 7351,   64, 7372, 9143,   64, 9164, 9143
     c,    0,  -16, 7116, 7113, 7372, 9164,   64, 7874, 8642,   64
     c, 7351, 9143,    0,  -16, 7116, 6859, 8012, 7755, 7497, 7367
     c, 7236, 7231, 7356, 7482, 7736, 7991, 8503, 8760, 9018, 9148
     c, 9279, 9284, 9159, 9033, 8779, 8524, 8012,    0,  -16, 7116
     c, 6859, 7372, 7351,   64, 9164, 9143,   64, 7372, 9164,    0
     c,  -16, 7116, 6858, 7372, 7351,   64, 7372, 8524, 8907, 9034
     c, 9160, 9157, 9027, 8898, 8513, 7361,    0,  -16, 7116, 7113
     c, 7372, 8258, 7351,   64, 7372, 9164,   64, 7351, 9143,    0
     c,  -16, 7116, 7240, 8268, 8247,   64, 7372, 9164,    0,  -16
     c, 7116, 7113, 7367, 7369, 7499, 7628, 7884, 8011, 8137, 8261
     c, 8247,   64, 9159, 9161, 9035, 8908, 8652, 8523, 8393, 8261
     c,    0,  -16, 7116, 6986, 8268, 8247,   64, 8007, 7622, 7493
     c, 7363, 7360, 7486, 7613, 7996, 8508, 8893, 9022, 9152, 9155
     c, 9029, 8902, 8519, 8007,    0,  -16, 7116, 6986, 7372, 9143
     c,   64, 7351, 9164,    0,  -16, 7116, 6859, 8268, 8247,   64
     c, 7110, 7238, 7365, 7489, 7615, 7742, 8125, 8381, 8766, 8895
     c, 9025, 9157, 9286, 9414,    0,  -16, 7116, 6986, 7351, 7863
     c, 7486, 7362, 7366, 7497, 7755, 8140, 8396, 8779, 9033, 9158
     c, 9154, 9022, 8631, 9143,    0,  -16, 7116, 6857, 6839, 7096
     c, 7483, 7871, 8390, 8780, 8759, 8634, 8381, 8127, 7745, 7489
     c, 7360, 7358, 7484, 7738, 8120, 8503, 9143,    0,  -16, 7116
     c, 6731, 8394, 8521, 8518, 8386, 8255, 8125, 7866, 7608, 7351
     c, 7223, 7096, 7099, 7232, 7363, 7493, 7752, 8010, 8267, 8652
     c, 9036, 9291, 9417, 9415, 9285, 9156, 8899, 8514,   64, 8386
     c, 8514, 8897, 9024, 9150, 9147, 9017, 8888, 8631, 8247, 7992
     c, 7866,    0,  -16, 7116, 6986, 8518, 8517, 8644, 8900, 9157
     c, 9287, 9289, 9163, 8908, 8524, 8139, 7881, 7622, 7492, 7360
     c, 7356, 7481, 7608, 7863, 8119, 8504, 8762, 8892,    0,  -16
     c, 7116, 6860, 8524, 8267, 8137, 8005, 7871, 7740, 7610, 7352
     c, 7095, 6839, 6712, 6714, 6843, 7099, 7354, 7608, 7991, 8375
     c, 8760, 9018, 9278, 9411, 9415, 9290, 9163, 8908, 8524, 8266
     c, 8264, 8389, 8642, 8896, 9278, 9533,    0,  -16, 7116, 6986
     c, 8776, 8775, 8902, 9158, 9287, 9289, 9163, 8780, 8268, 7883
     c, 7753, 7750, 7876, 8003, 8386, 8002, 7617, 7488, 7358, 7355
     c, 7481, 7608, 7991, 8375, 8760, 9018, 9148,    0,  -16, 7116
     c, 6986, 8262, 8006, 7751, 7625, 7755, 8140, 8524, 9035, 9419
     c, 9676,   64, 9035, 8772, 8510, 8250, 7992, 7735, 7479, 7224
     c, 7098, 7100, 7229, 7485, 7740,   64, 8130, 9282,    0,  -16
     c, 7116, 6860, 6839, 7096, 7612, 8001, 8132, 8264, 8267, 8140
     c, 8012, 7883, 7753, 7750, 7876, 8131, 8643, 9028, 9157, 9287
     c, 9281, 9148, 9018, 8760, 8375, 7863, 7480, 7226, 7100, 7102
     c,    0,  -16, 7116, 6732, 7621, 7366, 7240, 7241, 7371, 7628
     c, 7756, 8011, 8137, 8135, 8003, 7741, 7481, 7223, 6967, 6840
     c, 6842,   64, 7616, 8771, 9028, 9414, 9672, 9802, 9803, 9676
     c, 9548, 9290, 9030, 8768, 8635, 8632, 8759, 8887, 9144, 9273
     c, 9532,    0,  -16, 7116, 7112, 8892, 8638, 8385, 8259, 8134
     c, 8137, 8267, 8396, 8652, 8779, 8905, 8902, 8769, 8508, 8378
     c, 8120, 7863, 7607, 7352, 7226, 7228, 7357, 7613, 7868,    0
     c,  -16, 7116, 7239, 8500, 8247, 7996, 7874, 7880, 8011, 8268
     c, 8524, 8651, 8776, 8773, 8640, 8247, 7985, 7854, 7724, 7467
     c, 7340, 7342, 7473, 7732, 7990, 8376, 8890,    0,  -16, 7116
     c, 6732, 7621, 7366, 7240, 7241, 7371, 7628, 7756, 8011, 8137
     c, 8135, 8003, 7741, 7481, 7223, 6967, 6840, 6842,   64, 9801
     c, 9803, 9676, 9548, 9291, 9033, 8774, 8516, 8259, 8003,   64
     c, 8259, 8385, 8378, 8504, 8631, 8759, 9016, 9145, 9404,    0
     c,  -16, 7116, 7114, 7616, 7872, 8385, 8771, 9029, 9159, 9162
     c, 9036, 8780, 8651, 8521, 8388, 8255, 8124, 7994, 7736, 7479
     c, 7223, 7096, 7098, 7227, 7483, 7738, 8120, 8503, 8759, 9144
     c, 9402,    0,  -16, 7116, 5967, 6597, 6342, 6216, 6217, 6347
     c, 6604, 6732, 6987, 7113, 7111, 6978, 6846, 6583,   64, 6846
     c, 7238, 7498, 7627, 7884, 8012, 8267, 8393, 8391, 8258, 8126
     c, 7863,   64, 8126, 8518, 8778, 8907, 9164, 9292, 9547, 9673
     c, 9671, 9538, 9275, 9272, 9399, 9527, 9784, 9913,10172,    0
     c,  -16, 7116, 6603, 7237, 6982, 6856, 6857, 6987, 7244, 7372
     c, 7627, 7753, 7751, 7618, 7486, 7223,   64, 7486, 7878, 8138
     c, 8267, 8524, 8780, 9035, 9161, 9159, 9026, 8763, 8760, 8887
     c, 9015, 9272, 9401, 9660,    0,  -16, 7116, 6987, 8524, 8139
     c, 7881, 7622, 7492, 7360, 7356, 7481, 7608, 7863, 8119, 8504
     c, 8762, 9021, 9151, 9283, 9287, 9162, 9035, 8780, 8524, 8266
     c, 8263, 8388, 8641, 8895, 9277, 9532,    0,  -16, 7116, 6733
     c, 8394, 8521, 8518, 8386, 8255, 8125, 7866, 7608, 7351, 7223
     c, 7096, 7099, 7232, 7363, 7493, 7752, 8010, 8267, 8652, 9292
     c, 9547, 9674, 9800, 9797, 9667, 9538, 9281, 8897, 8642, 8515
     c,    0,  -16, 7116, 6988, 8646, 8516, 8387, 8130, 7874, 7748
     c, 7750, 7881, 8139, 8524, 8908, 9163, 9289, 9285, 9154, 8895
     c, 8379, 7993, 7736, 7351, 7095, 6968, 6970, 7099, 7355, 7610
     c, 7992, 8375, 8759, 9144, 9402,    0,  -16, 7116, 6733, 8394
     c, 8521, 8518, 8386, 8255, 8125, 7866, 7608, 7351, 7223, 7096
     c, 7099, 7232, 7363, 7493, 7752, 8010, 8267, 8652, 9164, 9419
     c, 9546, 9672, 9669, 9539, 9410, 9153, 8769, 8386, 8513, 8639
     c, 8634, 8760, 9015, 9272, 9401, 9660,    0,  -16, 7116, 6986
     c, 6967, 7224, 7482, 7870, 8129, 8389, 8520, 8523, 8396, 8268
     c, 8139, 8009, 8007, 8133, 8387, 8769, 9023, 9149, 9147, 9017
     c, 8888, 8503, 7991, 7608, 7354, 7228, 7230,    0,  -16, 7116
     c, 6985, 8262, 8006, 7751, 7625, 7755, 8140, 8524, 9035, 9419
     c, 9676,   64, 9035, 8772, 8510, 8250, 7992, 7735, 7479, 7224
     c, 7098, 7100, 7229, 7485, 7740,    0,  -16, 7116, 6603, 7237
     c, 6982, 6856, 6857, 6987, 7244, 7372, 7627, 7753, 7751, 7619
     c, 7488, 7356, 7354, 7480, 7735, 7991, 8248, 8377, 8637, 9029
     c, 9292,   64, 9029, 8897, 8763, 8760, 8887, 9015, 9272, 9401
     c, 9660,    0,  -16, 7116, 6731, 7365, 7110, 6984, 6985, 7115
     c, 7372, 7500, 7755, 7881, 7879, 7747, 7616, 7484, 7481, 7607
     c, 7863, 8120, 8507, 8766, 9026, 9157, 9289, 9291, 9164, 9036
     c, 8907, 8777, 8775, 8900, 9154, 9409,    0,  -16, 7116, 6349
     c, 6981, 6726, 6600, 6601, 6731, 6988, 7116, 7371, 7497, 7494
     c, 7351,   64, 8652, 7351,   64, 8652, 8375,   64,10188, 9931
     c, 9544, 9156, 8766, 8375,    0,  -16, 7116, 6732, 7750, 7494
     c, 7367, 7369, 7499, 7756, 8012, 8267, 8393, 8390, 8125, 8122
     c, 8248, 8503, 8759, 9016, 9146, 9148, 9021, 8765,   64, 9673
     c, 9675, 9548, 9292, 9035, 8777, 8518, 7997, 7738, 7480, 7223
     c, 6967, 6840, 6842,    0,  -16, 7116, 6731, 7365, 7110, 6984
     c, 6985, 7115, 7372, 7500, 7755, 7881, 7879, 7747, 7616, 7484
     c, 7482, 7608, 7735, 7991, 8248, 8506, 8765, 8895, 9157,   64
     c, 9420, 9157, 8763, 8501, 8240, 7980, 7723, 7596, 7598, 7729
     c, 7988, 8375, 8761, 9404,    0,  -16, 7116, 6987, 8646, 8516
     c, 8387, 8130, 7874, 7748, 7750, 7881, 8139, 8524, 8908, 9163
     c, 9289, 9285, 9154, 8894, 8507, 7992, 7735, 7351, 7224, 7226
     c, 7355, 7739, 7994, 8121, 8247, 8244, 8113, 7983, 7724, 7467
     c, 7340, 7342, 7473, 7732, 8119, 8505, 9276,    0,  -16, 7116
     c, 7113, 7244, 8247,   64, 9292, 8247,   64, 7244, 9292,    0
     c,  -16, 7116, 7114, 7093, 9397,    0,  -16, 7116, 7114, 9029
     c, 9015,   64, 9026, 8772, 8517, 8133, 7876, 7618, 7487, 7485
     c, 7610, 7864, 8119, 8503, 8760, 9018,    0,  -16, 7116, 6985
     c, 7500, 7479,   64, 7490, 7748, 8005, 8389, 8644, 8898, 9023
     c, 9021, 8890, 8632, 8375, 7991, 7736, 7482,    0,  -16, 7116
     c, 7113, 9026, 8772, 8517, 8133, 7876, 7618, 7487, 7485, 7610
     c, 7864, 8119, 8503, 8760, 9018,    0,  -16, 7116, 7114, 9036
     c, 9015,   64, 9026, 8772, 8517, 8133, 7876, 7618, 7487, 7485
     c, 7610, 7864, 8119, 8503, 8760, 9018,    0,  -16, 7116, 7113
     c, 7487, 9023, 9025, 8899, 8772, 8517, 8133, 7876, 7618, 7487
     c, 7485, 7610, 7864, 8119, 8503, 8760, 9018,    0,  -16, 7116
     c, 7623, 8908, 8652, 8395, 8264, 8247,   64, 7877, 8773,    0
     c,  -16, 7116, 7114, 9029, 9013, 8882, 8753, 8496, 8112, 7857
     c,   64, 9026, 8772, 8517, 8133, 7876, 7618, 7487, 7485, 7610
     c, 7864, 8119, 8503, 8760, 9018,    0,  -16, 7116, 7114, 7628
     c, 7607,   64, 7617, 8004, 8261, 8645, 8900, 9025, 9015,    0
     c,  -16, 7116, 7748, 8140, 8267, 8396, 8269, 8140,   64, 8261
     c, 8247,    0,  -16, 7116, 7621, 8268, 8395, 8524, 8397, 8268
     c,   64, 8389, 8372, 8241, 7984, 7728,    0,  -16, 7116, 7112
     c, 7628, 7607,   64, 8901, 7611,   64, 8127, 9015,    0,  -16
     c, 7116, 7748, 8268, 8247,    0,  -16, 7116, 6351, 6853, 6839
     c,   64, 6849, 7236, 7493, 7877, 8132, 8257, 8247,   64, 8257
     c, 8644, 8901, 9285, 9540, 9665, 9655,    0,  -16, 7116, 7114
     c, 7621, 7607,   64, 7617, 8004, 8261, 8645, 8900, 9025, 9015
     c,    0,  -16, 7116, 7114, 8133, 7876, 7618, 7487, 7485, 7610
     c, 7864, 8119, 8503, 8760, 9018, 9149, 9151, 9026, 8772, 8517
     c, 8133,    0,  -16, 7116, 6985, 7493, 7472,   64, 7490, 7748
     c, 8005, 8389, 8644, 8898, 9023, 9021, 8890, 8632, 8375, 7991
     c, 7736, 7482,    0,  -16, 7116, 7114, 9029, 9008,   64, 9026
     c, 8772, 8517, 8133, 7876, 7618, 7487, 7485, 7610, 7864, 8119
     c, 8503, 8760, 9018,    0,  -16, 7116, 7366, 7877, 7863,   64
     c, 7871, 8002, 8260, 8517, 8901,    0,  -16, 7116, 7241, 9026
     c, 8900, 8517, 8133, 7748, 7618, 7744, 7999, 8638, 8893, 9019
     c, 9018, 8888, 8503, 8119, 7736, 7610,    0,  -16, 7116, 7623
     c, 8268, 8251, 8376, 8631, 8887,   64, 7877, 8773,    0,  -16
     c, 7116, 7114, 7621, 7611, 7736, 7991, 8375, 8632, 9019,   64
     c, 9029, 9015,    0,  -16, 7116, 7240, 7493, 8247,   64, 9029
     c, 8247,    0,  -16, 7116, 6859, 7237, 7735,   64, 8261, 7735
     c,   64, 8261, 8759,   64, 9285, 8759,    0,  -16, 7116, 7241
     c, 7621, 9015,   64, 9029, 7607,    0,  -16, 7116, 7240, 7493
     c, 8247,   64, 9029, 8247, 7987, 7729, 7472, 7344,    0,  -16
     c, 7116, 7241, 9029, 7607,   64, 7621, 9029,   64, 7607, 9015
     c,    0,  -16, 7116, 6987, 8133, 7876, 7618, 7488, 7357, 7354
     c, 7480, 7735, 7991, 8248, 8635, 8894, 9154, 9285,   64, 8133
     c, 8389, 8516, 8642, 8890, 9016, 9143, 9271,    0,  -16, 7116
     c, 7114, 8652, 8395, 8137, 7877, 7746, 7614, 7480, 7344,   64
     c, 8652, 8908, 9162, 9159, 9029, 8900, 8643, 8259,   64, 8259
     c, 8514, 8768, 8894, 8891, 8761, 8632, 8375, 8119, 7864, 7737
     c, 7612,    0,  -16, 7116, 7114, 7234, 7492, 7749, 7877, 8132
     c, 8259, 8384, 8380, 8247,   64, 9285, 9154, 9024, 8247, 7987
     c, 7856,    0,  -16, 7116, 7113, 8517, 8133, 7876, 7618, 7487
     c, 7484, 7609, 7736, 7991, 8247, 8504, 8762, 8893, 8896, 8771
     c, 8517, 8263, 8137, 8139, 8268, 8524, 8779, 9033,    0,  -16
     c, 7116, 7240, 8899, 8772, 8517, 8133, 7876, 7874, 8000, 8383
     c,   64, 8383, 7870, 7612, 7610, 7736, 7991, 8375, 8632, 8890
     c,    0,  -16, 7116, 7239, 8524, 8267, 8138, 8137, 8264, 8647
     c, 9031,   64, 9031, 8517, 8131, 7744, 7613, 7611, 7737, 7991
     c, 8373, 8499, 8497, 8368, 8112, 7986,    0,  -16, 7116, 6986
     c, 7105, 7235, 7493, 7749, 7876, 7874, 7742, 7479,   64, 7742
     c, 8002, 8260, 8517, 8773, 9027, 9024, 8891, 8496,    0,  -16
     c, 7116, 6858, 6977, 7107, 7365, 7621, 7748, 7746, 7613, 7610
     c, 7736, 7863, 8119, 8376, 8635, 8765, 8896, 9029, 9032, 8907
     c, 8652, 8396, 8266, 8264, 8389, 8642, 8896, 9278,    0,  -16
     c, 7116, 7493, 8261, 7998, 7866, 7864, 7991, 8247, 8505, 8635
     c,    0,  -16, 7116, 7113, 7877, 7351,   64, 9156, 9029, 8901
     c, 8644, 8128, 7871, 7743,   64, 7743, 7998, 8125, 8376, 8503
     c, 8631, 8760,    0,  -16, 7116, 7240, 7372, 7628, 7883, 8010
     c, 9015,   64, 8261, 7479,    0,  -16, 7116, 6987, 7877, 7088
     c,   64, 7745, 7612, 7609, 7863, 8119, 8376, 8634, 8894,   64
     c, 9157, 8894, 8762, 8760, 8887, 9143, 9401, 9531,    0,  -16
     c, 7116, 7113, 7493, 7877, 7743, 7610, 7479,   64, 9157, 9026
     c, 8896, 8637, 8250, 7864, 7479,    0,  -16, 7116, 7240, 8524
     c, 8267, 8138, 8137, 8264, 8647, 9031,   64, 8647, 8262, 8005
     c, 7875, 7873, 8127, 8510, 8766,   64, 8510, 7997, 7740, 7610
     c, 7608, 7862, 8372, 8499, 8497, 8240, 7984,    0,  -16, 7116
     c, 7241, 8261, 8004, 7746, 7615, 7612, 7737, 7864, 8119, 8375
     c, 8632, 8890, 9021, 9024, 8899, 8772, 8517, 8261,    0,  -16
     c, 7116, 6859, 8005, 7479,   64, 8645, 8767, 8890, 9015,   64
     c, 7106, 7364, 7749, 9413,    0,  -16, 7116, 7113, 7615, 7612
     c, 7737, 7864, 8119, 8375, 8632, 8890, 9021, 9024, 8899, 8772
     c, 8517, 8261, 8004, 7746, 7615, 7088,    0,  -16, 7116, 7115
     c, 9413, 8133, 7876, 7618, 7487, 7484, 7609, 7736, 7991, 8247
     c, 8504, 8762, 8893, 8896, 8771, 8644, 8389,    0,  -16, 7116
     c, 6986, 8389, 7991,   64, 7234, 7492, 7877, 9285,    0,  -16
     c, 7116, 6986, 7105, 7235, 7493, 7749, 7876, 7874, 7612, 7609
     c, 7863, 8119, 8504, 8762, 9022, 9154, 9157,    0,  -16, 7116
     c, 6859, 7876, 7619, 7361, 7230, 7227, 7353, 7480, 7735, 8119
     c, 8504, 8890, 9149, 9280, 9283, 9029, 8773, 8515, 8255, 7994
     c, 7600,    0,  -16, 7116, 7113, 7365, 7621, 7875, 8626, 8880
     c, 9136,   64, 9285, 9155, 8896, 7605, 7346, 7216,    0,  -16
     c, 7116, 6731, 8780, 7728,   64, 6849, 6979, 7237, 7493, 7620
     c, 7618, 7485, 7482, 7608, 7863, 8119, 8504, 8762, 9021, 9282
     c, 9413,    0,  -16, 7116, 6731, 7749, 7492, 7233, 7102, 7099
     c, 7224, 7351, 7607, 7864, 8123,   64, 8255, 8123, 8248, 8375
     c, 8631, 8888, 9147, 9278, 9281, 9156, 9029,    0,  -16, 7116
     c, 7498, 8637, 8511, 8256, 8000, 7743, 7614, 7484, 7482, 7608
     c, 7863, 8119, 8376, 8506, 8768, 8635, 8632, 8759, 8887, 9144
     c, 9273, 9532,    0,  -16, 7116, 7625, 7612, 7871, 8260, 8390
     c, 8521, 8523, 8396, 8139, 8009, 7877, 7742, 7736, 7863, 7991
     c, 8248, 8506, 8637, 8640, 8764, 8891, 9147, 9404,    0,  -16
     c, 7116, 7622, 8510, 8511, 8384, 8128, 7871, 7742, 7612, 7610
     c, 7736, 7991, 8375, 8761, 9020,    0,  -16, 7116, 7498, 8637
     c, 8511, 8256, 8000, 7743, 7614, 7484, 7482, 7608, 7863, 8119
     c, 8376, 8506, 9292,   64, 8768, 8635, 8632, 8759, 8887, 9144
     c, 9273, 9532,    0,  -16, 7116, 7750, 7865, 8122, 8251, 8381
     c, 8383, 8256, 8128, 7871, 7741, 7738, 7864, 8119, 8375, 8632
     c, 8761, 9020,    0,  -16, 7116, 7877, 7868, 8385, 8644, 8774
     c, 8905, 8907, 8780, 8523, 8393, 8129, 7736, 7345, 7214, 7212
     c, 7339, 7596, 7727, 7864, 7991, 8247, 8504, 8633, 8892,    0
     c,  -16, 7116, 7497, 8637, 8511, 8256, 8000, 7743, 7614, 7484
     c, 7482, 7608, 7863, 8119, 8376, 8505,   64, 8768, 8505, 7982
     c, 7852, 7595, 7468, 7470, 7601, 7988, 8374, 8631, 9017, 9404
     c,    0,  -16, 7116, 7626, 7612, 7871, 8260, 8390, 8521, 8523
     c, 8396, 8139, 8009, 7877, 7743, 7607,   64, 7607, 7738, 7868
     c, 8127, 8384, 8640, 8767, 8765, 8634, 8632, 8759, 8887, 9144
     c, 9273, 9532,    0,  -16, 7116, 8005, 8389, 8388, 8516, 8517
     c, 8389,   64, 7996, 8256, 7994, 7992, 8119, 8247, 8504, 8633
     c, 8892,    0,  -16, 7116, 8005, 8389, 8388, 8516, 8517, 8389
     c,   64, 7996, 8256, 7470, 7340, 7083, 6956, 6958, 7089, 7476
     c, 7862, 8119, 8505, 8892,    0,  -16, 7116, 7625, 7612, 7871
     c, 8260, 8390, 8521, 8523, 8396, 8139, 8009, 7877, 7743, 7607
     c,   64, 7607, 7738, 7868, 8127, 8384, 8640, 8767, 8765, 8508
     c, 8124,   64, 8124, 8379, 8504, 8631, 8759, 9016, 9145, 9404
     c,    0,  -16, 7116, 7877, 7868, 8127, 8516, 8646, 8777, 8779
     c, 8652, 8395, 8265, 8133, 7998, 7992, 8119, 8247, 8504, 8633
     c, 8892,    0,  -16, 7116, 6604, 6588, 6847, 7104, 7231, 7230
     c, 7098, 6967,   64, 7098, 7228, 7487, 7744, 8000, 8127, 8126
     c, 7994, 7863,   64, 7994, 8124, 8383, 8640, 8896, 9023, 9021
     c, 8890, 8888, 9015, 9143, 9400, 9529, 9788,    0,  -16, 7116
     c, 7242, 7228, 7487, 7744, 7871, 7870, 7738, 7607,   64, 7738
     c, 7868, 8127, 8384, 8640, 8767, 8765, 8634, 8632, 8759, 8887
     c, 9144, 9273, 9532,    0,  -16, 7116, 7496, 8256, 8000, 7743
     c, 7614, 7484, 7482, 7608, 7863, 8119, 8376, 8505, 8635, 8637
     c, 8511, 8256, 8127, 8125, 8251, 8506, 8890, 9147, 9276,    0
     c,  -16, 7116, 7368, 7356, 7615, 7745, 7613, 6827,   64, 7613
     c, 7743, 8000, 8256, 8511, 8637, 8635, 8505, 8376, 8119,   64
     c, 7608, 7863, 8247, 8632, 8889, 9276,    0,  -16, 7116, 7497
     c, 8637, 8511, 8256, 8000, 7743, 7614, 7484, 7482, 7608, 7863
     c, 8119, 8376,   64, 8768, 8637, 8376, 7985, 7854, 7852, 7979
     c, 8236, 8367, 8374, 8631, 9017, 9404,    0,  -16, 7116, 7624
     c, 7612, 7871, 8001, 7999, 8383, 8510, 8508, 8377, 8376, 8503
     c, 8631, 8888, 9017, 9276,    0,  -16, 7116, 7751, 7740, 7999
     c, 8129, 8127, 8380, 8506, 8504, 8247,   64, 7736, 7991, 8503
     c, 8760, 8889, 9148,    0,  -16, 7116, 7878, 7868, 8127, 8387
     c,   64, 8780, 7994, 7992, 8119, 8375, 8632, 8761, 9020,   64
     c, 8004, 8900,    0,  -16, 7116, 7497, 7484, 7744, 7482, 7480
     c, 7607, 7863, 8120, 8378, 8637,   64, 8768, 8506, 8504, 8631
     c, 8759, 9016, 9145, 9404,    0,  -16, 7116, 7497, 7484, 7744
     c, 7611, 7608, 7735, 7863, 8248, 8506, 8637, 8640,   64, 8640
     c, 8764, 8891, 9147, 9404,    0,  -16, 7116, 7116, 7488, 7230
     c, 7099, 7097, 7223, 7479, 7736, 7994,   64, 8256, 7994, 7992
     c, 8119, 8375, 8632, 8890, 9021, 9024,   64, 9024, 9148, 9275
     c, 9531, 9788,    0,  -16, 7116, 7240, 7228, 7487, 7744, 8000
     c, 8127, 8120, 8247, 8631, 9017, 9276,   64, 8895, 8768, 8512
     c, 8383, 7864, 7735, 7479, 7352,    0,  -16, 7116, 7497, 7484
     c, 7744, 7482, 7480, 7607, 7863, 8120, 8378, 8637,   64, 8768
     c, 7982, 7852, 7595, 7468, 7470, 7601, 7988, 8374, 8631, 9017
     c, 9404,    0,  -16, 7116, 7496, 7484, 7743, 8000, 8256, 8510
     c, 8508, 8378, 8120, 7735, 7990, 8116, 8113, 7982, 7852, 7595
     c, 7468, 7470, 7601, 7988, 8374, 8889, 9276,    0,  -16, 7116
     c, 7242, 7351, 7736, 8122, 8380, 8639, 8899, 9031, 9033, 8907
     c, 8652, 8395, 8266, 8136, 8003, 7998, 8122, 8248, 8503, 8759
     c, 9016, 9145,    0,  -16, 7116, 7113, 8893, 8896, 8771, 8644
     c, 8389, 8133, 7876, 7618, 7487, 7484, 7609, 7736, 7991, 8247
     c, 8504, 8762, 8893, 9026, 9031, 8906, 8779, 8524, 8268, 8011
     c, 7753,    0,  -16, 7116, 7240, 8900, 8645, 8261, 8004, 7746
     c, 7615, 7612, 7737, 7864, 8119, 8503, 8760,   64, 7614, 8638
     c,    0,  -16, 7116, 7241, 8524, 8267, 8008, 7878, 7747, 7614
     c, 7610, 7736, 7863, 8119, 8376, 8635, 8765, 8896, 9029, 9033
     c, 8907, 8780, 8524,   64, 7746, 8898,    0,  -16, 7116, 6986
     c, 8780, 7728,   64, 8133, 7748, 7490, 7359, 7356, 7482, 7736
     c, 8119, 8375, 8760, 9018, 9149, 9152, 9026, 8772, 8389, 8133
     c,    0,  -16, 7116, 7113, 9027, 8900, 8517, 8133, 7748, 7619
     c, 7489, 7487, 7613, 7867, 8376, 8502, 8500, 8371, 8115,    0
     c,  -16, 7116, 8256,   64,    0,  -16, 7116, 7748,   64,    0
     c,  -16, 7116, 7240,   64,    0,  -16, 7116, 6986, 8140, 7755
     c, 7496, 7363, 7360, 7483, 7736, 8119, 8375, 8760, 9019, 9152
     c, 9155, 9032, 8779, 8396, 8140,    0,  -16, 7116, 6986, 7752
     c, 8009, 8396, 8375,    0,  -16, 7116, 6986, 7495, 7496, 7626
     c, 7755, 8012, 8524, 8779, 8906, 9032, 9030, 8900, 8641, 7351
     c, 9143,    0,  -16, 7116, 6986, 7628, 9036, 8260, 8644, 8899
     c, 9026, 9151, 9149, 9018, 8760, 8375, 7991, 7608, 7481, 7355
     c,    0,  -16, 7116, 6986, 8652, 7358, 9278,   64, 8652, 8631
     c,    0,  -16, 7116, 6986, 8908, 7628, 7491, 7620, 8005, 8389
     c, 8772, 9026, 9151, 9149, 9018, 8760, 8375, 7991, 7608, 7481
     c, 7355,    0,  -16, 7116, 6986, 9033, 8907, 8524, 8268, 7883
     c, 7624, 7491, 7486, 7610, 7864, 8247, 8375, 8760, 9018, 9149
     c, 9150, 9025, 8771, 8388, 8260, 7875, 7617, 7486,    0,  -16
     c, 7116, 6986, 9164, 7863,   64, 7372, 9164,    0,  -16, 7116
     c, 6986, 8012, 7627, 7497, 7495, 7621, 7876, 8387, 8770, 9024
     c, 9150, 9147, 9017, 8888, 8503, 7991, 7608, 7481, 7355, 7358
     c, 7488, 7746, 8131, 8644, 8901, 9031, 9033, 8907, 8524, 8012
     c,    0,  -16, 7116, 6986, 9029, 8898, 8640, 8255, 8127, 7744
     c, 7490, 7365, 7366, 7497, 7755, 8140, 8268, 8651, 8905, 9029
     c, 9024, 8891, 8632, 8247, 7991, 7608, 7482,    0,  -16, 7116
     c, 7621, 8249, 8120, 8247, 8376, 8249,    0,  -16, 7116, 7621
     c, 8376, 8247, 8120, 8249, 8376, 8374, 8244, 8115,    0,  -16
     c, 7116, 7621, 8261, 8132, 8259, 8388, 8261,   64, 8249, 8120
     c, 8247, 8376, 8249,    0,  -16, 7116, 7621, 8261, 8132, 8259
     c, 8388, 8261,   64, 8376, 8247, 8120, 8249, 8376, 8374, 8244
     c, 8115,    0,  -16, 7116, 7621, 8268, 8254,   64, 8249, 8120
     c, 8247, 8376, 8249,    0,  -16, 7116, 7113, 7495, 7496, 7626
     c, 7755, 8012, 8524, 8779, 8906, 9032, 9030, 8900, 8771, 8257
     c, 8254,   64, 8249, 8120, 8247, 8376, 8249,    0,  -16, 7116
     c, 7748, 8268, 8261,    0,  -16, 7116, 7240, 7756, 7749,   64
     c, 8780, 8773,    0,  -16, 7116, 7367, 8140, 7883, 7753, 7751
     c, 7877, 8132, 8388, 8645, 8775, 8777, 8651, 8396, 8140,    0
     c,  -16, 7116, 6986, 8016, 7987,   64, 8528, 8499,   64, 9161
     c, 8907, 8524, 8012, 7627, 7369, 7367, 7493, 7620, 7875, 8641
     c, 8896, 9023, 9149, 9146, 8888, 8503, 7991, 7608, 7354,    0
     c,  -16, 7116, 6859, 9424, 7088,    0,  -16, 7116, 7367, 8784
     c, 8526, 8267, 8007, 7874, 7870, 7993, 8245, 8498, 8752,    0
     c,  -16, 7116, 7367, 7760, 8014, 8267, 8519, 8642, 8638, 8505
     c, 8245, 7986, 7728,    0,  -16, 7116, 7748, 8272, 8240,    0
     c,  -16, 7116, 6605, 7104, 9408,    0,  -16, 7116, 6605, 8265
     c, 8247,   64, 7104, 9408,    0,  -16, 7116, 6605, 7107, 9411
     c,   64, 7101, 9405,    0,  -16, 7116, 6859, 7367, 9145,   64
     c, 9159, 7353,    0,  -16, 7116, 7240, 8262, 8250,   64, 7619
     c, 8893,   64, 8899, 7613,    0,  -16, 7116, 7621, 8257, 8128
     c, 8255, 8384, 8257,    0,  -16, 7116, 7621, 8396, 8267, 8137
     c, 8135, 8262, 8391, 8264,    0,  -16, 7116, 7621, 8266, 8139
     c, 8268, 8395, 8393, 8263, 8134,    0,  -16, 7116, 6605, 8645
     c, 9026, 9408, 9022, 8635,   64, 7104, 9408,    0,  -16, 7116
     c, 6987, 8400, 7472,   64, 9168, 8240,   64, 7491, 9283,   64
     c, 7357, 9149,    0,  -16, 7116, 6605, 9539, 9540, 9413, 9285
     c, 9156, 9026, 8765, 8506, 8248, 7991, 7479, 7224, 7097, 6971
     c, 6973, 7103, 7232, 8132, 8261, 8391, 8393, 8267, 8012, 7755
     c, 7625, 7623, 7748, 8001, 8634, 8888, 9143, 9399, 9528, 9529
     c,    0,  -16, 7116, 6859, 7113, 7239, 7363, 7357, 7225, 7095
     c,   64, 9417, 9287, 9155, 9149, 9273, 9399,   64, 7113, 7368
     c, 7879, 8647, 9160, 9417,   64, 7095, 7352, 7865, 8633, 9144
     c, 9399,    0,  -16, 7116, 7367, 7888, 7863,   64, 8656, 8631
     c,    0,  -16, 7116, 6732, 8272, 8247,   64, 7095, 9399,    0
     c,  -16, 7116, 6732, 9424, 7095, 9399,    0,  -16, 7116, 6605
     c, 8265, 8136, 8263, 8392, 8265,   64, 7097, 6968, 7095, 7224
     c, 7097,   64, 9401, 9272, 9399, 9528, 9401,    0,  -16, 7116
     c, 6732, 8266, 7750, 7362, 7231, 7229, 7355, 7610, 7866, 8123
     c, 8253,   64, 8266, 8774, 9154, 9279, 9277, 9147, 8890, 8634
     c, 8379, 8253,   64, 8253, 8121, 7990,   64, 8253, 8377, 8502
     c,   64, 7990, 8502,    0,  -16, 7116, 6732, 8260, 8135, 8009
     c, 7754, 7626, 7369, 7239, 7235, 7360, 7486, 7739, 8246,   64
     c, 8260, 8391, 8521, 8778, 8906, 9161, 9287, 9283, 9152, 9022
     c, 8763, 8246,    0,  -16, 7116, 6732, 8267, 8008, 7491, 7104
     c,   64, 8267, 8520, 9027, 9408,   64, 7104, 7485, 7992, 8245
     c,   64, 9408, 9021, 8504, 8245,    0,  -16, 7116, 6732, 8254
     c, 8507, 8762, 9018, 9275, 9405, 9407, 9281, 9026, 8770, 8385
     c,   64, 8385, 8643, 8773, 8775, 8649, 8394, 8138, 7881, 7751
     c, 7749, 7875, 8129,   64, 8129, 7746, 7490, 7233, 7103, 7101
     c, 7227, 7482, 7738, 7995, 8254,   64, 8254, 8121, 7990,   64
     c, 8254, 8377, 8502,   64, 7990, 8502,    0,  -16, 7116, 6732
     c, 8256, 8247, 8118,   64, 8252, 8118,   64, 8265, 8138, 7882
     c, 7753, 7751, 7876, 8256,   64, 8265, 8394, 8650, 8777, 8775
     c, 8644, 8256,   64, 8256, 7747, 7492, 7236, 7107, 7105, 7232
     c,   64, 8256, 8771, 9028, 9284, 9411, 9409, 9280,   64, 8256
     c, 7741, 7484, 7228, 7101, 7103, 7232,   64, 8256, 8765, 9020
     c, 9276, 9405, 9407, 9280,    0,  -16, 7116, 6732, 7103, 7231
     c, 7486, 7612, 7610, 7480,   64, 7103, 7104, 7233, 7489, 7616
     c, 7742, 7739, 7609, 7480,   64, 8267, 8009, 7878, 7875, 8125
     c, 8122, 7992, 8246,   64, 8267, 8521, 8646, 8643, 8381, 8378
     c, 8504, 8246,   64, 9407, 9408, 9281, 9025, 8896, 8766, 8763
     c, 8889, 9016,   64, 9407, 9279, 9022, 8892, 8890, 9016,   64
     c, 7356, 9148,    0,  -16, 7116, 8001, 8384, 8255, 8127, 8000
     c, 8001, 8130, 8258, 8385, 8383, 8253, 8124,   64, 8129, 8128
     c, 8256, 8257, 8129,    0,  -16, 7116, 8002, 8130, 8001, 7999
     c, 8126, 8382, 8511, 8513, 8386, 8130,   64, 8257, 8128, 8255
     c, 8384, 8257,    0,  -16, 7116, 7748, 8003, 8509,   64, 8515
     c, 7997,   64, 7744, 8768,    0,  -16, 7116, 7621, 8263, 8133
     c, 7874, 7616,   64, 8263, 8389, 8642, 8896,   64, 8261, 7873
     c,   64, 8261, 8641,   64, 8259, 8001,   64, 8259, 8513,   64
     c, 8129, 8385,   64, 7616, 8896,    0,  -16, 7116, 7621, 7616
     c, 7617, 7747, 7876, 8133, 8389, 8644, 8771, 8897, 8896,   64
     c, 8004, 8516,   64, 7875, 8643,   64, 7746, 8770,   64, 7745
     c, 8769,   64, 7616, 8896,    0,  -16, 7116, 7488, 7500, 7488
     c, 8256, 7500,   64, 7497, 8001,   64, 7494, 7872,   64, 7491
     c, 7617,    0,  -16, 7116, 7621, 8263, 8133, 7874, 7616,   64
     c, 8263, 8389, 8642, 8896,    0,  -16, 7116, 7621, 8896, 8897
     c, 8771, 8644, 8389, 8133, 7876, 7747, 7617, 7616,    0,  -16
     c, 7116, 6859, 9664, 9666, 9541, 9288, 8906, 8523, 8011, 7626
     c, 7240, 6981, 6850, 6848,    0,  -16, 7116, 7621, 7616, 7615
     c, 7741, 7868, 8123, 8379, 8636, 8765, 8895, 8896,    0,  -16
     c, 7116, 7494, 7490, 7744, 8127, 8383, 8768, 9026,    0,  -16
     c, 7116, 8259, 8253, 8510, 8640, 8514, 8259,    0,  -16, 7116
     c, 8260, 8256, 8642, 8772, 8774, 8647, 8519,    0,  -16, 7116
     c, 7744, 8256, 7874, 7748, 7750, 7879, 8007,    0,  -16, 7116
     c, 7621, 8906, 8779, 8524, 8140, 7883, 7754, 7624, 7622, 7748
     c, 7875, 8639, 8766, 8892, 8890, 8760, 8631, 8374, 7990, 7735
     c, 7608,    0,  -16, 7116, 6859, 7099, 6972, 6846, 6849, 6979
     c, 7108, 7365, 7621, 7876, 8003, 8509, 8636, 8891, 9147, 9404
     c, 9533, 9663, 9666, 9540, 9413,    0,  -16, 7116, 6859, 8256
     c, 8509, 8636, 8891, 9147, 9404, 9533, 9663, 9665, 9539, 9412
     c, 9157, 8901, 8644, 8515, 7997, 7868, 7611, 7355, 7100, 6973
     c, 6847, 6849, 6979, 7108, 7365, 7621, 7876, 8003, 8256,    0
     c,  -16, 7116, 6986, 7372, 7351,   64, 6988, 9420, 8130, 9400
     c,   64, 9276, 9401, 9527,   64, 9276, 9273,   64, 8889, 9273
     c,   64, 8889, 9272, 9527,    0,  -16, 7116, 7367, 8657, 8272
     c, 8015, 7757, 7498, 7366, 7360, 7485, 7739, 8122, 8378, 8763
     c, 9021, 9152,   64, 7362, 7493, 7751, 8136, 8392, 8775, 9029
     c, 9154, 9148, 9016, 8757, 8499, 8242, 7857,    0,  -16, 7116
     c, 5716, 5696,10816,    0,  -16, 7116, 6478, 6450,10062,    0
     c,  -16, 7116, 8256, 8276, 8236,    0,  -16, 7116, 6478, 6478
     c,10034,    0,  -16, 7116, 6478, 6464,10048,    0,  -16, 7116
     c, 6732, 6713, 9799,    0,  -16, 7116, 7367, 7348, 9164,    0
     c,  -16, 7116, 8256, 8270, 8242,    0,  -16, 7116, 7367, 7372
     c, 9140,    0,  -16, 7116, 6732, 6727, 9785,    0,  -16, 7116
     c, 7367, 7360, 9152,    0,  -16, 7116, 7621, 7611, 8901,    0
     c,  -16, 7116, 8256, 8263, 8249,    0,  -16, 7116, 7621, 7621
     c, 8891,    0,  -16, 7116, 6848, 8267, 8011, 7626, 7240, 6981
     c, 6850, 6848,    0,  -16, 7116, 6848, 6848, 6846, 6971, 7224
     c, 7606, 7989, 8245,    0,  -16, 7116, 8267, 8245, 8501, 8886
     c, 9272, 9531, 9662, 9664,    0,  -16, 7116, 8267, 9664, 9666
     c, 9541, 9288, 8906, 8523, 8267,    0,  -16, 7116, 6478, 6467
     c, 6849, 7359, 7998, 8510, 9151, 9665,10051,    0,  -16, 7116
     c, 8003, 8654, 8395, 8135, 8002, 7998, 8121, 8373, 8626,    0
     c,  -16, 7116, 7874, 7886, 8139, 8391, 8514, 8510, 8377, 8117
     c, 7858,    0,  -16, 7116, 6478, 6461, 6847, 7361, 8002, 8514
     c, 9153, 9663,10045,    0,  -16, 7116, 7367, 8264, 9156, 7356
     c, 8248,    0,  -16, 7116, 7240, 7232, 7751, 8761, 9280,    0
     c,  -16, 7116, 7367, 7356, 7364, 9148, 9156,    0,  -16, 7116
     c, 7240, 7482, 7234, 9278, 9030,    0,  -16, 7116, 7240, 7221
     c, 7477, 7862, 8119, 8506, 8636, 8767, 8771, 8646, 8520, 8393
     c, 8137, 8008, 7878, 7747, 7743, 7868, 7994, 8375, 8630, 9013
     c, 9269,    0,  -16, 7116, 7115, 9656, 9658, 9533, 9407, 9026
     c, 8771, 8388, 7876, 7491, 7234, 7105, 7103, 7230, 7485, 7868
     c, 8380, 8765, 9022, 9409, 9539, 9670, 9672,    0,  -16, 7116
     c, 7240, 9291, 9035, 8650, 8393, 8006, 7876, 7745, 7741, 7866
     c, 7992, 8119, 8375, 8504, 8634, 8765, 8769, 8644, 8518, 8137
     c, 7882, 7499, 7243,    0,  -16, 7116, 6857, 6856, 6854, 6979
     c, 7105, 7486, 7741, 8124, 8636, 9021, 9278, 9407, 9409, 9282
     c, 9027, 8644, 8132, 7747, 7490, 7103, 6973, 6842, 6840,    0
     c,  -16, 7116, 6601, 6594, 6720, 6974, 7229, 7612, 8124, 8637
     c, 9023, 9282, 9412, 9286, 8902, 8389, 8132, 7746, 7487, 7356
     c, 7353, 7478, 7604,    0,  -16, 7116, 6599, 6590, 6972, 7355
     c, 7995, 8380, 8766, 9025, 9156, 9158, 9031, 8775, 8390, 8004
     c, 7745, 7614, 7609, 7734, 7987,    0,  -16, 7116, 7875, 8131
     c, 7873, 7871, 8125, 8381, 8639, 8641, 8387, 8131,   64, 8130
     c, 8001, 7999, 8126, 8382, 8511, 8513, 8386, 8130,   64, 8257
     c, 8128, 8255, 8384, 8257,    0,  -16, 7116, 8261, 8261, 8389
     c, 8644, 8771, 8897, 8895, 8765, 8636, 8379, 8251,    0,  -16
     c, 7116, 6478, 6464, 7232,   64, 7872, 8640,   64, 9280,10048
     c,    0,  -16, 7116, 6478, 6461, 6467,10051,10045,    0,  -16
     c, 7116, 7240, 8270, 7232,   64, 8270, 9280,    0,  -16, 7116
     c, 6478, 6464,10048,   64, 7225, 9273,   64, 7986, 8498,    0
     c,  -16, 7116, 6478, 6464,10048,   64, 6464, 8240,   64,10048
     c, 8240,    0,  -16, 7116, 7367, 8135, 7750, 7492, 7361, 7359
     c, 7484, 7738, 8121, 8377, 8762, 9020, 9151, 9153, 9028, 8774
     c, 8391, 8135,    0,  -16, 7116, 7494, 7494, 7482, 9018, 9030
     c, 7494,    0,  -16, 7116, 7367, 8264, 7356, 9148, 8264,    0
     c,  -16, 7116, 7494, 8266, 7488, 8246, 9024, 8266,    0,  -16
     c, 7116, 7240, 8265, 8003, 7235, 7871, 7609, 8253, 8889, 8639
     c, 9283, 8515, 8265,    0,  -16, 7116, 7367, 8263, 8249,   64
     c, 7360, 9152,    0,  -16, 7116, 7621, 7621, 8891,   64, 8901
     c, 7611,    0,  -16, 7116, 7621, 8262, 8250,   64, 7619, 8893
     c,   64, 8899, 7613,    0,  -16, 7116, 7748, 8132, 7875, 7745
     c, 7743, 7869, 8124, 8380, 8637, 8767, 8769, 8643, 8388, 8132
     c,   64, 7873, 7871,   64, 8002, 7998,   64, 8131, 8125,   64
     c, 8259, 8253,   64, 8387, 8381,   64, 8514, 8510,   64, 8641
     c, 8639,    0,  -16, 7116, 7748, 7748, 7740, 8764, 8772, 7748
     c,   64, 7875, 7869,   64, 8003, 7997,   64, 8131, 8125,   64
     c, 8259, 8253,   64, 8387, 8381,   64, 8515, 8509,   64, 8643
     c, 8637,    0,  -16, 7116, 7621, 8262, 7613, 8893, 8262,   64
     c, 8259, 7870,   64, 8259, 8638,   64, 8256, 8126,   64, 8256
     c, 8382,    0,  -16, 7116, 7491, 7488, 8635, 8645, 7488,   64
     c, 7872, 8509,   64, 7872, 8515,   64, 8256, 8511,   64, 8256
     c, 8513,    0,  -16, 7116, 7621, 8250, 8899, 7619, 8250,   64
     c, 8253, 8642,   64, 8253, 7874,   64, 8256, 8386,   64, 8256
     c, 8130,    0,  -16, 7116, 7878, 9024, 7877, 7867, 9024,   64
     c, 8640, 8003,   64, 8640, 7997,   64, 8256, 8001,   64, 8256
     c, 7999,    0,  -16, 7116, 7494, 8262, 7739, 9026, 7490, 8763
     c, 8262,   64, 8256, 8262,   64, 8256, 7490,   64, 8256, 7739
     c,   64, 8256, 8763,   64, 8256, 9026,    0,  -16, 7116, 8263
     c, 8263, 8249,   64, 8263, 9156, 8257,   64, 8389, 8772, 8387
     c,    0,  -16, 7116, 7621, 8262, 8250,   64, 7875, 8643,   64
     c, 7613, 7867, 8122, 8378, 8635, 8893,    0,  -16, 7116, 7494
     c, 8262, 8250,   64, 7489, 7619, 8899, 9025,   64, 7995, 8507
     c,    0,  -16, 7116, 7367, 7620, 8890,   64, 8900, 7610,   64
     c, 7878, 7491, 7361,   64, 8646, 9027, 9153,    0,  -16, 7116
     c, 7113, 7753, 7095,   64, 8777, 9399,   64, 7621, 9399,   64
     c, 8901, 7095,   64, 7753, 8777,   64, 7621, 8901,    0,  -16
     c, 7116, 7113, 8267, 8252,   64, 7624, 8898,   64, 8904, 7618
     c,   64, 7100, 7478,   64, 9404, 9014,   64, 7100, 9404,   64
     c, 7478, 9014,    0,  -16, 7116, 6857, 7624, 8380,   64, 7362
     c, 8390,   64, 6838, 9398, 9408, 6838,    0,  -16, 7116, 7494
     c, 8006, 8002, 7490, 7486, 7998, 7994, 8506, 8510, 9022, 9026
     c, 8514, 8518, 8006,    0,  -16, 7116, 7367, 9154, 9028, 8774
     c, 8391, 8135, 7750, 7492, 7361, 7359, 7484, 7738, 8121, 8377
     c, 8762, 9020, 9150,   64, 9154, 8900, 8645, 8389, 8132, 8003
     c, 7873, 7871, 7997, 8124, 8379, 8635, 8892, 9150,    0,  -16
     c, 7116, 7367, 8264, 7356, 9148, 8264,   64, 8248, 9156, 7364
     c, 8248,    0,  -16, 7116, 6859, 8009, 8011, 8140, 8396, 8523
     c, 8521,   64, 6840, 6970, 7228, 7358, 7490, 7495, 7624, 7881
     c, 8649, 8904, 9031, 9026, 9150, 9276, 9530, 9656,   64, 6840
     c, 9656,   64, 8120, 7991, 8118, 8374, 8503, 8376,    0,  -16
     c, 7116, 7240, 8261, 8255,   64, 8255, 8118,   64, 8255, 8374
     c,   64, 8118, 8374,   64, 8261, 8136, 8010, 7755,   64, 8136
     c, 7755,   64, 8261, 8392, 8522, 8779,   64, 8392, 8779,   64
     c, 8261, 7751, 7495, 7237,   64, 8006, 7494, 7237,   64, 8261
     c, 8775, 9031, 9285,   64, 8518, 9030, 9285,   64, 8261, 8004
     c, 7875, 7872,   64, 8261, 8003, 7872,   64, 8261, 8516, 8643
     c, 8640,   64, 8261, 8515, 8640,    0,  -16, 7116, 7240, 8265
     c, 8263,   64, 8260, 8258,   64, 8255, 8253,   64, 8249, 8118
     c,   64, 8249, 8374,   64, 8118, 8374,   64, 8267, 8137, 8008
     c,   64, 8267, 8393, 8520,   64, 8008, 8265, 8520,   64, 8263
     c, 8004, 7747, 7620,   64, 8263, 8516, 8771, 8900,   64, 7747
     c, 8003, 8260, 8515, 8771,   64, 8258, 7999, 7742, 7486, 7360
     c, 7359, 7486,   64, 8258, 8511, 8766, 9022, 9152, 9151, 9022
     c,   64, 7742, 7998, 8255, 8510, 8766,   64, 8253, 7994, 7865
     c, 7608, 7480, 7353, 7227, 7225, 7480,   64, 8253, 8506, 8633
     c, 8888, 9016, 9145, 9275, 9273, 9016,   64, 7608, 7864, 8249
     c, 8632, 8888,    0,  -16, 7116, 7240, 8249, 8118,   64, 8249
     c, 8374,   64, 8118, 8374,   64, 8249, 8632, 9016, 9274, 9277
     c, 9150, 8894, 9152, 9283, 9157, 8902, 8645, 8776, 8650, 8395
     c, 8139, 7882, 7752, 7877, 7622, 7365, 7235, 7360, 7614, 7358
     c, 7229, 7226, 7480, 7864, 8249,    0,  -16, 7116, 7240, 8249
     c, 8118,   64, 8249, 8374,   64, 8118, 8374,   64, 8249, 8762
     c, 8764, 9021, 9024, 9281, 9286, 9161, 9034, 8778, 8523, 8011
     c, 7754, 7498, 7369, 7238, 7233, 7488, 7485, 7740, 7738, 8249
     c,    0,  -16, 7116, 7113, 7106, 7360,   64, 7495, 7746,   64
     c, 8267, 8259,   64, 9031, 8770,   64, 9410, 9152,    0,  -16
     c, 7116, 8129, 8257, 8128, 8255, 8384, 8257,    0,  -16, 7116
     c, 8002, 8130, 8001, 7999, 8126, 8382, 8511, 8513, 8386, 8130
     c,    0,  -16, 7116, 7748, 8132, 7875, 7745, 7743, 7869, 8124
     c, 8380, 8637, 8767, 8769, 8643, 8388, 8132,    0,  -16, 7116
     c, 7621, 8133, 7876, 7747, 7617, 7615, 7741, 7868, 8123, 8379
     c, 8636, 8765, 8895, 8897, 8771, 8644, 8389, 8133,    0,  -16
     c, 7116, 7367, 8135, 7750, 7492, 7361, 7359, 7484, 7738, 8121
     c, 8377, 8762, 9020, 9151, 9153, 9028, 8774, 8391, 8135,    0
     c,  -16, 7116, 6859, 8011, 7626, 7240, 6981, 6850, 6846, 6971
     c, 7224, 7606, 7989, 8501, 8886, 9272, 9531, 9662, 9666, 9541
     c, 9288, 8906, 8523, 8011,    0,  -16, 7116, 6097, 8017, 7504
     c, 7247, 6861, 6603, 6344, 6214, 6082, 6078, 6202, 6328, 6581
     c, 6835, 7217, 7472, 7983, 8495, 9008, 9265, 9651, 9909,10168
     c,10298,10430,10434,10310,10184, 9931, 9677, 9295, 9040, 8529
     c, 8017,    0,  -16, 7116, 5462, 8022, 7381, 6867, 6481, 6094
     c, 5835, 5575, 5442, 5438, 5561, 5813, 6066, 6447, 6829, 7339
     c, 7978, 8490, 9131, 9645,10031,10418,10677,10937,11070,11074
     c,10951,10699,10446,10065, 9683, 9173, 8534, 8022,    0,  -16
     c, 7116, 3049, 7913, 7144, 6631, 5989, 5346, 4831, 4315, 3927
     c, 3538, 3277, 3145, 3011, 3005, 3127, 3251, 3502, 3881, 4261
     c, 4769, 5278, 5915, 6553, 7064, 7831, 8599, 9368, 9881,10523
     c,11166,11681,12197,12585,12974,13235,13367,13501,13507,13385
     c,13261,13010,12631,12251,11743,11234,10597, 9959, 9448, 8681
     c, 7913,    0,  -16, 7116, 5716, 8273, 8015, 7630, 7246, 6863
     c, 6609, 5706, 5960, 6085, 6072, 6197, 6451, 6834, 7602, 7985
     c, 8239,   64, 8273, 8527, 8910, 9294, 9679, 9937,10826,10568
     c,10437,10424,10293,10035, 9650, 8882, 8497, 8239,    0,  -16
     c, 7116, 6097, 8273, 8015, 7630, 7246, 6863, 6609, 6219, 6087
     c, 6082, 6206, 6458, 6838, 7474, 8239,   64, 8273, 8527, 8910
     c, 9294, 9679, 9937,10315,10439,10434,10302,10042, 9654, 9010
     c, 8239,   64, 6218,10314,    0,  -16, 7116, 6985, 8898, 8769
     c, 8896, 9025, 9026, 8772, 8517, 8133, 7748, 7490, 7359, 7357
     c, 7482, 7736, 8119, 8375, 8760, 9018,   64, 8133, 7876, 7618
     c, 7487, 7485, 7610, 7864, 8119,   64, 7472, 9036,    0,  -16
     c, 7116, 6986, 8268, 7351,   64, 8268, 9143,   64, 8265, 9015
     c,   64, 7613, 8765,   64, 7095, 7863,   64, 8631, 9399,    0
     c,  -16, 7116, 6859, 7500, 7479,   64, 7628, 7607,   64, 7116
     c, 8652, 9035, 9162, 9288, 9286, 9156, 9027, 8642,   64, 8652
     c, 8907, 9034, 9160, 9158, 9028, 8899, 8642,   64, 7618, 8642
     c, 9025, 9152, 9278, 9275, 9145, 9016, 8631, 7095,   64, 8642
     c, 8897, 9024, 9150, 9147, 9017, 8888, 8631,    0,  -16, 7116
     c, 6858, 9033, 9158, 9164, 9033, 8779, 8396, 8140, 7755, 7497
     c, 7367, 7236, 7231, 7356, 7482, 7736, 8119, 8375, 8760, 9018
     c, 9148,   64, 8140, 7883, 7625, 7495, 7364, 7359, 7484, 7610
     c, 7864, 8119,    0,  -16, 7116, 6859, 7500, 7479,   64, 7628
     c, 7607,   64, 7116, 8396, 8779, 9033, 9159, 9284, 9279, 9148
     c, 9018, 8760, 8375, 7095,   64, 8396, 8651, 8905, 9031, 9156
     c, 9151, 9020, 8890, 8632, 8375,    0,  -16, 7116, 6858, 7500
     c, 7479,   64, 7628, 7607,   64, 8390, 8382,   64, 7116, 9164
     c, 9158, 9036,   64, 7618, 8386,   64, 7095, 9143, 9149, 9015
     c,    0,  -16, 7116, 6857, 7500, 7479,   64, 7628, 7607,   64
     c, 8390, 8382,   64, 7116, 9164, 9158, 9036,   64, 7618, 8386
     c,   64, 7095, 7991,    0,  -16, 7116, 6860, 9033, 9158, 9164
     c, 9033, 8779, 8396, 8140, 7755, 7497, 7367, 7236, 7231, 7356
     c, 7482, 7736, 8119, 8375, 8760, 9018,   64, 8140, 7883, 7625
     c, 7495, 7364, 7359, 7484, 7610, 7864, 8119,   64, 9023, 9015
     c,   64, 9151, 9143,   64, 8639, 9535,    0,  -16, 7116, 6732
     c, 7372, 7351,   64, 7500, 7479,   64, 9036, 9015,   64, 9164
     c, 9143,   64, 6988, 7884,   64, 8652, 9548,   64, 7490, 9026
     c,   64, 6967, 7863,   64, 8631, 9527,    0,  -16, 7116, 7622
     c, 8268, 8247,   64, 8396, 8375,   64, 7884, 8780,   64, 7863
     c, 8759,    0,  -16, 7116, 7368, 8652, 8635, 8504, 8247, 7991
     c, 7736, 7610, 7612, 7741, 7868, 7739,   64, 8524, 8507, 8376
     c, 8247,   64, 8140, 9036,    0,  -16, 7116, 6730, 7372, 7351
     c,   64, 7500, 7479,   64, 9164, 7487,   64, 8131, 9143,   64
     c, 8003, 9015,   64, 6988, 7884,   64, 8652, 9420,   64, 6967
     c, 7863,   64, 8631, 9399,    0,  -16, 7116, 7113, 7756, 7735
     c,   64, 7884, 7863,   64, 7372, 8268,   64, 7351, 9271, 9277
     c, 9143,    0,  -16, 7116, 6733, 7372, 7351,   64, 7500, 8250
     c,   64, 7372, 8247,   64, 9164, 8247,   64, 9164, 9143,   64
     c, 9292, 9271,   64, 6988, 7500,   64, 9164, 9676,   64, 6967
     c, 7735,   64, 8759, 9655,    0,  -16, 7116, 6860, 7500, 7479
     c,   64, 7628, 9145,   64, 7626, 9143,   64, 9164, 9143,   64
     c, 7116, 7628,   64, 8780, 9548,   64, 7095, 7863,    0,  -16
     c, 7116, 6859, 8140, 7755, 7497, 7367, 7235, 7232, 7356, 7482
     c, 7736, 8119, 8375, 8760, 9018, 9148, 9280, 9283, 9159, 9033
     c, 8779, 8396, 8140,   64, 8140, 7883, 7625, 7495, 7363, 7360
     c, 7484, 7610, 7864, 8119,   64, 8375, 8632, 8890, 9020, 9152
     c, 9155, 9031, 8905, 8651, 8396,    0,  -16, 7116, 6859, 7500
     c, 7479,   64, 7628, 7607,   64, 7116, 8652, 9035, 9162, 9288
     c, 9285, 9155, 9026, 8641, 7617,   64, 8652, 8907, 9034, 9160
     c, 9157, 9027, 8898, 8641,   64, 7095, 7991,    0,  -16, 7116
     c, 6859, 8140, 7755, 7497, 7367, 7235, 7232, 7356, 7482, 7736
     c, 8119, 8375, 8760, 9018, 9148, 9280, 9283, 9159, 9033, 8779
     c, 8396, 8140,   64, 8140, 7883, 7625, 7495, 7363, 7360, 7484
     c, 7610, 7864, 8119,   64, 8375, 8632, 8890, 9020, 9152, 9155
     c, 9031, 8905, 8651, 8396,   64, 7737, 7738, 7868, 8125, 8253
     c, 8508, 8634, 8755, 8882, 9138, 9268, 9269,   64, 8634, 8758
     c, 8884, 9011, 9139, 9268,    0,  -16, 7116, 6859, 7500, 7479
     c,   64, 7628, 7607,   64, 7116, 8652, 9035, 9162, 9288, 9286
     c, 9156, 9027, 8642, 7618,   64, 8652, 8907, 9034, 9160, 9158
     c, 9028, 8899, 8642,   64, 7095, 7991,   64, 8258, 8513, 8640
     c, 9017, 9144, 9272, 9401,   64, 8513, 8639, 8888, 9015, 9271
     c, 9401, 9402,    0,  -16, 7116, 6986, 9033, 9164, 9158, 9033
     c, 8779, 8396, 8012, 7627, 7369, 7367, 7493, 7620, 7875, 8641
     c, 8896, 9150,   64, 7367, 7621, 7876, 8642, 8897, 9024, 9150
     c, 9146, 8888, 8503, 8119, 7736, 7482, 7357, 7351, 7482,    0
     c,  -16, 7116, 7114, 8268, 8247,   64, 8396, 8375,   64, 7500
     c, 7366, 7372, 9292, 9286, 9164,   64, 7863, 8759,    0,  -16
     c, 7116, 6732, 7372, 7357, 7482, 7736, 8119, 8375, 8760, 9018
     c, 9149, 9164,   64, 7500, 7485, 7610, 7864, 8119,   64, 6988
     c, 7884,   64, 8780, 9548,    0,  -16, 7116, 6986, 7372, 8247
     c,   64, 7500, 8250,   64, 9164, 8247,   64, 7116, 7884,   64
     c, 8652, 9420,    0,  -16, 7116, 6732, 7244, 7735,   64, 7372
     c, 7740,   64, 8268, 7735,   64, 8268, 8759,   64, 8396, 8764
     c,   64, 9292, 8759,   64, 6860, 7756,   64, 8908, 9676,    0
     c,  -16, 7116, 6986, 7372, 9015,   64, 7500, 9143,   64, 9164
     c, 7351,   64, 7116, 7884,   64, 8652, 9420,   64, 7095, 7863
     c,   64, 8631, 9399,    0,  -16, 7116, 6987, 7372, 8257, 8247
     c,   64, 7500, 8385, 8375,   64, 9292, 8385,   64, 7116, 7884
     c,   64, 8780, 9548,   64, 7863, 8759,    0,  -16, 7116, 6986
     c, 9036, 7351,   64, 9164, 7479,   64, 7500, 7366, 7372, 9164
     c,   64, 7351, 9143, 9149, 9015,    0,  -16, 7116, 6986, 8268
     c, 7351,   64, 8268, 9143,   64, 8265, 9015,   64, 7613, 8765
     c,   64, 7095, 7863,   64, 8631, 9399,    0,  -16, 7116, 6859
     c, 7500, 7479,   64, 7628, 7607,   64, 7116, 8652, 9035, 9162
     c, 9288, 9286, 9156, 9027, 8642,   64, 8652, 8907, 9034, 9160
     c, 9158, 9028, 8899, 8642,   64, 7618, 8642, 9025, 9152, 9278
     c, 9275, 9145, 9016, 8631, 7095,   64, 8642, 8897, 9024, 9150
     c, 9147, 9017, 8888, 8631,    0,  -16, 7116, 7113, 7756, 7735
     c,   64, 7884, 7863,   64, 7372, 9292, 9286, 9164,   64, 7351
     c, 8247,    0,  -16, 7116, 6986, 8268, 7223,   64, 8268, 9271
     c,   64, 8265, 9143,   64, 7352, 9144,   64, 7223, 9271,    0
     c,  -16, 7116, 6858, 7500, 7479,   64, 7628, 7607,   64, 8390
     c, 8382,   64, 7116, 9164, 9158, 9036,   64, 7618, 8386,   64
     c, 7095, 9143, 9149, 9015,    0,  -16, 7116, 6986, 9036, 7351
     c,   64, 9164, 7479,   64, 7500, 7366, 7372, 9164,   64, 7351
     c, 9143, 9149, 9015,    0,  -16, 7116, 6732, 7372, 7351,   64
     c, 7500, 7479,   64, 9036, 9015,   64, 9164, 9143,   64, 6988
     c, 7884,   64, 8652, 9548,   64, 7490, 9026,   64, 6967, 7863
     c,   64, 8631, 9527,    0,  -16, 7116, 6859, 8140, 7755, 7497
     c, 7367, 7235, 7232, 7356, 7482, 7736, 8119, 8375, 8760, 9018
     c, 9148, 9280, 9283, 9159, 9033, 8779, 8396, 8140,   64, 8140
     c, 7883, 7625, 7495, 7363, 7360, 7484, 7610, 7864, 8119,   64
     c, 8375, 8632, 8890, 9020, 9152, 9155, 9031, 8905, 8651, 8396
     c,   64, 7877, 7870,   64, 8645, 8638,   64, 7874, 8642,   64
     c, 7873, 8641,    0,  -16, 7116, 7622, 8268, 8247,   64, 8396
     c, 8375,   64, 7884, 8780,   64, 7863, 8759,    0,  -16, 7116
     c, 6730, 7372, 7351,   64, 7500, 7479,   64, 9164, 7487,   64
     c, 8131, 9143,   64, 8003, 9015,   64, 6988, 7884,   64, 8652
     c, 9420,   64, 6967, 7863,   64, 8631, 9399,    0,  -16, 7116
     c, 6986, 8268, 7351,   64, 8268, 9143,   64, 8265, 9015,   64
     c, 7095, 7863,   64, 8631, 9399,    0,  -16, 7116, 6733, 7372
     c, 7351,   64, 7500, 8250,   64, 7372, 8247,   64, 9164, 8247
     c,   64, 9164, 9143,   64, 9292, 9271,   64, 6988, 7500,   64
     c, 9164, 9676,   64, 6967, 7735,   64, 8759, 9655,    0,  -16
     c, 7116, 6860, 7500, 7479,   64, 7628, 9145,   64, 7626, 9143
     c,   64, 9164, 9143,   64, 7116, 7628,   64, 8780, 9548,   64
     c, 7095, 7863,    0,  -16, 7116, 6859, 7373, 7240,   64, 9293
     c, 9160,   64, 7876, 7743,   64, 8772, 8639,   64, 7355, 7222
     c,   64, 9275, 9142,   64, 7371, 9163,   64, 7370, 9162,   64
     c, 7874, 8642,   64, 7873, 8641,   64, 7353, 9145,   64, 7352
     c, 9144,    0,  -16, 7116, 6859, 8140, 7755, 7497, 7367, 7235
     c, 7232, 7356, 7482, 7736, 8119, 8375, 8760, 9018, 9148, 9280
     c, 9283, 9159, 9033, 8779, 8396, 8140,   64, 8140, 7883, 7625
     c, 7495, 7363, 7360, 7484, 7610, 7864, 8119,   64, 8375, 8632
     c, 8890, 9020, 9152, 9155, 9031, 8905, 8651, 8396,    0,  -16
     c, 7116, 6732, 7372, 7351,   64, 7500, 7479,   64, 9036, 9015
     c,   64, 9164, 9143,   64, 6988, 9548,   64, 6967, 7863,   64
     c, 8631, 9527,    0,  -16, 7116, 6859, 7500, 7479,   64, 7628
     c, 7607,   64, 7116, 8652, 9035, 9162, 9288, 9285, 9155, 9026
     c, 8641, 7617,   64, 8652, 8907, 9034, 9160, 9157, 9027, 8898
     c, 8641,   64, 7095, 7991,    0,  -16, 7116, 6987, 7372, 8258
     c, 7223,   64, 7244, 8130,   64, 7244, 9164, 9286, 9036,   64
     c, 7352, 9016,   64, 7223, 9143, 9277, 9015,    0,  -16, 7116
     c, 7114, 8268, 8247,   64, 8396, 8375,   64, 7500, 7366, 7372
     c, 9292, 9286, 9164,   64, 7863, 8759,    0,  -16, 7116, 7114
     c, 7367, 7369, 7499, 7628, 7884, 8011, 8137, 8261, 8247,   64
     c, 7369, 7627, 7883, 8137,   64, 9287, 9289, 9163, 9036, 8780
     c, 8651, 8521, 8389, 8375,   64, 9289, 9035, 8779, 8521,   64
     c, 7863, 8759,    0,  -16, 7116, 6987, 8268, 8247,   64, 8396
     c, 8375,   64, 8007, 7622, 7493, 7363, 7360, 7486, 7613, 7996
     c, 8636, 9021, 9150, 9280, 9283, 9157, 9030, 8647, 8007,   64
     c, 8007, 7750, 7621, 7491, 7488, 7614, 7741, 7996,   64, 8636
     c, 8893, 9022, 9152, 9155, 9029, 8902, 8647,   64, 7884, 8780
     c,   64, 7863, 8759,    0,  -16, 7116, 6986, 7372, 9015,   64
     c, 7500, 9143,   64, 9164, 7351,   64, 7116, 7884,   64, 8652
     c, 9420,   64, 7095, 7863,   64, 8631, 9399,    0,  -16, 7116
     c, 6860, 8268, 8247,   64, 8396, 8375,   64, 7109, 7238, 7493
     c, 7617, 7743, 7870, 8125,   64, 7238, 7365, 7489, 7615, 7742
     c, 8125, 8509, 8894, 9023, 9153, 9285, 9414,   64, 8509, 8766
     c, 8895, 9025, 9157, 9414, 9541,   64, 7884, 8780,   64, 7863
     c, 8759,    0,  -16, 7116, 6859, 7226, 7351, 7863, 7611, 7359
     c, 7234, 7238, 7369, 7627, 8012, 8524, 8907, 9161, 9286, 9282
     c, 9151, 8891, 8631, 9143, 9274,   64, 7611, 7486, 7362, 7366
     c, 7497, 7755, 8012,   64, 8524, 8779, 9033, 9158, 9154, 9022
     c, 8891,   64, 7352, 7736,   64, 8760, 9144,    0,  -16, 7116
     c, 6986, 8652, 6967,   64, 8652, 8759,   64, 8522, 8631,   64
     c, 7485, 8637,   64, 6711, 7479,   64, 8247, 9015,    0,  -16
     c, 7116, 6732, 7884, 7095,   64, 8012, 7223,   64, 7500, 8908
     c, 9291, 9417, 9415, 9284, 9155, 8770,   64, 8908, 9163, 9289
     c, 9287, 9156, 9027, 8770,   64, 7618, 8770, 9025, 9151, 9149
     c, 9018, 8760, 8247, 6711,   64, 8770, 8897, 9023, 9021, 8890
     c, 8632, 8247,    0,  -16, 7116, 6987, 9290, 9418, 9548, 9414
     c, 9416, 9290, 9163, 8908, 8524, 8139, 7881, 7622, 7491, 7359
     c, 7356, 7481, 7608, 7991, 8375, 8632, 8890, 9020,   64, 8524
     c, 8267, 8009, 7750, 7619, 7487, 7484, 7609, 7736, 7991,    0
     c,  -16, 7116, 6731, 7884, 7095,   64, 8012, 7223,   64, 7500
     c, 8652, 9035, 9162, 9287, 9283, 9151, 8891, 8633, 8376, 7863
     c, 6711,   64, 8652, 8907, 9034, 9159, 9155, 9023, 8763, 8505
     c, 8248, 7863,    0,  -16, 7116, 6731, 7884, 7095,   64, 8012
     c, 7223,   64, 8518, 8254,   64, 7500, 9420, 9286, 9292,   64
     c, 7618, 8386,   64, 6711, 8631, 8892, 8503,    0,  -16, 7116
     c, 6730, 7884, 7095,   64, 8012, 7223,   64, 8518, 8254,   64
     c, 7500, 9420, 9286, 9292,   64, 7618, 8386,   64, 6711, 7607
     c,    0,  -16, 7116, 6988, 9290, 9418, 9548, 9414, 9416, 9290
     c, 9163, 8908, 8524, 8139, 7881, 7622, 7491, 7359, 7356, 7481
     c, 7608, 7991, 8247, 8632, 8890, 9150,   64, 8524, 8267, 8009
     c, 7750, 7619, 7487, 7484, 7609, 7736, 7991,   64, 8247, 8504
     c, 8762, 9022,   64, 8638, 9534,    0,  -16, 7116, 6605, 7756
     c, 6967,   64, 7884, 7095,   64, 9420, 8631,   64, 9548, 8759
     c,   64, 7372, 8268,   64, 9036, 9932,   64, 7490, 9026,   64
     c, 6583, 7479,   64, 8247, 9143,    0,  -16, 7116, 7495, 8652
     c, 7863,   64, 8780, 7991,   64, 8268, 9164,   64, 7479, 8375
     c,    0,  -16, 7116, 7113, 9036, 8379, 8249, 8120, 7863, 7607
     c, 7352, 7226, 7228, 7357, 7484, 7355,   64, 8908, 8251, 8121
     c, 7863,   64, 8524, 9420,    0,  -16, 7116, 6731, 7884, 7095
     c,   64, 8012, 7223,   64, 9676, 7487,   64, 8387, 8887,   64
     c, 8259, 8759,   64, 7500, 8396,   64, 9164, 9932,   64, 6711
     c, 7607,   64, 8375, 9143,    0,  -16, 7116, 6986, 8140, 7351
     c,   64, 8268, 7479,   64, 7756, 8652,   64, 6967, 8887, 9149
     c, 8759,    0,  -16, 7116, 6606, 7756, 6967,   64, 7756, 7863
     c,   64, 7884, 7993,   64, 9548, 7863,   64, 9548, 8759,   64
     c, 9676, 8887,   64, 7372, 7884,   64, 9548,10060,   64, 6583
     c, 7351,   64, 8375, 9271,    0,  -16, 7116, 6733, 7884, 7095
     c,   64, 7884, 8762,   64, 7881, 8759,   64, 9548, 8759,   64
     c, 7500, 7884,   64, 9164, 9932,   64, 6711, 7479,    0,  -16
     c, 7116, 6859, 8396, 8011, 7753, 7494, 7363, 7231, 7228, 7353
     c, 7480, 7735, 8119, 8504, 8762, 9021, 9152, 9284, 9287, 9162
     c, 9035, 8780, 8396,   64, 8396, 8139, 7881, 7622, 7491, 7359
     c, 7356, 7481, 7735,   64, 8119, 8376, 8634, 8893, 9024, 9156
     c, 9159, 9034, 8780,    0,  -16, 7116, 6731, 7884, 7095,   64
     c, 8012, 7223,   64, 7500, 9036, 9419, 9545, 9543, 9412, 9154
     c, 8641, 7617,   64, 9036, 9291, 9417, 9415, 9284, 9026, 8641
     c,   64, 6711, 7607,    0,  -16, 7116, 6859, 8396, 8011, 7753
     c, 7494, 7363, 7231, 7228, 7353, 7480, 7735, 8119, 8504, 8762
     c, 9021, 9152, 9284, 9287, 9162, 9035, 8780, 8396,   64, 8396
     c, 8139, 7881, 7622, 7491, 7359, 7356, 7481, 7735,   64, 8119
     c, 8376, 8634, 8893, 9024, 9156, 9159, 9034, 8780,   64, 7481
     c, 7482, 7612, 7869, 7997, 8252, 8378, 8371, 8498, 8754, 8884
     c, 8885,   64, 8378, 8500, 8627, 8755, 8884,    0,  -16, 7116
     c, 6732, 7884, 7095,   64, 8012, 7223,   64, 7500, 8908, 9291
     c, 9417, 9415, 9284, 9155, 8770, 7618,   64, 8908, 9163, 9289
     c, 9287, 9156, 9027, 8770,   64, 8258, 8513, 8640, 8760, 8887
     c, 9143, 9273, 9274,   64, 8640, 8889, 9016, 9144, 9273,   64
     c, 6711, 7607,    0,  -16, 7116, 6860, 9290, 9418, 9548, 9414
     c, 9416, 9290, 9163, 8780, 8268, 7883, 7625, 7623, 7749, 7876
     c, 8768, 9022,   64, 7623, 7877, 8769, 8896, 9022, 9019, 8889
     c, 8760, 8375, 7863, 7480, 7353, 7227, 7229, 7095, 7225, 7353
     c,    0,  -16, 7116, 6987, 8652, 7863,   64, 8780, 7991,   64
     c, 7884, 7494, 7756, 9676, 9542, 9548,   64, 7479, 8375,    0
     c,  -16, 7116, 6733, 7756, 7361, 7229, 7226, 7352, 7735, 8247
     c, 8632, 8890, 9021, 9548,   64, 7884, 7489, 7357, 7354, 7480
     c, 7735,   64, 7372, 8268,   64, 9164, 9932,    0,  -16, 7116
     c, 6986, 7756, 7863,   64, 7884, 7993,   64, 9548, 7863,   64
     c, 7500, 8268,   64, 9036, 9804,    0,  -16, 7116, 6605, 7628
     c, 7351,   64, 7756, 7481,   64, 8652, 7351,   64, 8652, 8375
     c,   64, 8780, 8505,   64, 9676, 8375,   64, 7244, 8140,   64
     c, 9292,10060,    0,  -16, 7116, 6859, 7756, 8631,   64, 7884
     c, 8759,   64, 9548, 6967,   64, 7500, 8268,   64, 9036, 9804
     c,   64, 6711, 7479,   64, 8247, 9015,    0,  -16, 7116, 6987
     c, 7756, 8258, 7863,   64, 7884, 8386, 7991,   64, 9676, 8386
     c,   64, 7500, 8268,   64, 9164, 9932,   64, 7479, 8375,    0
     c,  -16, 7116, 6859, 9420, 6967,   64, 9548, 7095,   64, 7884
     c, 7494, 7756, 9548,   64, 6967, 8759, 9021, 8631,    0,  -16
     c, 7116, 6986, 7369, 7495, 9019, 9145, 9143,   64, 7494, 9018
     c,   64, 7369, 7367, 7493, 9017, 9143,   64, 8002, 7486, 7356
     c, 7354, 7480, 7351,   64, 7356, 7608,   64, 7486, 7484, 7610
     c, 7608, 7351,   64, 8383, 9028,   64, 8777, 8774, 8900, 9156
     c, 9158, 8903, 8777,   64, 8777, 8902, 9156,    0,  -16, 7116
     c, 6986, 8137, 7351,   64, 8393, 9143,   64, 8265, 9015,   64
     c, 7613, 8765,   64, 7095, 7863,   64, 8631, 9399,   64, 8144
     c, 7887, 7757, 7755, 7881, 8136, 8392, 8649, 8779, 8781, 8655
     c, 8400, 8144,    0,  -16, 7116, 7115, 7747, 7746, 7618, 7619
     c, 7748, 8005, 8517, 8772, 8899, 9025, 9018, 9144, 9271,   64
     c, 8899, 8890, 9016, 9271, 9399,   64, 8897, 8768, 7999, 7614
     c, 7484, 7482, 7608, 7991, 8375, 8632, 8890,   64, 7999, 7742
     c, 7612, 7610, 7736, 7991,    0,  -16, 7116, 6858, 7500, 7479
     c,   64, 7628, 7607,   64, 7618, 7876, 8133, 8389, 8772, 9026
     c, 9151, 9149, 9018, 8760, 8375, 8119, 7864, 7610,   64, 8389
     c, 8644, 8898, 9023, 9021, 8890, 8632, 8375,   64, 7116, 7628
     c,    0,  -16, 7116, 6985, 8898, 8769, 8896, 9025, 9026, 8772
     c, 8517, 8133, 7748, 7490, 7359, 7357, 7482, 7736, 8119, 8375
     c, 8760, 9018,   64, 8133, 7876, 7618, 7487, 7485, 7610, 7864
     c, 8119,    0,  -16, 7116, 6987, 8908, 8887,   64, 9036, 9015
     c,   64, 8898, 8644, 8389, 8133, 7748, 7490, 7359, 7357, 7482
     c, 7736, 8119, 8375, 8632, 8890,   64, 8133, 7876, 7618, 7487
     c, 7485, 7610, 7864, 8119,   64, 8524, 9036,   64, 8887, 9399
     c,    0,  -16, 7116, 6985, 7487, 9023, 9025, 8899, 8772, 8517
     c, 8133, 7748, 7490, 7359, 7357, 7482, 7736, 8119, 8375, 8760
     c, 9018,   64, 8895, 8898, 8772,   64, 8133, 7876, 7618, 7487
     c, 7485, 7610, 7864, 8119,    0,  -16, 7116, 7366, 8651, 8522
     c, 8649, 8778, 8779, 8652, 8396, 8139, 8009, 7991,   64, 8396
     c, 8267, 8137, 8119,   64, 7621, 8645,   64, 7607, 8503,    0
     c,  -16, 7116, 7114, 8133, 7876, 7747, 7617, 7615, 7741, 7868
     c, 8123, 8379, 8636, 8765, 8895, 8897, 8771, 8644, 8389, 8133
     c,   64, 7876, 7746, 7742, 7868,   64, 8636, 8766, 8770, 8644
     c,   64, 8771, 8900, 9157, 9156, 8900,   64, 7741, 7612, 7482
     c, 7481, 7607, 7990, 8630, 9013, 9140,   64, 7481, 7608, 7991
     c, 8631, 9014, 9140, 9139, 9009, 8624, 7856, 7473, 7347, 7348
     c, 7478, 7863,    0,  -16, 7116, 6859, 7500, 7479,   64, 7628
     c, 7607,   64, 7618, 7876, 8261, 8517, 8900, 9026, 9015,   64
     c, 8517, 8772, 8898, 8887,   64, 7116, 7628,   64, 7095, 7991
     c,   64, 8503, 9399,    0,  -16, 7116, 7622, 8268, 8139, 8266
     c, 8395, 8268,   64, 8261, 8247,   64, 8389, 8375,   64, 7877
     c, 8389,   64, 7863, 8759,    0,  -16, 7116, 7622, 8396, 8267
     c, 8394, 8523, 8396,   64, 8517, 8499, 8369, 8112, 7856, 7729
     c, 7730, 7859, 7986, 7857,   64, 8389, 8371, 8241, 8112,   64
     c, 8005, 8517,    0,  -16, 7116, 6858, 7500, 7479,   64, 7628
     c, 7607,   64, 8901, 7611,   64, 8255, 9015,   64, 8127, 8887
     c,   64, 7116, 7628,   64, 8517, 9285,   64, 7095, 7991,   64
     c, 8503, 9271,    0,  -16, 7116, 7622, 8268, 8247,   64, 8396
     c, 8375,   64, 7884, 8396,   64, 7863, 8759,    0,  -16, 7116
     c, 6225, 6853, 6839,   64, 6981, 6967,   64, 6978, 7236, 7621
     c, 7877, 8260, 8386, 8375,   64, 7877, 8132, 8258, 8247,   64
     c, 8386, 8644, 9029, 9285, 9668, 9794, 9783,   64, 9285, 9540
     c, 9666, 9655,   64, 6469, 6981,   64, 6455, 7351,   64, 7863
     c, 8759,   64, 9271,10167,    0,  -16, 7116, 6859, 7493, 7479
     c,   64, 7621, 7607,   64, 7618, 7876, 8261, 8517, 8900, 9026
     c, 9015,   64, 8517, 8772, 8898, 8887,   64, 7109, 7621,   64
     c, 7095, 7991,   64, 8503, 9399,    0,  -16, 7116, 6986, 8133
     c, 7748, 7490, 7359, 7357, 7482, 7736, 8119, 8375, 8760, 9018
     c, 9149, 9151, 9026, 8772, 8389, 8133,   64, 8133, 7876, 7618
     c, 7487, 7485, 7610, 7864, 8119,   64, 8375, 8632, 8890, 9021
     c, 9023, 8898, 8644, 8389,    0,  -16, 7116, 6858, 7493, 7472
     c,   64, 7621, 7600,   64, 7618, 7876, 8133, 8389, 8772, 9026
     c, 9151, 9149, 9018, 8760, 8375, 8119, 7864, 7610,   64, 8389
     c, 8644, 8898, 9023, 9021, 8890, 8632, 8375,   64, 7109, 7621
     c,   64, 7088, 7984,    0,  -16, 7116, 6986, 8901, 8880,   64
     c, 9029, 9008,   64, 8898, 8644, 8389, 8133, 7748, 7490, 7359
     c, 7357, 7482, 7736, 8119, 8375, 8632, 8890,   64, 8133, 7876
     c, 7618, 7487, 7485, 7610, 7864, 8119,   64, 8496, 9392,    0
     c,  -16, 7116, 7112, 7749, 7735,   64, 7877, 7863,   64, 7871
     c, 8002, 8260, 8517, 8901, 9028, 9027, 8898, 8771, 8900,   64
     c, 7365, 7877,   64, 7351, 8247,    0,  -16, 7116, 7241, 8899
     c, 9029, 9025, 8899, 8772, 8517, 8005, 7748, 7619, 7617, 7744
     c, 7999, 8637, 8892, 9019,   64, 7618, 7745, 8000, 8638, 8893
     c, 9020, 9017, 8888, 8631, 8119, 7864, 7737, 7611, 7607, 7737
     c,    0,  -16, 7116, 7368, 8012, 7995, 8120, 8375, 8631, 8888
     c, 9018,   64, 8140, 8123, 8248, 8375,   64, 7621, 8645,    0
     c,  -16, 7116, 6859, 7493, 7482, 7608, 7991, 8247, 8632, 8890
     c,   64, 7621, 7610, 7736, 7991,   64, 8901, 8887,   64, 9029
     c, 9015,   64, 7109, 7621,   64, 8517, 9029,   64, 8887, 9399
     c,    0,  -16, 7116, 7113, 7493, 8247,   64, 7621, 8249,   64
     c, 9029, 8247,   64, 7237, 8005,   64, 8517, 9285,    0,  -16
     c, 7116, 6732, 7237, 7735,   64, 7365, 7738,   64, 8261, 7735
     c,   64, 8261, 8759,   64, 8389, 8762,   64, 9285, 8759,   64
     c, 6853, 7749,   64, 8901, 9669,    0,  -16, 7116, 6986, 7493
     c, 8887,   64, 7621, 9015,   64, 9029, 7479,   64, 7237, 8005
     c,   64, 8517, 9285,   64, 7223, 7991,   64, 8503, 9271,    0
     c,  -16, 7116, 6985, 7493, 8247,   64, 7621, 8249,   64, 9029
     c, 8247, 7987, 7729, 7472, 7344, 7217, 7346, 7473,   64, 7237
     c, 8005,   64, 8517, 9285,    0,  -16, 7116, 7113, 8901, 7479
     c,   64, 9029, 7607,   64, 7621, 7489, 7493, 9029,   64, 7479
     c, 9015, 9019, 8887,    0,  -16, 7116, 6860, 8133, 7748, 7490
     c, 7360, 7229, 7226, 7352, 7735, 7991, 8248, 8635, 8894, 9154
     c, 9285,   64, 8133, 7876, 7618, 7488, 7357, 7354, 7480, 7735
     c,   64, 8133, 8389, 8644, 8770, 9018, 9144, 9271,   64, 8389
     c, 8516, 8642, 8890, 9016, 9271, 9399,    0,  -16, 7116, 6858
     c, 8524, 8139, 7881, 7621, 7490, 7358, 7224, 7088,   64, 8524
     c, 8267, 8009, 7749, 7618, 7486, 7352, 7216,   64, 8524, 8780
     c, 9035, 9162, 9159, 9029, 8900, 8515, 8003,   64, 8780, 9034
     c, 9031, 8901, 8772, 8515,   64, 8003, 8514, 8768, 8894, 8891
     c, 8761, 8632, 8247, 7991, 7736, 7609, 7484,   64, 8003, 8386
     c, 8640, 8766, 8763, 8633, 8504, 8247,    0,  -16, 7116, 6986
     c, 7106, 7364, 7621, 7877, 8132, 8259, 8384, 8380, 8248, 7856
     c,   64, 7235, 7492, 8004, 8259,   64, 9285, 9154, 9024, 8377
     c, 7988, 7728,   64, 9157, 9026, 8896, 8377,    0,  -16, 7116
     c, 7114, 8772, 8517, 8261, 7876, 7617, 7486, 7483, 7609, 7736
     c, 7991, 8247, 8632, 8891, 9022, 9025, 8899, 8392, 8266, 8268
     c, 8397, 8653, 8908, 9162,   64, 8261, 8004, 7745, 7614, 7610
     c, 7736,   64, 8247, 8504, 8763, 8894, 8898, 8772, 8519, 8393
     c, 8395, 8524, 8780, 9162,    0,  -16, 7116, 7113, 9026, 8772
     c, 8517, 8005, 7748, 7746, 8000, 8383,   64, 8005, 7876, 7874
     c, 8128, 8383,   64, 8383, 7742, 7484, 7482, 7608, 7991, 8375
     c, 8632, 8890,   64, 8383, 7870, 7612, 7610, 7736, 7991,    0
     c,  -16, 7116, 7113, 8524, 8267, 8138, 8137, 8264, 8647, 9287
     c, 9288, 8903, 8389, 8003, 7616, 7485, 7483, 7609, 7991, 8373
     c, 8499, 8497, 8368, 8112, 7985,   64, 8646, 8131, 7744, 7613
     c, 7611, 7737, 7991,    0,  -16, 7116, 6859, 6977, 7107, 7365
     c, 7749, 7876, 7874, 7742, 7479,   64, 7621, 7748, 7746, 7614
     c, 7351,   64, 7742, 8002, 8260, 8517, 8773, 9028, 9155, 9152
     c, 9019, 8624,   64, 8773, 9027, 9024, 8891, 8496,    0,  -16
     c, 7116, 6731, 6849, 6979, 7237, 7621, 7748, 7746, 7613, 7610
     c, 7736, 7863,   64, 7493, 7620, 7618, 7485, 7482, 7608, 7863
     c, 8119, 8376, 8634, 8893, 9024, 9157, 9161, 9035, 8780, 8524
     c, 8266, 8264, 8389, 8642, 8896, 9278,   64, 8376, 8635, 8765
     c, 8896, 9029, 9033, 8907, 8780,    0,  -16, 7116, 7494, 8261
     c, 7998, 7866, 7864, 7991, 8375, 8633, 8763,   64, 8389, 8126
     c, 7994, 7992, 8119,    0,  -16, 7116, 6986, 7749, 7223,   64
     c, 7877, 7351,   64, 9029, 9156, 9284, 9157, 8901, 8644, 8128
     c, 7871, 7615,   64, 7871, 8126, 8376, 8503,   64, 7871, 7998
     c, 8248, 8375, 8631, 8888, 9147,    0,  -16, 7116, 6986, 7372
     c, 7628, 7883, 8010, 8136, 8890, 9016, 9143,   64, 7628, 7882
     c, 8008, 8762, 8888, 9143, 9271,   64, 8261, 7223,   64, 8261
     c, 7351,    0,  -16, 7116, 6731, 7621, 6832,   64, 7749, 6960
     c,   64, 7618, 7484, 7481, 7735, 7991, 8248, 8506, 8765,   64
     c, 9029, 8634, 8632, 8759, 9143, 9401, 9531,   64, 9157, 8762
     c, 8760, 8887,    0,  -16, 7116, 6986, 7749, 7479,   64, 7877
     c, 7743, 7610, 7479,   64, 9157, 9025, 8765,   64, 9285, 9154
     c, 9024, 8765, 8507, 8121, 7864, 7479,   64, 7365, 7877,    0
     c,  -16, 7116, 7112, 8524, 8267, 8138, 8137, 8264, 8647, 9031
     c,   64, 8647, 8134, 7877, 7747, 7745, 7999, 8382, 8766,   64
     c, 8647, 8262, 8005, 7875, 7873, 8127, 8382,   64, 8382, 7869
     c, 7612, 7482, 7480, 7734, 8372, 8499, 8497, 8240, 7984,   64
     c, 8382, 7997, 7740, 7610, 7608, 7862, 8372,    0,  -16, 7116
     c, 7113, 8261, 7876, 7617, 7486, 7483, 7609, 7736, 7991, 8247
     c, 8632, 8891, 9022, 9025, 8899, 8772, 8517, 8261,   64, 8261
     c, 8004, 7745, 7614, 7610, 7736,   64, 8247, 8504, 8763, 8894
     c, 8898, 8772,    0,  -16, 7116, 6859, 8004, 7479,   64, 8004
     c, 7607,   64, 8772, 8759,   64, 8772, 8887,   64, 7106, 7364
     c, 7749, 9413,   64, 7106, 7363, 7748, 9412,    0,  -16, 7116
     c, 6985, 7484, 7609, 7736, 7991, 8247, 8632, 8891, 9022, 9025
     c, 8899, 8772, 8517, 8261, 7876, 7617, 7486, 6960,   64, 8247
     c, 8504, 8763, 8894, 8898, 8772,   64, 8261, 8004, 7745, 7614
     c, 7088,    0,  -16, 7116, 6987, 9413, 8133, 7748, 7489, 7358
     c, 7355, 7481, 7608, 7863, 8119, 8504, 8763, 8894, 8897, 8771
     c, 8644, 8389,   64, 8133, 7876, 7617, 7486, 7482, 7608,   64
     c, 8119, 8376, 8635, 8766, 8770, 8644,   64, 8644, 9412,    0
     c,  -16, 7116, 6986, 8388, 7991,   64, 8388, 8119,   64, 7234
     c, 7492, 7877, 9285,   64, 7234, 7491, 7876, 9284,    0,  -16
     c, 7116, 6986, 7105, 7235, 7493, 7877, 8004, 8002, 7740, 7737
     c, 7991,   64, 7749, 7876, 7874, 7612, 7609, 7736, 7991, 8119
     c, 8504, 8762, 9021, 9152, 9155, 9029, 8900, 9027, 9152,   64
     c, 9021, 9155,    0,  -16, 7116, 6859, 7876, 7619, 7361, 7230
     c, 7227, 7353, 7480, 7735, 8119, 8504, 8890, 9149, 9280, 9283
     c, 9029, 8773, 8515, 8255, 7994, 7600,   64, 7227, 7481, 7736
     c, 8120, 8505, 8891, 9149,   64, 9283, 9028, 8772, 8514, 8255
     c, 7993, 7728,    0,  -16, 7116, 7113, 7365, 7621, 7876, 8002
     c, 8627, 8753, 8880,   64, 7621, 7748, 7874, 8499, 8625, 8880
     c, 9136,   64, 9285, 9155, 8896, 7605, 7346, 7216,    0,  -16
     c, 7116, 6731, 8652, 7856,   64, 8780, 7728,   64, 6849, 6979
     c, 7237, 7621, 7748, 7746, 7613, 7610, 7864, 8248, 8505, 8892
     c, 9151,   64, 7493, 7620, 7618, 7485, 7482, 7608, 7863, 8247
     c, 8504, 8762, 9021, 9151, 9413,    0,  -16, 7116, 6731, 7233
     c, 7491, 7876, 7749, 7492, 7233, 7102, 7099, 7224, 7351, 7607
     c, 7864, 8123, 8254,   64, 7099, 7225, 7352, 7608, 7865, 8123
     c,   64, 8126, 8123, 8248, 8375, 8631, 8888, 9147, 9278, 9281
     c, 9156, 9029, 8900, 9155, 9281,   64, 8123, 8249, 8376, 8632
     c, 8889, 9147,    0,  -16, 7116, 6987, 9029, 8766, 8634, 8632
     c, 8759, 9143, 9401, 9531,   64, 9157, 8894, 8762, 8760, 8887
     c,   64, 8766, 8769, 8644, 8389, 8133, 7748, 7489, 7358, 7355
     c, 7481, 7608, 7863, 8119, 8376, 8635, 8766,   64, 8133, 7876
     c, 7617, 7486, 7482, 7608,    0,  -16, 7116, 6985, 8012, 7487
     c, 7484, 7609, 7736,   64, 8140, 7615,   64, 7615, 7746, 8004
     c, 8261, 8517, 8772, 8899, 9025, 9022, 8891, 8632, 8247, 7991
     c, 7736, 7611, 7615,   64, 8772, 8898, 8894, 8763, 8504, 8247
     c,   64, 7628, 8140,    0,  -16, 7116, 7113, 8898, 8897, 9025
     c, 9026, 8900, 8645, 8261, 7876, 7617, 7486, 7483, 7609, 7736
     c, 7991, 8247, 8632, 8891,   64, 8261, 8004, 7745, 7614, 7610
     c, 7736,    0,  -16, 7116, 6987, 9292, 8766, 8634, 8632, 8759
     c, 9143, 9401, 9531,   64, 9420, 8894, 8762, 8760, 8887,   64
     c, 8766, 8769, 8644, 8389, 8133, 7748, 7489, 7358, 7355, 7481
     c, 7608, 7863, 8119, 8376, 8635, 8766,   64, 8133, 7876, 7617
     c, 7486, 7482, 7608,   64, 8908, 9420,    0,  -16, 7116, 7113
     c, 7612, 8125, 8510, 8896, 9026, 8900, 8645, 8261, 7876, 7617
     c, 7486, 7483, 7609, 7736, 7991, 8247, 8632, 8890,   64, 8261
     c, 8004, 7745, 7614, 7610, 7736,    0,  -16, 7116, 7368, 9291
     c, 9162, 9289, 9418, 9419, 9292, 9036, 8779, 8650, 8520, 8389
     c, 7991, 7859, 7729,   64, 9036, 8778, 8648, 8516, 8251, 8119
     c, 7988, 7858, 7729, 7472, 7216, 7089, 7090, 7219, 7346, 7217
     c,   64, 7877, 9157,    0,  -16, 7116, 6986, 9157, 8631, 8500
     c, 8241, 7856, 7472, 7217, 7090, 7091, 7220, 7347, 7218,   64
     c, 9029, 8503, 8372, 8113, 7856,   64, 8766, 8769, 8644, 8389
     c, 8133, 7748, 7489, 7358, 7355, 7481, 7608, 7863, 8119, 8376
     c, 8635, 8766,   64, 8133, 7876, 7617, 7486, 7482, 7608,    0
     c,  -16, 7116, 6987, 8012, 7223,   64, 8140, 7351,   64, 7614
     c, 7874, 8132, 8389, 8645, 8900, 9027, 9025, 8763, 8760, 8887
     c,   64, 8645, 8899, 8897, 8635, 8632, 8759, 9143, 9401, 9531
     c,   64, 7628, 8140,    0,  -16, 7116, 7495, 8652, 8523, 8650
     c, 8779, 8652,   64, 7617, 7747, 8005, 8389, 8516, 8513, 8251
     c, 8248, 8375,   64, 8261, 8388, 8385, 8123, 8120, 8247, 8631
     c, 8889, 9019,    0,  -16, 7116, 7495, 8780, 8651, 8778, 8907
     c, 8780,   64, 7745, 7875, 8133, 8517, 8644, 8641, 8247, 8116
     c, 7986, 7857, 7600, 7344, 7217, 7218, 7347, 7474, 7345,   64
     c, 8389, 8516, 8513, 8119, 7988, 7858, 7600,    0,  -16, 7116
     c, 6986, 8012, 7223,   64, 8140, 7351,   64, 9028, 8899, 9026
     c, 9155, 9156, 9029, 8901, 8644, 8128, 7871, 7615,   64, 7871
     c, 8126, 8376, 8503,   64, 7871, 7998, 8248, 8375, 8631, 8888
     c, 9147,   64, 7628, 8140,    0,  -16, 7116, 7623, 8652, 8126
     c, 7994, 7992, 8119, 8503, 8761, 8891,   64, 8780, 8254, 8122
     c, 8120, 8247,   64, 8268, 8780,    0,  -16, 7116, 6096, 6209
     c, 6339, 6597, 6981, 7108, 7106, 6974, 6711,   64, 6853, 6980
     c, 6978, 6846, 6583,   64, 6974, 7234, 7492, 7749, 8005, 8260
     c, 8387, 8385, 7991,   64, 8005, 8259, 8257, 7863,   64, 8254
     c, 8514, 8772, 9029, 9285, 9540, 9667, 9665, 9403, 9400, 9527
     c,   64, 9285, 9539, 9537, 9275, 9272, 9399, 9783,10041,10171
     c,    0,  -16, 7116, 6731, 6849, 6979, 7237, 7621, 7748, 7746
     c, 7614, 7351,   64, 7493, 7620, 7618, 7486, 7223,   64, 7614
     c, 7874, 8132, 8389, 8645, 8900, 9027, 9025, 8763, 8760, 8887
     c,   64, 8645, 8899, 8897, 8635, 8632, 8759, 9143, 9401, 9531
     c,    0,  -16, 7116, 7113, 8261, 7876, 7617, 7486, 7483, 7609
     c, 7736, 7991, 8247, 8632, 8891, 9022, 9025, 8899, 8772, 8517
     c, 8261,   64, 8261, 8004, 7745, 7614, 7610, 7736,   64, 8247
     c, 8504, 8763, 8894, 8898, 8772,    0,  -16, 7116, 6858, 6977
     c, 7107, 7365, 7749, 7876, 7874, 7742, 7216,   64, 7621, 7748
     c, 7746, 7614, 7088,   64, 7742, 7873, 8132, 8389, 8645, 8900
     c, 9027, 9153, 9150, 9019, 8760, 8375, 8119, 7864, 7739, 7742
     c,   64, 8900, 9026, 9022, 8891, 8632, 8375,   64, 6704, 7600
     c,    0,  -16, 7116, 6986, 9029, 8240,   64, 9157, 8368,   64
     c, 8766, 8769, 8644, 8389, 8133, 7748, 7489, 7358, 7355, 7481
     c, 7608, 7863, 8119, 8376, 8635, 8766,   64, 8133, 7876, 7617
     c, 7486, 7482, 7608,   64, 7856, 8752,    0,  -16, 7116, 7112
     c, 7233, 7363, 7621, 8005, 8132, 8130, 7998, 7735,   64, 7877
     c, 8004, 8002, 7870, 7607,   64, 7998, 8258, 8516, 8773, 9029
     c, 9156, 9155, 9026, 8899, 9028,    0,  -16, 7116, 7241, 9027
     c, 9026, 9154, 9155, 9028, 8645, 8261, 7876, 7747, 7745, 7872
     c, 8764, 8891,   64, 7746, 7873, 8765, 8892, 8889, 8760, 8375
     c, 7991, 7608, 7481, 7482, 7610, 7609,    0,  -16, 7116, 7367
     c, 8524, 7998, 7866, 7864, 7991, 8375, 8633, 8763,   64, 8652
     c, 8126, 7994, 7992, 8119,   64, 7749, 8901,    0,  -16, 7116
     c, 6731, 6849, 6979, 7237, 7621, 7748, 7745, 7483, 7481, 7735
     c,   64, 7493, 7620, 7617, 7355, 7353, 7480, 7735, 7991, 8248
     c, 8506, 8766,   64, 9029, 8766, 8634, 8632, 8759, 9143, 9401
     c, 9531,   64, 9157, 8894, 8762, 8760, 8887,    0,  -16, 7116
     c, 6986, 7105, 7235, 7493, 7877, 8004, 8001, 7739, 7737, 7991
     c,   64, 7749, 7876, 7873, 7611, 7609, 7736, 7991, 8119, 8504
     c, 8762, 9021, 9153, 9157, 9029, 9155,    0,  -16, 7116, 6350
     c, 6465, 6595, 6853, 7237, 7364, 7361, 7099, 7097, 7351,   64
     c, 7109, 7236, 7233, 6971, 6969, 7096, 7351, 7607, 7864, 8122
     c, 8252,   64, 8517, 8252, 8249, 8376, 8631, 8887, 9144, 9402
     c, 9532, 9664, 9669, 9541, 9667,   64, 8645, 8380, 8377, 8631
     c,    0,  -16, 7116, 6986, 7361, 7620, 7877, 8261, 8387, 8384
     c,   64, 8133, 8259, 8256, 8124, 7994, 7736, 7479, 7351, 7224
     c, 7225, 7354, 7481, 7352,   64, 8124, 8121, 8247, 8631, 8888
     c, 9147,   64, 9156, 9027, 9154, 9283, 9284, 9157, 9029, 8772
     c, 8514, 8384, 8252, 8249, 8375,    0,  -16, 7116, 6858, 6977
     c, 7107, 7365, 7749, 7876, 7873, 7611, 7609, 7863,   64, 7621
     c, 7748, 7745, 7483, 7481, 7608, 7863, 8119, 8376, 8634, 8894
     c,   64, 9285, 8759, 8628, 8369, 7984, 7600, 7345, 7218, 7219
     c, 7348, 7475, 7346,   64, 9157, 8631, 8500, 8241, 7984,    0
     c,  -16, 7116, 6986, 9157, 9027, 8769, 7739, 7481, 7351,   64
     c, 7489, 7619, 7877, 8261, 8771,   64, 7619, 7876, 8260, 8771
     c, 9027,   64, 7481, 7737, 8248, 8632, 8889,   64, 7737, 8247
     c, 8631, 8889, 9019,    0,  -16, 7116, 6860, 8779, 8650, 8777
     c, 8906, 8907, 8652, 8268, 7883, 7625, 7494, 7479,   64, 8268
     c, 8011, 7753, 7622, 7607,   64, 9547, 9418, 9545, 9674, 9675
     c, 9548, 9292, 9035, 8905, 8887,   64, 9292, 9163, 9033, 9015
     c,   64, 7109, 9413,   64, 7095, 7991,   64, 8503, 9399,    0
     c,  -16, 7116, 6859, 8907, 8778, 8905, 9034, 8907, 8652, 8268
     c, 7883, 7625, 7494, 7479,   64, 8268, 8011, 7753, 7622, 7607
     c,   64, 8901, 8887,   64, 9029, 9015,   64, 7109, 9029,   64
     c, 7095, 7991,   64, 8503, 9399,    0,  -16, 7116, 6859, 8779
     c, 8650, 8777, 8906, 8907, 8652,   64, 9036, 8268, 7883, 7625
     c, 7494, 7479,   64, 8268, 8011, 7753, 7622, 7607,   64, 8906
     c, 8887,   64, 9036, 9015,   64, 7109, 8901,   64, 7095, 7991
     c,   64, 8503, 9399,    0,  -16, 7116, 6225, 8267, 8138, 8265
     c, 8394, 8267, 8012, 7628, 7243, 6985, 6854, 6839,   64, 7628
     c, 7371, 7113, 6982, 6967,   64, 9675, 9546, 9673, 9802, 9675
     c, 9420, 9036, 8651, 8393, 8262, 8247,   64, 9036, 8779, 8521
     c, 8390, 8375,   64, 9669, 9655,   64, 9797, 9783,   64, 6469
     c, 9797,   64, 6455, 7351,   64, 7863, 8759,   64, 9271,10167
     c,    0,  -16, 7116, 6225, 8267, 8138, 8265, 8394, 8267, 8012
     c, 7628, 7243, 6985, 6854, 6839,   64, 7628, 7371, 7113, 6982
     c, 6967,   64, 9547, 9418, 9545, 9674, 9675, 9420,   64, 9804
     c, 9036, 8651, 8393, 8262, 8247,   64, 9036, 8779, 8521, 8390
     c, 8375,   64, 9674, 9655,   64, 9804, 9783,   64, 6469, 9669
     c,   64, 6455, 7351,   64, 7863, 8759,   64, 9271,10167,    0
     c,  -16, 7116, 7622, 8261, 8247,   64, 8389, 8375,   64, 7877
     c, 8389,   64, 7863, 8759,    0,  -16, 7116, 7112, 8900, 8645
     c, 8261, 7876, 7618, 7487, 7484, 7609, 7736, 8119, 8503, 8760
     c,   64, 8261, 8004, 7746, 7615, 7612, 7737, 7864, 8119,   64
     c, 7614, 8638,    0,  -16, 7116, 7114, 8524, 8139, 7880, 7750
     c, 7619, 7486, 7482, 7608, 7863, 8119, 8504, 8763, 8893, 9024
     c, 9157, 9161, 9035, 8780, 8524,   64, 8524, 8267, 8008, 7878
     c, 7747, 7614, 7610, 7736, 7863,   64, 8119, 8376, 8635, 8765
     c, 8896, 9029, 9033, 8907, 8780,   64, 7746, 8898,    0,  -16
     c, 7116, 6859, 8652, 7856,   64, 8780, 7728,   64, 8133, 7620
     c, 7362, 7231, 7228, 7354, 7608, 7991, 8375, 8888, 9146, 9277
     c, 9280, 9154, 8900, 8517, 8133,   64, 8133, 7748, 7490, 7359
     c, 7356, 7482, 7736, 7991,   64, 8375, 8760, 9018, 9149, 9152
     c, 9026, 8772, 8517,    0,  -16, 7116, 7113, 8517, 8772, 9026
     c, 9027, 8900, 8517, 8133, 7748, 7619, 7489, 7487, 7613, 7867
     c, 8376,   64, 8133, 7876, 7747, 7617, 7615, 7741, 8376, 8502
     c, 8500, 8371, 8115,    0,  -16, 7116, 6859, 7493, 7364, 7234
     c, 7232, 7357, 7865, 7991,   64, 7232, 7358, 7866, 7991, 7989
     c, 7858, 7600, 7472, 7345, 7219, 7222, 7354, 7614, 7873, 8260
     c, 8517, 8773, 9156, 9282, 9278, 9146, 8888, 8631, 8503, 8376
     c, 8378, 8507, 8634, 8505,   64, 8773, 9028, 9154, 9150, 9018
     c, 8888,    0,  -16, 7116, 6605, 9163, 9034, 9161, 9290, 9163
     c, 8908, 8524, 8139, 7881, 7751, 7620, 7488, 7223, 7091, 6961
     c,   64, 8524, 8267, 8009, 7879, 7748, 7483, 7351, 7220, 7090
     c, 6961, 6704, 6448, 6321, 6322, 6451, 6578, 6449,   64, 9931
     c, 9802, 9929,10058,10059, 9932, 9676, 9419, 9290, 9160, 9029
     c, 8631, 8499, 8369,   64, 9676, 9418, 9288, 9156, 8891, 8759
     c, 8628, 8498, 8369, 8112, 7856, 7729, 7730, 7859, 7986, 7857
     c,   64, 7109, 9797,    0,  -16, 7116, 6732, 9419, 9290, 9417
     c, 9546, 9419, 9036, 8652, 8267, 8009, 7879, 7748, 7616, 7351
     c, 7219, 7089,   64, 8652, 8395, 8137, 8007, 7876, 7611, 7479
     c, 7348, 7218, 7089, 6832, 6576, 6449, 6450, 6579, 6706, 6577
     c,   64, 9157, 8894, 8762, 8760, 8887, 9271, 9529, 9659,   64
     c, 9285, 9022, 8890, 8888, 9015,   64, 7237, 9285,    0,  -16
     c, 7116, 6732, 9163, 9034, 9161, 9290, 9291, 9036,   64, 9548
     c, 8652, 8267, 8009, 7879, 7748, 7616, 7351, 7219, 7089,   64
     c, 8652, 8395, 8137, 8007, 7876, 7611, 7479, 7348, 7218, 7089
     c, 6832, 6576, 6449, 6450, 6579, 6706, 6577,   64, 9420, 8894
     c, 8762, 8760, 8887, 9271, 9529, 9659,   64, 9548, 9022, 8890
     c, 8888, 9015,   64, 7237, 9157,    0,  -16, 7116, 5969, 8523
     c, 8394, 8521, 8650, 8523, 8268, 7884, 7499, 7241, 7111, 6980
     c, 6848, 6583, 6451, 6321,   64, 7884, 7627, 7369, 7239, 7108
     c, 6843, 6711, 6580, 6450, 6321, 6064, 5808, 5681, 5682, 5811
     c, 5938, 5809,   64,10059, 9930,10057,10186,10059, 9676, 9292
     c, 8907, 8649, 8519, 8388, 8256, 7991, 7859, 7729,   64, 9292
     c, 9035, 8777, 8647, 8516, 8251, 8119, 7988, 7858, 7729, 7472
     c, 7216, 7089, 7090, 7219, 7346, 7217,   64, 9797, 9534, 9402
     c, 9400, 9527, 9911,10169,10299,   64, 9925, 9662, 9530, 9528
     c, 9655,   64, 6469, 9925,    0,  -16, 7116, 5969, 8523, 8394
     c, 8521, 8650, 8523, 8268, 7884, 7499, 7241, 7111, 6980, 6848
     c, 6583, 6451, 6321,   64, 7884, 7627, 7369, 7239, 7108, 6843
     c, 6711, 6580, 6450, 6321, 6064, 5808, 5681, 5682, 5811, 5938
     c, 5809,   64, 9803, 9674, 9801, 9930, 9931, 9676,   64,10188
     c, 9292, 8907, 8649, 8519, 8388, 8256, 7991, 7859, 7729,   64
     c, 9292, 9035, 8777, 8647, 8516, 8251, 8119, 7988, 7858, 7729
     c, 7472, 7216, 7089, 7090, 7219, 7346, 7217,   64,10060, 9534
     c, 9402, 9400, 9527, 9911,10169,10299,   64,10188, 9662, 9530
     c, 9528, 9655,   64, 6469, 9797,    0,  -16, 7116, 7495, 7617
     c, 7747, 8005, 8389, 8516, 8513, 8251, 8248, 8375,   64, 8261
     c, 8388, 8385, 8123, 8120, 8247, 8631, 8889, 9019,    0,  -16
     c, 7116, 8256,   64,    0,  -16, 7116, 7748,   64,    0,  -16
     c, 7116, 7240,   64,    0,  -16, 7116, 6986, 8140, 7755, 7496
     c, 7363, 7360, 7483, 7736, 8119, 8375, 8760, 9019, 9152, 9155
     c, 9032, 8779, 8396, 8140,   64, 8140, 7883, 7754, 7624, 7491
     c, 7488, 7611, 7737, 7864, 8119,   64, 8375, 8632, 8761, 8891
     c, 9024, 9027, 8904, 8778, 8651, 8396,    0,  -16, 7116, 6986
     c, 7752, 8009, 8396, 8375,   64, 8267, 8247,   64, 7735, 8887
     c,    0,  -16, 7116, 6986, 7496, 7623, 7494, 7367, 7368, 7498
     c, 7627, 8012, 8524, 8907, 9034, 9160, 9158, 9028, 8642, 8000
     c, 7743, 7485, 7354, 7351,   64, 8524, 8779, 8906, 9032, 9030
     c, 8900, 8514, 8000,   64, 7353, 7482, 7738, 8376, 8760, 9017
     c, 9146,   64, 7738, 8375, 8887, 9016, 9146, 9148,    0,  -16
     c, 7116, 6986, 7496, 7623, 7494, 7367, 7368, 7498, 7627, 8012
     c, 8524, 8907, 9033, 9030, 8900, 8515, 8131,   64, 8524, 8779
     c, 8905, 8902, 8772, 8515,   64, 8515, 8770, 9024, 9150, 9147
     c, 9017, 8888, 8503, 7991, 7608, 7481, 7355, 7356, 7485, 7612
     c, 7483,   64, 8897, 9022, 9019, 8889, 8760, 8503,    0,  -16
     c, 7116, 6986, 8522, 8503,   64, 8652, 8631,   64, 8652, 7229
     c, 9277,   64, 8119, 9015,    0,  -16, 7116, 6986, 7628, 7362
     c,   64, 7362, 7620, 8005, 8389, 8772, 9026, 9151, 9149, 9018
     c, 8760, 8375, 7991, 7608, 7481, 7355, 7356, 7485, 7612, 7483
     c,   64, 8389, 8644, 8898, 9023, 9021, 8890, 8632, 8375,   64
     c, 7628, 8908,   64, 7627, 8267, 8908,    0,  -16, 7116, 6986
     c, 8905, 8776, 8903, 9032, 9033, 8907, 8652, 8268, 7883, 7625
     c, 7495, 7363, 7357, 7482, 7736, 8119, 8375, 8760, 9018, 9149
     c, 9150, 9025, 8771, 8388, 8260, 7875, 7617, 7486,   64, 8268
     c, 8011, 7753, 7623, 7491, 7485, 7610, 7864, 8119,   64, 8375
     c, 8632, 8890, 9021, 9022, 8897, 8643, 8388,    0,  -16, 7116
     c, 6986, 7372, 7366,   64, 7368, 7498, 7756, 8012, 8649, 8905
     c, 9034, 9164,   64, 7498, 7755, 8011, 8649,   64, 9164, 9161
     c, 9030, 8513, 8383, 8252, 8247,   64, 9030, 8385, 8255, 8124
     c, 8119,    0,  -16, 7116, 6986, 8012, 7627, 7497, 7494, 7620
     c, 8003, 8515, 8900, 9030, 9033, 8907, 8524, 8012,   64, 8012
     c, 7755, 7625, 7622, 7748, 8003,   64, 8515, 8772, 8902, 8905
     c, 8779, 8524,   64, 8003, 7618, 7489, 7359, 7355, 7481, 7608
     c, 7991, 8503, 8888, 9017, 9147, 9151, 9025, 8898, 8515,   64
     c, 8003, 7746, 7617, 7487, 7483, 7609, 7736, 7991,   64, 8503
     c, 8760, 8889, 9019, 9023, 8897, 8770, 8515,    0,  -16, 7116
     c, 6986, 9029, 8898, 8640, 8255, 8127, 7744, 7490, 7365, 7366
     c, 7497, 7755, 8140, 8396, 8779, 9033, 9158, 9152, 9020, 8890
     c, 8632, 8247, 7863, 7608, 7482, 7483, 7612, 7739, 7610,   64
     c, 8127, 7872, 7618, 7493, 7494, 7625, 7883, 8140,   64, 8396
     c, 8651, 8905, 9030, 9024, 8892, 8762, 8504, 8247,    0,  -16
     c, 7116, 7621, 8249, 8120, 8247, 8376, 8249,    0,  -16, 7116
     c, 7621, 8247, 8120, 8249, 8376, 8374, 8244, 8115,    0,  -16
     c, 7116, 7621, 8261, 8132, 8259, 8388, 8261,   64, 8249, 8120
     c, 8247, 8376, 8249,    0,  -16, 7116, 7621, 8261, 8132, 8259
     c, 8388, 8261,   64, 8247, 8120, 8249, 8376, 8374, 8244, 8115
     c,    0,  -16, 7116, 7621, 8268, 8138, 8254, 8394, 8268,   64
     c, 8266, 8260,   64, 8249, 8120, 8247, 8376, 8249,    0,  -16
     c, 7116, 7113, 7624, 7751, 7622, 7495, 7496, 7626, 7755, 8012
     c, 8396, 8779, 8906, 9032, 9030, 8900, 8771, 8257, 8254,   64
     c, 8396, 8651, 8778, 8904, 8902, 8772, 8514,   64, 8249, 8120
     c, 8247, 8376, 8249,    0,  -16, 7116, 7748, 8268, 8133,   64
     c, 8396, 8133,    0,  -16, 7116, 7240, 7756, 7621,   64, 7884
     c, 7621,   64, 8780, 8645,   64, 8908, 8645,    0,  -16, 7116
     c, 7367, 8140, 7883, 7753, 7751, 7877, 8132, 8388, 8645, 8775
     c, 8777, 8651, 8396, 8140,    0,  -16, 7116, 7240, 8268, 8256
     c,   64, 7625, 8899,   64, 8905, 7619,    0,  -16, 7116, 6859
     c, 9424, 7088,    0,  -16, 7116, 7367, 8784, 8526, 8267, 8007
     c, 7874, 7870, 7993, 8245, 8498, 8752,   64, 8526, 8266, 8135
     c, 8002, 7998, 8121, 8246, 8498,    0,  -16, 7116, 7367, 7760
     c, 8014, 8267, 8519, 8642, 8638, 8505, 8245, 7986, 7728,   64
     c, 8014, 8266, 8391, 8514, 8510, 8377, 8246, 7986,    0,  -16
     c, 7116, 7367, 7888, 7856,   64, 8016, 7984,   64, 7888, 8784
     c,   64, 7856, 8752,    0,  -16, 7116, 7367, 8528, 8496,   64
     c, 8656, 8624,   64, 7760, 8656,   64, 7728, 8624,    0,  -16
     c, 7116, 7239, 8528, 8141, 8010, 8008, 8133, 8387, 8386, 7872
     c, 8382, 8381, 8123, 7992, 7990, 8115, 8496,   64, 8270, 8139
     c, 8135, 8260,   64, 8252, 8121, 8117, 8242,    0,  -16, 7116
     c, 7368, 8016, 8397, 8522, 8520, 8389, 8131, 8130, 8640, 8126
     c, 8125, 8379, 8504, 8502, 8371, 7984,   64, 8270, 8395, 8391
     c, 8260,   64, 8252, 8377, 8373, 8242,    0,  -16, 7116, 7367
     c, 8656, 7744, 8624,    0,  -16, 7116, 7367, 7888, 8768, 7856
     c,    0,  -16, 7116, 7748, 8272, 8240,    0,  -16, 7116, 7367
     c, 7888, 7856,   64, 8656, 8624,    0,  -16, 7116, 6605, 7104
     c, 9408,    0,  -16, 7116, 6605, 8265, 8247,   64, 7104, 9408
     c,    0,  -16, 7116, 6732, 8264, 8247,   64, 7232, 9280,   64
     c, 7223, 9271,    0,  -16, 7116, 6732, 8264, 8247,   64, 7240
     c, 9288,   64, 7232, 9280,    0,  -16, 7116, 6859, 7367, 9145
     c,   64, 9159, 7353,    0,  -16, 7116, 7621, 8257, 8128, 8255
     c, 8384, 8257,    0,  -16, 7116, 6605, 8265, 8136, 8263, 8392
     c, 8265,   64, 7104, 9408,   64, 8249, 8120, 8247, 8376, 8249
     c,    0,  -16, 7116, 6605, 7107, 9411,   64, 7101, 9405,    0
     c,  -16, 7116, 6605, 9161, 7351,   64, 7107, 9411,   64, 7101
     c, 9405,    0,  -16, 7116, 6605, 7109, 9413,   64, 7104, 9408
     c,   64, 7099, 9403,    0,  -16, 7116, 6732, 9289, 7232, 9271
     c,    0,  -16, 7116, 6732, 7241, 9280, 7223,    0,  -16, 7116
     c, 6732, 9292, 7237, 9278,   64, 7228, 9276,   64, 7223, 9271
     c,    0,  -16, 7116, 6732, 7244, 9285, 7230,   64, 7228, 9276
     c,   64, 7223, 9271,    0,  -16, 7116, 6733, 9403, 9147, 8892
     c, 8638, 8258, 8131, 7876, 7620, 7363, 7233, 7231, 7357, 7612
     c, 7868, 8125, 8254, 8642, 8900, 9157, 9413,    0,  -16, 7116
     c, 6732, 7101, 7103, 7234, 7491, 7747, 8002, 8511, 8766, 9022
     c, 9279, 9409,   64, 7103, 7233, 7490, 7746, 8001, 8510, 8765
     c, 9021, 9278, 9409, 9411,    0,  -16, 7116, 6859, 7230, 8259
     c, 9278,   64, 7230, 8258, 9278,    0,  -16, 7116, 7494, 8524
     c, 7878,   64, 8524, 8651, 7878,    0,  -16, 7116, 7494, 8012
     c, 8646,   64, 8012, 7883, 8646,    0,  -16, 7116, 6986, 7372
     c, 7498, 7752, 8135, 8391, 8776, 9034, 9164,   64, 7372, 7497
     c, 7751, 8134, 8390, 8775, 9033, 9164,    0,  -16, 7116, 7621
     c, 8266, 8139, 8268, 8395, 8393, 8263, 8134,    0,  -16, 7116
     c, 7621, 8396, 8267, 8137, 8135, 8262, 8391, 8264,    0,  -16
     c, 7116, 7621, 8266, 8395, 8268, 8139, 8137, 8263, 8390,    0
     c,  -16, 7116, 7621, 8140, 8267, 8393, 8391, 8262, 8135, 8264
     c,    0,  -16, 7116, 6601, 6981, 7493, 8249,   64, 7365, 8247
     c,   64, 9424, 8247,    0,  -16, 7116, 6732, 9288, 8392, 7879
     c, 7622, 7364, 7233, 7231, 7356, 7610, 7865, 8376, 9272,    0
     c,  -16, 7116, 6732, 7240, 7233, 7357, 7483, 7737, 8120, 8376
     c, 8761, 9019, 9149, 9281, 9288,    0,  -16, 7116, 6732, 7240
     c, 8136, 8647, 8902, 9156, 9281, 9279, 9148, 8890, 8633, 8120
     c, 7224,    0,  -16, 7116, 6732, 7224, 7231, 7363, 7493, 7751
     c, 8136, 8392, 8775, 9029, 9155, 9279, 9272,    0,  -16, 7116
     c, 6732, 9288, 8392, 7879, 7622, 7364, 7233, 7231, 7356, 7610
     c, 7865, 8376, 9272,   64, 7232, 8768,    0,  -16, 7116, 6605
     c, 9026, 9408, 9022,   64, 8645, 9280, 8635,   64, 7104, 9280
     c,    0,  -16, 7116, 7240, 8006, 8265, 8518,   64, 7619, 8264
     c, 8899,   64, 8264, 8247,    0,  -16, 7116, 6605, 7490, 7104
     c, 7486,   64, 7877, 7232, 7867,   64, 7232, 9408,    0,  -16
     c, 7116, 7240, 7994, 8247, 8506,   64, 7613, 8248, 8893,   64
     c, 8265, 8248,    0,  -16, 7116, 7114, 9024, 8899, 8772, 8517
     c, 8261, 7876, 7617, 7486, 7483, 7609, 7736, 7991, 8247, 8632
     c, 8890, 9021, 9154, 9159, 9034, 8907, 8652, 8268, 8011, 7882
     c, 7881, 8009, 8010,   64, 8261, 8004, 7745, 7614, 7610, 7736
     c,   64, 8247, 8504, 8762, 8893, 9026, 9031, 8906, 8652,    0
     c,  -16, 7116, 6986, 7244, 8247,   64, 7372, 8249,   64, 9292
     c, 8247,   64, 7244, 9292,   64, 7371, 9163,    0,  -16, 7116
     c, 6096, 6469, 7109, 8249,   64, 6980, 8247,   64,10328, 8247
     c,    0,  -16, 7116, 6732, 9423, 9294, 9421, 9550, 9551, 9424
     c, 9168, 8911, 8653, 8523, 8392, 8260, 7992, 7860, 7730,   64
     c, 8782, 8652, 8520, 8252, 8120, 7989, 7859, 7601, 7344, 7088
     c, 6961, 6962, 7091, 7218, 7089,    0,  -16, 7116, 6732, 9423
     c, 9294, 9421, 9550, 9551, 9424, 9168, 8911, 8653, 8523, 8392
     c, 8260, 7992, 7860, 7730,   64, 8782, 8652, 8520, 8252, 8120
     c, 7989, 7859, 7601, 7344, 7088, 6961, 6962, 7091, 7218, 7089
     c,   64, 8135, 7750, 7492, 7361, 7359, 7484, 7738, 8121, 8377
     c, 8762, 9020, 9151, 9153, 9028, 8774, 8391, 8135,    0,  -16
     c, 7116, 6733, 9535, 9405, 9148, 8892, 8637, 8510, 8130, 8003
     c, 7748, 7492, 7235, 7105, 7103, 7229, 7484, 7740, 7997, 8126
     c, 8514, 8643, 8900, 9156, 9411, 9537, 9535,    0,  -16, 7116
     c, 6732, 9420, 7095,   64, 7756, 8010, 8008, 7878, 7621, 7365
     c, 7111, 7113, 7243, 7500, 7756, 8011, 8394, 8778, 9163, 9420
     c,   64, 8894, 8637, 8507, 8505, 8759, 9015, 9272, 9402, 9404
     c, 9150, 8894,    0,  -16, 7116, 6733, 9412, 9283, 9410, 9539
     c, 9540, 9413, 9285, 9156, 9026, 8765, 8506, 8248, 7991, 7607
     c, 7224, 7098, 7101, 7231, 8003, 8261, 8391, 8393, 8267, 8012
     c, 7755, 7625, 7623, 7748, 8001, 8634, 8888, 9271, 9399, 9528
     c, 9529,   64, 7607, 7352, 7226, 7229, 7359, 7617,   64, 7623
     c, 7749, 8762, 9016, 9271,    0,  -16, 7116, 6606, 8900, 8774
     c, 8519, 8135, 7878, 7749, 7618, 7615, 7741, 7996, 8380, 8637
     c, 8767,   64, 8135, 7877, 7746, 7743, 7869, 7996,   64, 8903
     c, 8767, 8765, 9020, 9276, 9534, 9665, 9667, 9542, 9416, 9162
     c, 8907, 8524, 8140, 7755, 7498, 7240, 7110, 6979, 6976, 7101
     c, 7227, 7481, 7736, 8119, 8503, 8888, 9145, 9274,   64, 9031
     c, 8895, 8893, 9020,    0,  -16, 7116, 6986, 8016, 7987,   64
     c, 8528, 8499,   64, 9033, 8904, 9031, 9160, 9161, 8907, 8524
     c, 8012, 7627, 7369, 7367, 7493, 7620, 7875, 8641, 8896, 9150
     c,   64, 7367, 7621, 7876, 8642, 8897, 9024, 9150, 9146, 8888
     c, 8503, 7991, 7608, 7354, 7355, 7484, 7611, 7482,    0,  -16
     c, 7116, 6987, 8396, 7472,   64, 9164, 8240,   64, 7489, 9281
     c,   64, 7355, 9147,    0,  -16, 7116, 7240, 8649, 8520, 8647
     c, 8776, 8777, 8651, 8396, 8140, 7883, 7753, 7751, 7877, 8131
     c, 8768,   64, 7877, 8514, 8768, 8894, 8892, 8762, 8504,   64
     c, 8004, 7746, 7616, 7614, 7740, 7994, 8631,   64, 7740, 8377
     c, 8631, 8757, 8755, 8625, 8368, 8112, 7857, 7731, 7732, 7861
     c, 7988, 7859,    0,  -16, 7116, 7240, 8268, 8138, 8264, 8394
     c, 8268,   64, 8268, 8240,   64, 8257, 8126, 8240, 8382, 8257
     c,   64, 7493, 7748, 8005, 7750, 7493,   64, 7493, 9029,   64
     c, 8517, 8772, 9029, 8774, 8517,    0,  -16, 7116, 7240, 8268
     c, 8138, 8264, 8394, 8268,   64, 8268, 8254,   64, 8258, 8128
     c, 8380, 8250, 8124, 8384, 8258,   64, 8254, 8240,   64, 8244
     c, 8114, 8240, 8370, 8244,   64, 7493, 7748, 8005, 7750, 7493
     c,   64, 7493, 9029,   64, 8517, 8772, 9029, 8774, 8517,   64
     c, 7479, 7734, 7991, 7736, 7479,   64, 7479, 9015,   64, 8503
     c, 8758, 9015, 8760, 8503,    0,  -16, 7116, 7114, 9036, 9015
     c,   64, 7372, 9036,   64, 8002, 9026,   64, 7351, 9015,    0
     c,  -16, 7116, 6606, 8140, 7755, 7369, 7110, 6979, 6976, 7101
     c, 7354, 7736, 8119, 8503, 8888, 9274, 9533, 9664, 9667, 9542
     c, 9289, 8907, 8524, 8140,   64, 8259, 8130, 8129, 8256, 8384
     c, 8513, 8514, 8387, 8259,   64, 8258, 8257, 8385, 8386, 8258
     c,    0,  -16, 7116, 7241, 8012, 7755, 7881, 8136,   64, 8012
     c, 7883, 7881,   64, 8652, 8907, 8777, 8520,   64, 8652, 8779
     c, 8777,   64, 8136, 7879, 7750, 7620, 7617, 7743, 7870, 8125
     c, 8509, 8766, 8895, 9025, 9028, 8902, 8775, 8520, 8136,   64
     c, 8253, 8247,   64, 8381, 8375,   64, 7738, 8890,    0,  -16
     c, 7116, 7114, 8268, 7883, 7625, 7494, 7493, 7618, 7872, 8255
     c, 8383, 8768, 9026, 9157, 9158, 9033, 8779, 8396, 8268,   64
     c, 8255, 8247,   64, 8383, 8375,   64, 7739, 8891,    0,  -16
     c, 7116, 6478, 8012, 7627, 7241, 6982, 6851, 6847, 6972, 7225
     c, 7607, 7990, 8502, 8887, 9273, 9532, 9663, 9667, 9542, 9289
     c, 8907, 8524, 8012,   64, 8268, 8246,   64, 6849, 9665,    0
     c,  -16, 7116, 6862, 8005, 7620, 7362, 7231, 7230, 7355, 7609
     c, 7992, 8120, 8505, 8763, 8894, 8895, 8770, 8516, 8133, 8005
     c,   64, 9675, 8907, 9418, 8644,   64, 9675, 9669, 9545, 8771
     c,   64, 9546, 8772,    0,  -16, 7116, 6730, 7113, 7243, 7500
     c, 7884, 8139, 8265, 8262, 8131, 8001, 7743, 7357,   64, 7884
     c, 8011, 8137, 8133, 8002, 7743,   64, 8780, 8503,   64, 8908
     c, 8375,   64, 7357, 9149,    0,  -16, 7116, 7114, 7628, 7613
     c,   64, 7756, 7617,   64, 7617, 7747, 7876, 8133, 8517, 8900
     c, 9026, 9024, 8894, 8636,   64, 8517, 8772, 8898, 8896, 8506
     c, 8504, 8631, 8887, 9145,   64, 7372, 7756,    0,  -16, 7116
     c, 7114, 8260, 7875, 7617, 7486, 7485, 7610, 7864, 8247, 8375
     c, 8760, 9018, 9149, 9150, 9025, 8771, 8388, 8260,   64, 8266
     c, 7752, 8268, 8260,   64, 8394, 8904, 8396, 8388,   64, 8255
     c, 8126, 8125, 8252, 8380, 8509, 8510, 8383, 8255,   64, 8254
     c, 8253, 8381, 8382, 8254,    0,  -16, 7116, 6860, 8138, 8268
     c, 8247,   64, 8522, 8396, 8375,   64, 7242, 7372, 7365, 7490
     c, 7744, 8127, 8255,   64, 7626, 7500, 7492, 7617,   64, 9418
     c, 9292, 9285, 9154, 8896, 8511, 8383,   64, 9034, 9164, 9156
     c, 9025,   64, 7739, 8891,    0,  -16, 7116, 6859, 7500, 7479
     c,   64, 7628, 7607,   64, 7116, 8652, 9035, 9162, 9288, 9285
     c, 9155, 9026, 8641, 7617,   64, 8652, 8907, 9034, 9160, 9157
     c, 9027, 8898, 8641,   64, 7095, 9143, 9148, 9015,    0,  -16
     c, 7116, 6985, 9163, 8651, 8138, 7752, 7493, 7362, 7359, 7484
     c, 7737, 8119, 8630, 9142,   64, 9163, 8778, 8392, 8133, 8002
     c, 7999, 8124, 8377, 8759, 9142,    0,  -16, 7116, 6733, 7873
     c, 7617, 7360, 7231, 7101, 7099, 7225, 7352, 7607, 7863, 8120
     c, 8249, 8379, 8381, 8255, 8128, 7873,   64, 8394, 8001,   64
     c, 9288, 8256,   64, 9537, 8382,    0,  -16, 7116, 6986, 7879
     c, 8633,   64, 8647, 7865,   64, 7363, 9149,   64, 9155, 7357
     c,    0,  -16, 7116, 6732, 7740, 7485, 7357, 7100, 6970, 6969
     c, 7095, 7350, 7478, 7735, 7865, 7866, 7740, 7360, 7235, 7237
     c, 7368, 7626, 8011, 8523, 8906, 9160, 9285, 9283, 9152, 8764
     c, 8634, 8633, 8759, 9014, 9142, 9399, 9529, 9530, 9404, 9149
     c, 9021, 8764,   64, 7237, 7367, 7625, 8010, 8522, 8905, 9159
     c, 9285,    0,  -16, 7116, 6732, 7749, 7492, 7364, 7109, 6983
     c, 6984, 7114, 7371, 7499, 7754, 7880, 7879, 7749, 7361, 7230
     c, 7228, 7353, 7607, 7990, 8502, 8887, 9145, 9276, 9278, 9153
     c, 8773, 8647, 8648, 8778, 9035, 9163, 9418, 9544, 9543, 9413
     c, 9156, 9028, 8773,   64, 7228, 7354, 7608, 7991, 8503, 8888
     c, 9146, 9276,    0,  -16, 7116, 7113, 7353, 9159,   64, 8134
     c, 8646, 9159, 9027, 9023,   64, 8646, 9030, 9027,    0,  -16
     c, 7116, 7113, 9159, 7353,   64, 7489, 7485, 7353, 7866, 8378
     c,   64, 7485, 7482, 7866,    0,  -16, 7116, 7113, 9145, 7367
     c,   64, 8390, 7878, 7367, 7491, 7487,   64, 7878, 7494, 7491
     c,    0,  -16, 7116, 7113, 7367, 9145,   64, 8122, 8634, 9145
     c, 9021, 9025,   64, 8634, 9018, 9021,    0,  -16, 7116, 6733
     c, 7237, 7110, 7112, 7242, 7499, 7755, 8010, 8137, 8263, 8386
     c,   64, 7112, 7370, 7626, 7881, 8008, 8134, 8258, 8247,   64
     c, 9413, 9542, 9544, 9418, 9163, 8907, 8650, 8521, 8391, 8258
     c,   64, 9544, 9290, 9034, 8777, 8648, 8518, 8386, 8375,    0
     c,  -16, 7116, 6732, 7115, 7239, 7365, 7619, 8002, 8514, 8899
     c, 9157, 9287, 9419,   64, 7115, 7240, 7366, 7620, 8003, 8515
     c, 8900, 9158, 9288, 9419,   64, 8003, 7746, 7617, 7487, 7484
     c, 7610, 7864, 8119, 8375, 8632, 8890, 9020, 9023, 8897, 8770
     c, 8515,   64, 8002, 7745, 7615, 7612, 7737,   64, 8761, 8892
     c, 8895, 8769, 8514,    0,  -16, 7116, 6732, 7624, 7612,   64
     c, 7751, 7741,   64, 8775, 8765,   64, 8904, 8892,   64, 7115
     c, 7369, 7624, 8007, 8519, 8904, 9161, 9419,   64, 7097, 7355
     c, 7612, 7997, 8509, 8892, 9147, 9401,    0,  -16, 7116, 6732
     c, 9417, 7497, 7240, 7110, 7108, 7234, 7489, 7745, 8002, 8132
     c, 8134, 8008, 9416,   64, 7109, 7235, 7362, 7617,   64, 8133
     c, 8007, 7880, 7625,   64, 7098, 9018, 9275, 9405, 9407, 9281
     c, 9026, 8770, 8513, 8383, 8381, 8507, 7099,   64, 9406, 9280
     c, 9153, 8898,   64, 8382, 8508, 8635, 8890,    0,  -16, 7116
     c, 6731, 7869, 7614, 7486, 7229, 7099, 7098, 7224, 7479, 7607
     c, 7864, 7994, 7995, 7869, 7234, 7108, 7111, 7241, 7498, 7883
     c, 8395, 8906, 9160, 9286, 9283, 9152, 8765, 8635, 8633, 8759
     c, 9015, 9144, 9274,   64, 7615, 7362, 7236, 7239, 7369, 7498
     c,   64, 8395, 8778, 9032, 9158, 9155, 9024, 8765,    0,  -16
     c, 7116, 6861, 6983, 7370, 7623, 7612,   64, 7241, 7494, 7484
     c,   64, 7623, 8010, 8263, 8253,   64, 7881, 8134, 8125,   64
     c, 8263, 8650, 8903, 8887,   64, 8521, 8774, 8759,   64, 8903
     c, 9290, 9416, 9541, 9538, 9407, 9277, 9019, 8633, 7991,   64
     c, 9161, 9288, 9413, 9410, 9279, 9149, 8891, 8505, 7991,    0
     c,  -16, 7116, 6732, 7103, 7615, 7488, 7363, 7365, 7496, 7754
     c, 8139, 8395, 8778, 9032, 9157, 9155, 9024, 8895, 9407,   64
     c, 7102, 7870, 7616, 7491, 7493, 7624, 7882, 8139,   64, 8395
     c, 8650, 8904, 9029, 9027, 8896, 8638, 9406,   64, 7354, 9146
     c,   64, 7353, 9145,    0,  -16, 7116, 6733, 6854, 7241, 7622
     c, 7610,   64, 7112, 7493, 7482,   64, 7622, 8009, 8390, 8378
     c,   64, 7880, 8261, 8250,   64, 8390, 8777, 9158, 9147, 9401
     c,   64, 8648, 9029, 9018, 9272, 9659,    0,  -16, 7116, 6859
     c, 9289, 7225,   64, 9289, 8904, 8136,   64, 9031, 8647, 8136
     c,   64, 9289, 9158, 9152,   64, 9031, 9028, 9152,   64, 8128
     c, 7232,   64, 7999, 7615, 7232,   64, 8128, 8121,   64, 7999
     c, 7996, 8121,    0,  -16, 7116, 6732, 6979, 7239, 7869,   64
     c, 7237, 7867, 8258, 8898, 9283, 9413, 9415, 9289, 9034, 8906
     c, 8649, 8519, 8517, 8642, 8768, 8893, 8890, 8632,   64, 8906
     c, 8777, 8647, 8645, 8897, 9022, 9019, 8889, 8632,    0,  -16
     c, 7116, 6732, 7107, 7494, 8004,   64, 7365, 7875, 8262, 8644
     c,   64, 8133, 8515, 8902, 9156,   64, 8773, 9027, 9414,   64
     c, 7101, 7488, 7998,   64, 7359, 7869, 8256, 8638,   64, 8127
     c, 8509, 8896, 9150,   64, 8767, 9021, 9408,    0,  -16, 7116
     c, 6732, 7242, 7752, 8006, 8131, 8128, 7997, 7739, 7225,   64
     c, 7242, 7625, 7880, 8134, 8259,   64, 8256, 8125, 7867, 7610
     c, 7225,   64, 9290, 8905, 8648, 8390, 8259,   64, 8256, 8381
     c, 8635, 8890, 9273,   64, 9290, 8776, 8518, 8387, 8384, 8509
     c, 8763, 9273,   64, 7106, 9410,   64, 7105, 9409,    0,  -16
     c, 7116, 7748, 8129, 8127, 8383, 8385, 8129,   64, 8129, 8383
     c,   64, 8385, 8127,    0,  -16, 7116, 7240, 7618, 8128, 8510
     c, 8764, 8889, 8887, 8757, 8628,   64, 7617, 8382,   64, 7616
     c, 7999, 8509, 8763, 8889,    0,  -16, 7116, 7240, 8903, 8773
     c, 8515, 8001, 7616,   64, 8386, 7615,   64, 8652, 8779, 8905
     c, 8903, 8772, 8514, 8128, 7614,    0,  -16, 7116, 6986, 8389
     c, 7876, 7490, 7360, 7358, 7484, 7739, 8123, 8636, 9022, 9152
     c, 9154, 9028, 8773, 8389,   64, 9028, 8389,   64, 8773, 8132
     c, 7490,   64, 7876, 7360,   64, 7484, 8123,   64, 7739, 8380
     c, 9022,   64, 8636, 9152,    0,  -16, 7116, 6986, 8389, 7876
     c, 7490, 7360, 7358, 7484, 7739, 8123, 8636, 9022, 9152, 9154
     c, 9028, 8773, 8389,   64, 9028, 8389,   64, 8773, 8132, 7490
     c,   64, 7876, 7360,   64, 7484, 8123,   64, 7739, 8380, 9022
     c,   64, 8636, 9152,    0,  -16, 7116, 7241, 8389, 8004, 7746
     c, 7616, 7614, 7740, 7995, 8251, 8636, 8894, 9024, 9026, 8900
     c, 8645, 8389,   64, 7874, 8645,   64, 7744, 8772,   64, 7614
     c, 8899,   64, 7741, 9026,   64, 7868, 8896,   64, 7995, 8766
     c,    0,  -16, 7116, 7240, 7883, 7860,   64, 8652, 8629,   64
     c, 7620, 8902,   64, 7619, 8901,   64, 7611, 8893,   64, 7610
     c, 8892,    0,  -16, 7116, 7240, 7756, 7738,   64, 8774, 8756
     c,   64, 7748, 8774,   64, 7747, 8773,   64, 7739, 8765,   64
     c, 7738, 8764,    0,  -16, 7116, 7240, 7760, 7739,   64, 7748
     c, 8134, 8518, 8773, 8899, 8897, 8767, 8381, 8124, 7739,   64
     c, 7748, 8133, 8517, 8772,   64, 8645, 8771, 8769, 8639, 8381
     c,    0,  -16, 7116, 6605, 6985, 6982,   64, 9545, 9542,   64
     c, 6985, 9545,   64, 6984, 9544,   64, 6983, 9543,   64, 6982
     c, 9542,    0,  -16, 7116, 7240, 7620, 7617,   64, 8900, 8897
     c,   64, 7620, 8900,   64, 7619, 8899,   64, 7618, 8898,   64
     c, 7617, 8897,    0,  -16, 7116, 7240, 7622, 8890,   64, 7622
     c, 7876, 8131, 8515, 8772, 8901, 8903, 8647, 8645, 8515,   64
     c, 7876, 8515,   64, 8131, 8901,   64, 8775, 8772,   64, 8646
     c, 8902,   64, 8890, 8636, 8381, 7997, 7740, 7611, 7609, 7865
     c, 7867, 7997,   64, 8636, 7997,   64, 8381, 7611,   64, 7740
     c, 7737,   64, 7610, 7866,    0,  -16, 7116, 7240, 8003, 7877
     c, 7879, 7623, 7621, 7748, 8003, 8387, 8644, 8902,   64, 7751
     c, 7748,   64, 7622, 7878,   64, 7621, 8387,   64, 8003, 8644
     c,   64, 8902, 8889,    0,  -16, 7116, 6092, 6828, 6956, 7085
     c, 7086, 6959, 6831, 6702, 6700, 6826, 7081, 7337, 7722, 7980
     c, 8110, 8242, 8253, 8151, 8158, 8291, 8421, 8678, 8806, 9061
     c, 9187, 9183, 9052, 8922, 8663, 8019, 7247, 6989, 6730, 6600
     c, 6468, 6464, 6588, 6841, 7223, 7734, 8246, 8759, 9016, 9275
     c, 9406, 9410, 9285, 9159, 8905, 8522, 8010, 7625, 7367, 7236
     c, 7232, 7357, 7611,   64, 6830, 6829, 6957, 6958, 6830,   64
     c, 8663, 8147, 7503, 7116, 6857, 6727, 6596, 6592, 6716, 6842
     c, 7223,   64, 8246, 8631, 8888, 9147, 9278, 9282, 9157, 9031
     c, 8777, 8522,    0,  -16, 7116, 6612, 7743, 7869, 8124, 8380
     c, 8637, 8767, 8769, 8643, 8388, 8132, 7875, 7746, 7615, 7612
     c, 7737, 7991, 8374, 8758, 9143, 9401, 9531, 9662, 9666, 9541
     c, 9288, 9033, 8650, 8266, 7881, 7624, 7366, 7107, 6975, 6970
     c, 7093, 7345, 7599, 7981, 8492, 9132, 9645,10031,10289,   64
     c, 7366, 7236, 7104, 7098, 7222, 7474, 7728, 8110, 8621, 9133
     c, 9646, 9903,10289,   64, 8003, 8515,   64, 7874, 8642,   64
     c, 7745, 8769,   64, 7744, 8768,   64, 7743, 8767,   64, 7870
     c, 8638,   64, 7997, 8509,   64,10182,10180,10436,10438,10182
     c,   64,10310,10308,   64,10181,10437,   64,10172,10170,10426
     c,10428,10172,   64,10300,10298,   64,10171,10427,    0,  -16
     c, 7116, 6478, 6994, 6958,   64, 7634, 7598,   64, 8914, 8878
     c,   64, 9554, 9518,   64, 7621, 8903,   64, 7620, 8902,   64
     c, 7619, 8901,   64, 7611, 8893,   64, 7610, 8892,   64, 7609
     c, 8891,    0,  -16, 7116, 7748, 8129, 8127, 8383, 8385, 8129
     c,   64, 8129, 8383,   64, 8385, 8127,    0,  -16, 7116, 7240
     c, 7618, 8128, 8510, 8764, 8889, 8887, 8757, 8628,   64, 7617
     c, 8382,   64, 7616, 7999, 8509, 8763, 8889,    0,  -16, 7116
     c, 7240, 8903, 8773, 8515, 8001, 7616,   64, 8386, 7615,   64
     c, 8652, 8779, 8905, 8903, 8772, 8514, 8128, 7614,    0,  -16
     c, 7116, 6986, 8005, 7620, 7491, 7361, 7359, 7485, 7612, 7995
     c, 8507, 8892, 9021, 9151, 9153, 9027, 8900, 8517, 8005,   64
     c, 7620, 7490, 7487, 7613, 7740, 7995,   64, 8892, 9022, 9025
     c, 8899, 8772, 8517,    0,  -16, 7116, 6986, 8389, 7876, 7490
     c, 7360, 7358, 7484, 7739, 8123, 8636, 9022, 9152, 9154, 9028
     c, 8773, 8389,   64, 9028, 8389,   64, 8773, 8132, 7490,   64
     c, 7876, 7360,   64, 7484, 8123,   64, 7739, 8380, 9022,   64
     c, 8636, 9152,    0,  -16, 7116, 7241, 8389, 8004, 7746, 7616
     c, 7614, 7740, 7995, 8251, 8636, 8894, 9024, 9026, 8900, 8645
     c, 8389,   64, 7874, 8645,   64, 7744, 8772,   64, 7614, 8899
     c,   64, 7741, 9026,   64, 7868, 8896,   64, 7995, 8766,    0
     c,  -16, 7116, 7240, 7883, 7860,   64, 8652, 8629,   64, 7620
     c, 8902,   64, 7619, 8901,   64, 7611, 8893,   64, 7610, 8892
     c,    0,  -16, 7116, 7240, 7756, 7738,   64, 8774, 8756,   64
     c, 7748, 8774,   64, 7747, 8773,   64, 7739, 8765,   64, 7738
     c, 8764,    0,  -16, 7116, 7240, 7760, 7739,   64, 7748, 8134
     c, 8518, 8773, 8899, 8897, 8767, 8381, 8124, 7739,   64, 7748
     c, 8133, 8517, 8772,   64, 8645, 8771, 8769, 8639, 8381,    0
     c,  -16, 7116, 6605, 6985, 6982,   64, 9545, 9542,   64, 6985
     c, 9545,   64, 6984, 9544,   64, 6983, 9543,   64, 6982, 9542
     c,    0,  -16, 7116, 7240, 7620, 7617,   64, 8900, 8897,   64
     c, 7620, 8900,   64, 7619, 8899,   64, 7618, 8898,   64, 7617
     c, 8897,    0,  -16, 7116, 7240, 8143, 8773, 8254, 8253,   64
     c, 8646, 8127,   64, 8521, 8519, 8000, 8253, 8633,   64, 8886
     c, 8633, 8378, 8122, 7865, 7735, 7733, 7859, 8241,   64, 8886
     c, 8632, 8377, 7865, 7861, 7987, 8241,   64, 8378, 7992, 7733
     c,    0,  -16, 7116, 7240, 8003, 7877, 7879, 7623, 7621, 7748
     c, 8003, 8387, 8644, 8902,   64, 7751, 7748,   64, 7622, 7878
     c,   64, 7621, 8387,   64, 8003, 8644,   64, 8902, 8377,    0
     c,  -16, 7116, 6092, 6828, 6956, 7085, 7086, 6959, 6831, 6702
     c, 6700, 6826, 7081, 7337, 7722, 7980, 8110, 8242, 8253, 8151
     c, 8158, 8291, 8421, 8678, 8806, 9061, 9187, 9183, 9052, 8922
     c, 8663, 8019, 7247, 6989, 6730, 6600, 6468, 6464, 6588, 6841
     c, 7223, 7734, 8246, 8759, 9016, 9275, 9406, 9410, 9285, 9159
     c, 8905, 8522, 8010, 7625, 7367, 7236, 7232, 7357, 7611,   64
     c, 6830, 6829, 6957, 6958, 6830,   64, 8663, 8147, 7503, 7116
     c, 6857, 6727, 6596, 6592, 6716, 6842, 7223,   64, 8246, 8631
     c, 8888, 9147, 9278, 9282, 9157, 9031, 8777, 8522,    0,  -16
     c, 7116, 7128, 7745, 7875, 8132, 8388, 8643, 8769, 8767, 8637
     c, 8380, 8124, 7869, 7742, 7617, 7620, 7751, 8009, 8394, 8906
     c, 9417, 9799,10052,10176,10171,10039, 9909, 9650, 9263, 8748
     c, 8105, 7591,   64, 8906, 9289, 9671, 9924,10048,10043, 9911
     c, 9781, 9522, 9135, 8491, 8105,   64, 8003, 8515,   64, 7874
     c, 8642,   64, 7745, 8769,   64, 7744, 8768,   64, 7743, 8767
     c,   64, 7870, 8638,   64, 7997, 8509,   64,10694,10692,10948
     c,10950,10694,   64,10822,10820,   64,10693,10949,   64,10684
     c,10682,10938,10940,10684,   64,10812,10810,   64,10683,10939
     c,    0,  -16, 7116, 6478, 6996, 6956,   64, 7124, 7084,   64
     c, 7636, 7596,   64, 8144, 8400, 8398, 8142, 8145, 8275, 8532
     c, 8916, 9171, 9425, 9550, 9545, 9414, 9156, 8899, 8643, 8388
     c, 8262, 8132, 7873, 7744, 7871, 8124, 8250, 8380, 8637, 8893
     c, 9148, 9402, 9527, 9522, 9391, 9133, 8876, 8492, 8237, 8111
     c, 8114, 8370, 8368, 8112,   64, 8272, 8270,   64, 8143, 8399
     c,   64, 9171, 9297, 9422, 9417, 9286, 9156,   64, 8262, 8260
     c, 8001, 7744, 7999, 8252, 8250,   64, 9148, 9274, 9399, 9394
     c, 9263, 9133,   64, 8242, 8240,   64, 8113, 8369,    0,  -16
     c, 7116, 6097, 6992, 6960,   64, 7120, 7088,   64, 9424, 9392
     c,   64, 9552, 9520,   64, 6480,10064,   64, 6448, 7600,   64
     c, 8880,10032,    0,  -16, 7116, 6223, 6864, 8130, 6704,   64
     c, 6736, 8002,   64, 6608, 8001,   64, 6608, 9552, 9801, 9424
     c,   64, 6833, 9521,   64, 6704, 9520, 9783, 9392,    0,  -16
     c, 7116, 7113, 9063, 8673, 8282, 8021, 7889, 7756, 7620, 7612
     c, 7732, 7855, 7979, 8230, 8607, 8985,   64, 8673, 8412, 8150
     c, 8018, 7884, 7748, 7740, 7860, 7982, 8106, 8356, 8607,    0
     c,  -16, 7116, 7113, 7527, 7905, 8282, 8533, 8657, 8780, 8900
     c, 8892, 8756, 8623, 8491, 8230, 7839, 7449,   64, 7905, 8156
     c, 8406, 8530, 8652, 8772, 8764, 8628, 8494, 8362, 8100, 7839
     c,    0,  -16, 7116, 7113, 7655, 7616, 7577,   64, 7783, 7744
     c, 7705,   64, 7655, 9063,   64, 7577, 8985,    0,  -16, 7116
     c, 7113, 8807, 8768, 8729,   64, 8935, 8896, 8857,   64, 7527
     c, 8935,   64, 7449, 8857,    0,  -16, 7116, 6985, 8807, 8289
     c, 8029, 7896, 7892, 8016, 8520, 8518, 8388, 8259, 7616, 8253
     c, 8380, 8506, 8504, 7984, 7852, 7848, 7971, 8223, 8729,   64
     c, 8548, 8288, 8157, 8024, 8020, 8143, 8268,   64, 8244, 8113
     c, 7980, 7976, 8099, 8224, 8476,    0,  -16, 7116, 7114, 7783
     c, 8289, 8541, 8664, 8660, 8528, 8008, 8006, 8132, 8259, 8896
     c, 8253, 8124, 7994, 7992, 8496, 8620, 8616, 8483, 8223, 7705
     c,   64, 8036, 8288, 8413, 8536, 8532, 8399, 8268,   64, 8244
     c, 8369, 8492, 8488, 8355, 8224, 7964,    0,  -16, 7116, 7113
     c, 8804, 8417, 8158, 7898, 7765, 7759, 7881, 8005, 8378, 8502
     c, 8624, 8619, 8486, 8355, 8095,   64, 8417, 8157, 8026, 7893
     c, 7888, 8010, 8134, 8507, 8631, 8753, 8747, 8614, 8354, 8095
     c, 7708,    0,  -16, 7116, 7113, 7780, 8161, 8414, 8666, 8789
     c, 8783, 8649, 8517, 8122, 7990, 7856, 7851, 7974, 8099, 8351
     c,   64, 8161, 8413, 8538, 8661, 8656, 8522, 8390, 7995, 7863
     c, 7729, 7723, 7846, 8098, 8351, 8732,    0,  -16, 7116, 4808
     c, 5184, 6080, 8227,   64, 5952, 8099,   64, 5824, 8224,   64
     c, 9328, 8776, 8224,    0,  -16, 7116, 6351, 9700, 9572, 9443
     c, 9442, 9569, 9697, 9826, 9828, 9702, 9447, 9191, 8934, 8676
     c, 8546, 8415, 8280, 8136, 8104, 7967, 7836,   64, 9571, 9570
     c, 9698, 9699, 9571,   64, 8280, 8232,   64, 8676, 8545, 8408
     c, 8376, 8232, 8097, 7966, 7836, 7578, 7321, 7065, 6810, 6684
     c, 6686, 6815, 6943, 7070, 7069, 6940, 6812,   64, 6814, 6813
     c, 6941, 6942, 6814,    0,  -16, 7116, 6986, 8268, 7223,   64
     c, 8265, 7351, 7223,   64, 8265, 9143, 9271,   64, 8268, 9271
     c,   64, 7613, 8893,   64, 7484, 9020,    0,  -16, 7116, 6986
     c, 7500, 7479,   64, 7627, 7608,   64, 7500, 8524, 8907, 9034
     c, 9160, 9157, 9027, 8898, 8513,   64, 7627, 8523, 8906, 9032
     c, 9029, 8899, 8514,   64, 7618, 8514, 8897, 9024, 9150, 9147
     c, 9017, 8888, 8503, 7479,   64, 7617, 8513, 8896, 9022, 9019
     c, 8889, 8504, 7608,    0,  -16, 7116, 6987, 9287, 9161, 8907
     c, 8652, 8140, 7883, 7625, 7495, 7364, 7359, 7484, 7610, 7864
     c, 8119, 8631, 8888, 9146, 9276,   64, 9287, 9159, 9033, 8906
     c, 8651, 8139, 7882, 7623, 7492, 7487, 7612, 7865, 8120, 8632
     c, 8889, 9018, 9148, 9276,    0,  -16, 7116, 6987, 7500, 7479
     c,   64, 7627, 7608,   64, 7500, 8396, 8779, 9033, 9159, 9284
     c, 9279, 9148, 9018, 8760, 8375, 7479,   64, 7627, 8395, 8778
     c, 8905, 9031, 9156, 9151, 9020, 8890, 8761, 8376, 7608,    0
     c,  -16, 7116, 7114, 7628, 7607,   64, 7755, 7736,   64, 7628
     c, 9164,   64, 7755, 9163, 9164,   64, 7746, 8514, 8513,   64
     c, 7745, 8513,   64, 7736, 9144, 9143,   64, 7607, 9143,    0
     c,  -16, 7116, 7113, 7628, 7607,   64, 7755, 7735, 7607,   64
     c, 7628, 9164,   64, 7755, 9163, 9164,   64, 7746, 8514, 8513
     c,   64, 7745, 8513,    0,  -16, 7116, 6987, 9287, 9161, 8907
     c, 8652, 8140, 7883, 7625, 7495, 7364, 7359, 7484, 7610, 7864
     c, 8119, 8631, 8888, 9146, 9276, 9280, 8640,   64, 9287, 9159
     c, 9033, 8906, 8651, 8139, 7882, 7753, 7623, 7492, 7487, 7612
     c, 7738, 7865, 8120, 8632, 8889, 9018, 9148, 9151, 8639, 8640
     c,    0,  -16, 7116, 6859, 7372, 7351,   64, 7372, 7500, 7479
     c, 7351,   64, 9164, 9036, 9015, 9143,   64, 9164, 9143,   64
     c, 7490, 9026,   64, 7489, 9025,    0,  -16, 7116, 7749, 8268
     c, 8247, 8375,   64, 8268, 8396, 8375,    0,  -16, 7116, 7241
     c, 8780, 8764, 8633, 8376, 8120, 7865, 7740, 7612,   64, 8780
     c, 8908, 8892, 8761, 8632, 8375, 8119, 7864, 7737, 7612,    0
     c,  -16, 7116, 6987, 7500, 7479, 7607,   64, 7500, 7628, 7607
     c,   64, 9292, 9164, 7616,   64, 9292, 7615,   64, 8003, 9143
     c, 9271,   64, 8131, 9271,    0,  -16, 7116, 7112, 7628, 7607
     c,   64, 7628, 7756, 7736,   64, 7736, 9144, 9143,   64, 7607
     c, 9143,    0,  -16, 7116, 6732, 7244, 7223,   64, 7367, 7351
     c, 7223,   64, 7367, 8247,   64, 7244, 8250,   64, 9292, 8250
     c,   64, 9159, 8247,   64, 9159, 9143, 9271,   64, 9292, 9271
     c,    0,  -16, 7116, 6859, 7372, 7351,   64, 7497, 7479, 7351
     c,   64, 7497, 9143,   64, 7372, 9018,   64, 9036, 9018,   64
     c, 9036, 9164, 9143,    0,  -16, 7116, 6859, 8012, 7755, 7497
     c, 7367, 7236, 7231, 7356, 7482, 7736, 7991, 8503, 8760, 9018
     c, 9148, 9279, 9284, 9159, 9033, 8779, 8524, 8012,   64, 8139
     c, 7754, 7495, 7364, 7359, 7484, 7737, 8120, 8376, 8761, 9020
     c, 9151, 9156, 9031, 8778, 8395, 8139,    0,  -16, 7116, 6986
     c, 7500, 7479,   64, 7627, 7607, 7479,   64, 7500, 8652, 8907
     c, 9034, 9160, 9157, 9027, 8898, 8641, 7617,   64, 7627, 8651
     c, 8906, 9032, 9029, 8899, 8642, 7618,    0,  -16, 7116, 6859
     c, 8012, 7755, 7497, 7367, 7236, 7231, 7356, 7482, 7736, 7991
     c, 8503, 8760, 9018, 9148, 9279, 9284, 9159, 9033, 8779, 8524
     c, 8012,   64, 8139, 7754, 7495, 7364, 7359, 7484, 7737, 8120
     c, 8376, 8761, 9020, 9151, 9156, 9031, 8778, 8395, 8139,   64
     c, 8378, 9013, 9141,   64, 8378, 8506, 9141,    0,  -16, 7116
     c, 6986, 7500, 7479,   64, 7627, 7607, 7479,   64, 7500, 8524
     c, 8907, 9034, 9160, 9157, 9027, 8898, 8513, 7617,   64, 7627
     c, 8523, 8906, 9032, 9029, 8899, 8514, 7618,   64, 8257, 9015
     c, 9143,   64, 8385, 9143,    0,  -16, 7116, 6986, 9161, 8907
     c, 8524, 8012, 7627, 7369, 7367, 7493, 7620, 7875, 8513, 8768
     c, 8895, 9021, 9018, 8889, 8504, 7992, 7737, 7610, 7354,   64
     c, 9161, 8905, 8778, 8523, 8011, 7626, 7497, 7495, 7621, 7876
     c, 8514, 8769, 9023, 9149, 9146, 8888, 8503, 7991, 7608, 7354
     c,    0,  -16, 7116, 7241, 8267, 8247,   64, 8395, 8375, 8247
     c,   64, 7500, 9164, 9163,   64, 7500, 7499, 9163,    0,  -16
     c, 7116, 6859, 7372, 7357, 7482, 7736, 8119, 8375, 8760, 9018
     c, 9149, 9164,   64, 7372, 7500, 7485, 7610, 7737, 8120, 8376
     c, 8761, 8890, 9021, 9036, 9164,    0,  -16, 7116, 6986, 7244
     c, 8247,   64, 7244, 7372, 8250,   64, 9292, 9164, 8250,   64
     c, 9292, 8247,    0,  -16, 7116, 6605, 6860, 7607,   64, 6860
     c, 6988, 7610,   64, 8268, 7610,   64, 8265, 7607,   64, 8265
     c, 8887,   64, 8268, 8890,   64, 9676, 9548, 8890,   64, 9676
     c, 8887,    0,  -16, 7116, 6986, 7372, 9015, 9143,   64, 7372
     c, 7500, 9143,   64, 9164, 9036, 7351,   64, 9164, 7479, 7351
     c,    0,  -16, 7116, 7114, 7372, 8258, 8247, 8375,   64, 7372
     c, 7500, 8386,   64, 9292, 9164, 8258,   64, 9292, 8386, 8375
     c,    0,  -16, 7116, 6986, 9036, 7351,   64, 9164, 7479,   64
     c, 7372, 9164,   64, 7372, 7371, 9035,   64, 7480, 9144, 9143
     c,   64, 7351, 9143,    0,  -16, 7116, 6602, 9036, 8778, 8519
     c, 8130, 7871, 7483, 7096, 6839, 6583, 6456, 6458, 6587, 6714
     c, 6585,   64, 9036, 8904, 8638, 8503,   64, 9036, 8631,   64
     c, 8503, 8505, 8380, 8254, 8000, 7745, 7489, 7360, 7358, 7483
     c, 7864, 8247, 8759, 9016,    0,  -16, 7116, 6732, 8651, 8522
     c, 8392, 8131, 7869, 7739, 7480, 7223,   64, 8522, 8391, 8127
     c, 7996, 7866, 7608, 7223, 6967, 6840, 6842, 6971, 7098, 6969
     c,   64, 7878, 7748, 7619, 7363, 7236, 7238, 7368, 7626, 7883
     c, 8268, 9036, 9291, 9417, 9415, 9285, 9028, 8515, 8259,   64
     c, 9036, 9163, 9289, 9287, 9157, 9028,   64, 8515, 8898, 9025
     c, 9151, 9148, 9017, 8888, 8631, 8375, 8248, 8250, 8381,   64
     c, 8515, 8770, 8897, 9023, 9020, 8889, 8631,    0,  -16, 7116
     c, 6987, 7370, 7240, 7238, 7364, 7747, 8131, 8644, 8901, 9159
     c, 9289, 9291, 9164, 8908, 8523, 8136, 7877, 7617, 7485, 7482
     c, 7608, 7991, 8247, 8632, 8890, 9020, 9022, 8896, 8640, 8383
     c, 8253,   64, 8908, 8651, 8264, 8005, 7745, 7613, 7610, 7736
     c, 7991,    0,  -16, 7116, 6731, 8651, 8522, 8392, 8131, 7869
     c, 7739, 7480, 7223,   64, 8522, 8391, 8127, 7996, 7866, 7608
     c, 7223, 6967, 6840, 6842, 6971, 7227, 7482, 7736, 7991, 8375
     c, 8632, 8890, 9150, 9283, 9286, 9161, 8907, 8652, 8012, 7627
     c, 7369, 7239, 7237, 7364, 7620, 7749, 7879,    0,  -16, 7116
     c, 7114, 8905, 8776, 8774, 8901, 9157, 9287, 9289, 9163, 8908
     c, 8524, 8267, 8138, 8008, 8006, 8132, 8387,   64, 8524, 8266
     c, 8136, 8133, 8387,   64, 8387, 8131, 7746, 7488, 7358, 7355
     c, 7481, 7608, 7863, 8247, 8632, 8890, 9020, 9022, 8896, 8640
     c, 8383, 8253,   64, 8131, 7874, 7616, 7486, 7482, 7608,    0
     c,  -16, 7116, 6858, 8906, 8776, 8515, 8253, 8123, 7864, 7607
     c,   64, 8134, 8004, 7747, 7491, 7365, 7367, 7497, 7755, 8140
     c, 9420, 9035, 8906, 8775, 8511, 8380, 8250, 7992, 7607, 7351
     c, 7096, 6969, 6970, 7099, 7226, 7097,   64, 8396, 8907, 9035
     c,   64, 7871, 8000, 8257, 8769, 9026, 9285, 9022,    0,  -16
     c, 7116, 6859, 7241, 7111, 7109, 7235, 7490, 7874, 8259, 8516
     c, 8903, 9034, 9035, 8908, 8780, 8523, 8265, 8135, 8004, 8001
     c, 8127, 8382, 8638, 8895, 9153, 9283,   64, 8908, 8651, 8393
     c, 8263, 8132, 8128, 8382,   64, 9283, 9151, 8891, 8633, 8376
     c, 7863, 7479, 7224, 7098, 7099, 7228, 7355, 7226,   64, 9151
     c, 8892, 8634, 8248, 7863,    0,  -16, 7116, 6732, 7494, 7367
     c, 7369, 7499, 7884, 8268, 7873, 7611, 7481, 7352, 7095, 6839
     c, 6712, 6714, 6843, 6970, 6841,   64, 8268, 7875, 7744, 7483
     c, 7353, 7095,   64, 7230, 7359, 7616, 8771, 9028, 9414, 9672
     c, 9802, 9803, 9676, 9548, 9291, 9032, 8902, 8640, 8508, 8505
     c, 8759, 8887, 9144, 9402,   64, 9548, 9290, 9030, 8768, 8636
     c, 8633, 8759,    0,  -16, 7116, 7111, 8906, 8647, 8386, 8125
     c, 7995, 7736, 7479,   64, 9158, 8900, 8515, 8131, 7876, 7750
     c, 7752, 7882, 8139, 8652, 9164, 8906, 8776, 8514, 8252, 8122
     c, 7864, 7479, 7223, 7096, 7098, 7227, 7354, 7225,    0,  -16
     c, 7116, 7112, 9164, 8906, 8647, 8386, 7993, 7733,   64, 9157
     c, 8899, 8514, 8130, 7875, 7749, 7751, 7881, 8139, 8652, 9164
     c, 8905, 8775, 8382, 8122, 7992, 7733, 7604, 7347, 7220, 7222
     c, 7352, 7610, 7867, 8252, 8765,    0,  -16, 7116, 6732, 7494
     c, 7367, 7369, 7627, 8012, 8268, 7873, 7611, 7481, 7352, 7095
     c, 6839, 6712, 6714, 6843, 6970, 6841,   64, 8268, 7875, 7744
     c, 7483, 7353, 7095,   64, 9291, 8903, 8645, 8388, 8003,   64
     c, 9675, 9546, 9673, 9802, 9803, 9676, 9548, 9291, 8902, 8773
     c, 8516, 8003,   64, 8003, 8386, 8512, 8633, 8759,   64, 8003
     c, 8258, 8384, 8505, 8759, 8887, 9144, 9402,    0,  -16, 7116
     c, 7113, 7625, 7495, 7493, 7619, 7874, 8258, 8643, 8900, 9287
     c, 9418, 9419, 9292, 9164, 8907, 8778, 8519, 7997, 7867, 7608
     c, 7351,   64, 8778, 8518, 8255, 8124, 7994, 7736, 7351, 7095
     c, 6968, 6970, 7099, 7355, 7610, 7992, 8247, 8631, 8888, 9146
     c,    0,  -16, 7116, 6478, 8268, 7747, 7357, 7098, 6840, 6583
     c, 6327, 6200, 6202, 6331, 6458, 6329,   64, 8268, 8005, 7873
     c, 7740, 7736, 7991,   64, 8268, 8136, 8003, 7868, 7864, 7991
     c,   64, 9420, 8899, 8250, 7991,   64, 9420, 9157, 9025, 8892
     c, 8888, 9143, 9271, 9528, 9786,   64, 9420, 9288, 9155, 9020
     c, 9016, 9143,    0,  -16, 7116, 6860, 8268, 8136, 7874, 7613
     c, 7483, 7224, 6967, 6711, 6584, 6586, 6715, 6842, 6713,   64
     c, 8268, 8263, 8380, 8503,   64, 8268, 8391, 8508, 8503,   64
     c,10059, 9930,10057,10186,10187,10060, 9804, 9547, 9288, 9158
     c, 8897, 8635, 8503,    0,  -16, 7116, 6987, 8396, 8139, 7881
     c, 7622, 7492, 7360, 7356, 7481, 7608, 7863, 8119, 8504, 8762
     c, 9021, 9151, 9283, 9287, 9162, 9035, 8907, 8650, 8392, 8132
     c, 7999, 7996,   64, 8139, 7880, 7620, 7488, 7484, 7609, 7863
     c,    0,  -16, 7116, 6731, 8651, 8522, 8392, 8131, 7869, 7739
     c, 7480, 7223,   64, 8522, 8391, 8127, 7996, 7866, 7608, 7223
     c, 6967, 6840, 6842, 6971, 7098, 6969,   64, 7878, 7748, 7619
     c, 7363, 7236, 7238, 7368, 7626, 7883, 8268, 8780, 9163, 9290
     c, 9416, 9413, 9283, 9154, 8769, 8513, 8258,   64, 8780, 9035
     c, 9162, 9288, 9285, 9155, 9026, 8769,    0,  -16, 7116, 6987
     c, 8648, 8646, 8516, 8387, 8130, 7874, 7748, 7750, 7881, 8139
     c, 8524, 8908, 9163, 9289, 9285, 9154, 8895, 8379, 7993, 7736
     c, 7351, 7095, 6968, 6970, 7099, 7355, 7610, 7992, 8375, 8759
     c, 9016, 9274,   64, 8908, 9035, 9161, 9157, 9026, 8767, 8380
     c, 7865, 7351,    0,  -16, 7116, 6732, 8651, 8522, 8392, 8131
     c, 7869, 7739, 7480, 7223,   64, 8522, 8391, 8127, 7996, 7866
     c, 7608, 7223, 6967, 6840, 6842, 6971, 7098, 6969,   64, 7878
     c, 7748, 7619, 7363, 7236, 7238, 7368, 7626, 7883, 8268, 8908
     c, 9291, 9417, 9415, 9285, 9156, 8771, 8259,   64, 8908, 9163
     c, 9289, 9287, 9157, 9028, 8771,   64, 8259, 8642, 8768, 8889
     c, 9015,   64, 8259, 8514, 8640, 8761, 9015, 9143, 9400, 9658
     c,    0,  -16, 7116, 6986, 7753, 7623, 7621, 7747, 8002, 8386
     c, 8771, 9028, 9415, 9546, 9547, 9420, 9292, 9035, 8906, 8776
     c, 8645, 8382, 8251, 7992, 7735,   64, 8776, 8644, 8509, 8378
     c, 8120, 7735, 7351, 7096, 6970, 6971, 7100, 7227, 7098,    0
     c,  -16, 7116, 7113, 9162, 9032, 8771, 8509, 8379, 8120, 7863
     c,   64, 8390, 8260, 8003, 7747, 7621, 7623, 7753, 8011, 8396
     c, 9548, 9291, 9162, 9031, 8767, 8636, 8506, 8248, 7863, 7607
     c, 7352, 7225, 7226, 7355, 7482, 7353,   64, 8652, 9163, 9291
     c,    0,  -16, 7116, 6859, 6984, 7243, 7500, 7628, 7882, 7879
     c, 7748, 7356, 7353, 7479,   64, 7628, 7754, 7751, 7359, 7228
     c, 7225, 7479, 7735, 7992, 8379, 8638, 8768,   64, 9292, 8768
     c, 8636, 8633, 8887, 9015, 9272, 9530,   64, 9420, 8896, 8764
     c, 8761, 8887,    0,  -16, 7116, 6858, 6984, 7243, 7500, 7628
     c, 7882, 7879, 7747, 7484, 7481, 7607,   64, 7628, 7754, 7751
     c, 7488, 7356, 7353, 7607, 7735, 8120, 8507, 8766, 9026, 9157
     c, 9289, 9291, 9164, 9036, 8907, 8777, 8774, 8900, 9154, 9409
     c, 9665,    0,  -16, 7116, 6731, 7110, 6982, 6855, 6857, 6987
     c, 7244, 7756, 7626, 7494, 7357, 7223,   64, 7494, 7485, 7351
     c,   64, 8780, 8522, 8262, 7869, 7609, 7351,   64, 8780, 8650
     c, 8518, 8381, 8247,   64, 8518, 8509, 8375,   64,10060, 9803
     c, 9545, 9286, 8893, 8633, 8375,    0,  -16, 7116, 6986, 8007
     c, 7878, 7622, 7495, 7497, 7627, 7884, 8140, 8395, 8521, 8518
     c, 8386, 8125, 7866, 7608, 7223, 6967, 6840, 6842, 6971, 7098
     c, 6969,   64, 8140, 8267, 8393, 8390, 8258, 7997, 7738, 7480
     c, 7223,   64, 9675, 9546, 9673, 9802, 9803, 9676, 9420, 9163
     c, 8905, 8646, 8386, 8253, 8250, 8376, 8503, 8631, 8888, 9146
     c,    0,  -16, 7116, 6859, 7240, 7499, 7756, 7884, 8139, 8137
     c, 7875, 7872, 7998,   64, 7884, 8011, 8009, 7747, 7744, 7998
     c, 8254, 8639, 8897, 9156, 9286,   64, 9548, 9286, 8894, 8634
     c,   64, 9676, 9414, 9153, 8893, 8634, 8376, 7991, 7479, 7224
     c, 7098, 7099, 7228, 7355, 7226,    0,  -16, 7116, 6858, 9290
     c, 9160, 8899, 8768, 8638, 8379, 8121, 7864, 7479,   64, 8390
     c, 8260, 8003, 7747, 7621, 7623, 7753, 8011, 8396, 9676, 9419
     c, 9290, 9159, 9027, 8765, 8506, 8120, 7479, 6967, 6840, 6842
     c, 6971, 7227, 7482, 7864, 8119, 8503, 8888, 9146,   64, 8780
     c, 9291, 9419,    0,  -16, 7116, 6986, 8901, 8887, 9015,   64
     c, 8901, 9029, 9015,   64, 8898, 8644, 8389, 8005, 7748, 7490
     c, 7359, 7357, 7482, 7736, 7991, 8375, 8632, 8890,   64, 8898
     c, 8388, 8004, 7747, 7618, 7487, 7485, 7610, 7737, 7992, 8376
     c, 8890,    0,  -16, 7116, 6986, 7500, 7479, 7607,   64, 7500
     c, 7628, 7607,   64, 7618, 7876, 8133, 8517, 8772, 9026, 9151
     c, 9149, 9018, 8760, 8503, 8119, 7864, 7610,   64, 7618, 8132
     c, 8516, 8771, 8898, 9023, 9021, 8890, 8761, 8504, 8120, 7610
     c,    0,  -16, 7116, 7113, 9026, 8772, 8517, 8133, 7876, 7618
     c, 7487, 7485, 7610, 7864, 8119, 8503, 8760, 9018,   64, 9026
     c, 8897, 8771, 8516, 8132, 7875, 7746, 7615, 7613, 7738, 7865
     c, 8120, 8504, 8761, 8891, 9018,    0,  -16, 7116, 6986, 8908
     c, 8887, 9015,   64, 8908, 9036, 9015,   64, 8898, 8644, 8389
     c, 8005, 7748, 7490, 7359, 7357, 7482, 7736, 7991, 8375, 8632
     c, 8890,   64, 8898, 8388, 8004, 7747, 7618, 7487, 7485, 7610
     c, 7737, 7992, 8376, 8890,    0,  -16, 7116, 7113, 7614, 9022
     c, 9025, 8899, 8772, 8517, 8133, 7876, 7618, 7487, 7485, 7610
     c, 7864, 8119, 8503, 8760, 9018,   64, 7615, 8895, 8897, 8771
     c, 8516, 8132, 7875, 7746, 7615, 7613, 7738, 7865, 8120, 8504
     c, 8761, 8891, 9018,    0,  -16, 7116, 7496, 8908, 8652, 8395
     c, 8264, 8247, 8375,   64, 8908, 8907, 8651, 8394,   64, 8523
     c, 8392, 8375,   64, 7877, 8773, 8772,   64, 7877, 7876, 8772
     c,    0,  -16, 7116, 6986, 9029, 8901, 8886, 8755, 8626, 8369
     c, 8113, 7858, 7731, 7475,   64, 9029, 9014, 8883, 8625, 8368
     c, 7984, 7729, 7475,   64, 8898, 8644, 8389, 8005, 7748, 7490
     c, 7359, 7357, 7482, 7736, 7991, 8375, 8632, 8890,   64, 8898
     c, 8388, 8004, 7747, 7618, 7487, 7485, 7610, 7737, 7992, 8376
     c, 8890,    0,  -16, 7116, 6986, 7500, 7479, 7607,   64, 7500
     c, 7628, 7607,   64, 7617, 8004, 8261, 8645, 8900, 9025, 9015
     c,   64, 7617, 8003, 8260, 8516, 8771, 8897, 8887, 9015,    0
     c,  -16, 7116, 7749, 8268, 8139, 8138, 8265, 8393, 8522, 8523
     c, 8396, 8268,   64, 8267, 8266, 8394, 8395, 8267,   64, 8261
     c, 8247, 8375,   64, 8261, 8389, 8375,    0,  -16, 7116, 7749
     c, 8268, 8139, 8138, 8265, 8393, 8522, 8523, 8396, 8268,   64
     c, 8267, 8266, 8394, 8395, 8267,   64, 8261, 8240, 8368,   64
     c, 8261, 8389, 8368,    0,  -16, 7116, 6985, 7500, 7479, 7607
     c,   64, 7500, 7628, 7607,   64, 9029, 8901, 7611,   64, 9029
     c, 7610,   64, 7998, 8759, 9015,   64, 8127, 9015,    0,  -16
     c, 7116, 7749, 8268, 8247, 8375,   64, 8268, 8396, 8375,    0
     c,  -16, 7116, 6352, 6853, 6839, 6967,   64, 6853, 6981, 6967
     c,   64, 6977, 7364, 7621, 8005, 8260, 8385, 8375,   64, 6977
     c, 7363, 7620, 7876, 8131, 8257, 8247, 8375,   64, 8385, 8772
     c, 9029, 9413, 9668, 9793, 9783,   64, 8385, 8771, 9028, 9284
     c, 9539, 9665, 9655, 9783,    0,  -16, 7116, 6986, 7493, 7479
     c, 7607,   64, 7493, 7621, 7607,   64, 7617, 8004, 8261, 8645
     c, 8900, 9025, 9015,   64, 7617, 8003, 8260, 8516, 8771, 8897
     c, 8887, 9015,    0,  -16, 7116, 7114, 8133, 7876, 7618, 7487
     c, 7485, 7610, 7864, 8119, 8503, 8760, 9018, 9149, 9151, 9026
     c, 8772, 8517, 8133,   64, 8132, 7875, 7746, 7615, 7613, 7738
     c, 7865, 8120, 8504, 8761, 8890, 9021, 9023, 8898, 8771, 8516
     c, 8132,    0,  -16, 7116, 6986, 7493, 7472, 7600,   64, 7493
     c, 7621, 7600,   64, 7618, 7876, 8133, 8517, 8772, 9026, 9151
     c, 9149, 9018, 8760, 8503, 8119, 7864, 7610,   64, 7618, 8132
     c, 8516, 8771, 8898, 9023, 9021, 8890, 8761, 8504, 8120, 7610
     c,    0,  -16, 7116, 6986, 8901, 8880, 9008,   64, 8901, 9029
     c, 9008,   64, 8898, 8644, 8389, 8005, 7748, 7490, 7359, 7357
     c, 7482, 7736, 7991, 8375, 8632, 8890,   64, 8898, 8388, 8004
     c, 7747, 7618, 7487, 7485, 7610, 7737, 7992, 8376, 8890,    0
     c,  -16, 7116, 7367, 7877, 7863, 7991,   64, 7877, 8005, 7991
     c,   64, 7999, 8130, 8388, 8645, 9029,   64, 7999, 8129, 8387
     c, 8644, 9028, 9029,    0,  -16, 7116, 7241, 9026, 8900, 8517
     c, 8133, 7748, 7618, 7744, 7999, 8637, 8892,   64, 8765, 8891
     c, 8890, 8760,   64, 8889, 8504, 8120, 7737,   64, 7864, 7738
     c, 7610,   64, 9026, 8898, 8772,   64, 8899, 8516, 8132, 7747
     c,   64, 7876, 7746, 7872,   64, 7745, 8000, 8638, 8893, 9019
     c, 9018, 8888, 8503, 8119, 7736, 7610,    0,  -16, 7116, 7622
     c, 8268, 8247, 8375,   64, 8268, 8396, 8375,   64, 7877, 8773
     c, 8772,   64, 7877, 7876, 8772,    0,  -16, 7116, 6986, 7493
     c, 7483, 7608, 7863, 8247, 8504, 8891,   64, 7493, 7621, 7611
     c, 7737, 7992, 8248, 8505, 8891,   64, 8901, 8887, 9015,   64
     c, 8901, 9029, 9015,    0,  -16, 7116, 7240, 7493, 8247,   64
     c, 7493, 7621, 8249,   64, 9029, 8901, 8249,   64, 9029, 8247
     c,    0,  -16, 7116, 6732, 7109, 7735,   64, 7109, 7237, 7738
     c,   64, 8261, 7738,   64, 8258, 7735,   64, 8258, 8759,   64
     c, 8261, 8762,   64, 9413, 9285, 8762,   64, 9413, 8759,    0
     c,  -16, 7116, 7113, 7493, 8887, 9015,   64, 7493, 7621, 9015
     c,   64, 9029, 8901, 7479,   64, 9029, 7607, 7479,    0,  -16
     c, 7116, 7240, 7493, 8247,   64, 7493, 7621, 8249,   64, 9029
     c, 8901, 8249, 7728,   64, 9029, 8247, 7856, 7728,    0,  -16
     c, 7116, 7113, 8772, 7479,   64, 9029, 7736,   64, 7493, 9029
     c,   64, 7493, 7492, 8772,   64, 7736, 9016, 9015,   64, 7479
     c, 9015,    0,  -16, 7116, 7369, 8637, 8511, 8256, 8000, 7743
     c, 7614, 7484, 7482, 7608, 7863, 8119, 8376, 8506,   64, 8000
     c, 7742, 7612, 7609, 7863,   64, 8768, 8506, 8504, 8759, 9016
     c, 9145, 9404,   64, 8896, 8634, 8632, 8759,    0,  -16, 7116
     c, 7496, 7484, 7743, 8003,   64, 8396, 7610, 7608, 7863, 7991
     c, 8248, 8506, 8637, 8640, 8764, 8891, 9019, 9276,   64, 8524
     c, 7738, 7736, 7863,    0,  -16, 7116, 7494, 8511, 8382, 8510
     c, 8511, 8384, 8128, 7871, 7742, 7612, 7610, 7736, 7991, 8375
     c, 8761, 9020,   64, 8128, 7870, 7740, 7737, 7991,    0,  -16
     c, 7116, 7369, 8637, 8511, 8256, 8000, 7743, 7614, 7484, 7482
     c, 7608, 7863, 8119, 8376, 8506,   64, 8000, 7742, 7612, 7609
     c, 7863,   64, 9292, 8506, 8504, 8759, 9016, 9145, 9404,   64
     c, 9420, 8634, 8632, 8759,    0,  -16, 7116, 7494, 7865, 8122
     c, 8251, 8381, 8383, 8256, 8128, 7871, 7742, 7612, 7610, 7736
     c, 7991, 8375, 8761, 9020,   64, 8128, 7870, 7740, 7737, 7991
     c,    0,  -16, 7116, 7878, 8256, 8643, 8902, 9033, 9035, 8908
     c, 8651, 8521, 7342, 7340, 7467, 7724, 7855, 7992, 8119, 8375
     c, 8632, 8761, 9020,   64, 8521, 8388, 8256, 7863, 7602, 7342
     c,    0,  -16, 7116, 7369, 8637, 8511, 8256, 8000, 7743, 7614
     c, 7484, 7482, 7608, 7863, 8119, 8376, 8506,   64, 8000, 7742
     c, 7612, 7609, 7863,   64, 8768, 7982,   64, 8896, 8503, 8242
     c, 7982, 7852, 7595, 7468, 7470, 7601, 7859, 8245, 8759, 9145
     c, 9404,    0,  -16, 7116, 7497, 7484, 7743, 8003,   64, 8396
     c, 7479,   64, 8524, 7607,   64, 7869, 8127, 8384, 8512, 8767
     c, 8765, 8634, 8632, 8759,   64, 8512, 8639, 8637, 8506, 8504
     c, 8759, 9016, 9145, 9404,    0,  -16, 7116, 7748, 8390, 8261
     c, 8388, 8517, 8390,   64, 8128, 7866, 7864, 8119, 8376, 8505
     c, 8764,   64, 8256, 7994, 7992, 8119,    0,  -16, 7116, 7748
     c, 8390, 8261, 8388, 8517, 8390,   64, 8128, 7342,   64, 8256
     c, 7863, 7602, 7342, 7212, 6955, 6828, 6830, 6961, 7219, 7605
     c, 8119, 8505, 8764,    0,  -16, 7116, 7496, 7484, 7743, 8003
     c,   64, 8396, 7479,   64, 8524, 7607,   64, 8640, 8639, 8767
     c, 8640, 8512, 8254, 7869,   64, 7869, 8252, 8376, 8503,   64
     c, 7869, 8124, 8248, 8503, 8631, 9017, 9276,    0,  -16, 7116
     c, 7748, 7740, 7999, 8259,   64, 8652, 7866, 7864, 8119, 8376
     c, 8505, 8764,   64, 8780, 7994, 7992, 8119,    0,  -16, 7116
     c, 6604, 6588, 6847, 7104, 7359, 7357, 7095,   64, 7104, 7231
     c, 7229, 6967,   64, 7357, 7615, 7872, 8000, 8255, 8253, 7991
     c,   64, 8000, 8127, 8125, 7863,   64, 8253, 8511, 8768, 8896
     c, 9151, 9149, 9018, 9016, 9143,   64, 8896, 9023, 9021, 8890
     c, 8888, 9143, 9400, 9529, 9788,    0,  -16, 7116, 7113, 7100
     c, 7359, 7616, 7871, 7869, 7607,   64, 7616, 7743, 7741, 7479
     c,   64, 7869, 8127, 8384, 8512, 8767, 8765, 8634, 8632, 8759
     c,   64, 8512, 8639, 8637, 8506, 8504, 8759, 9016, 9145, 9404
     c,    0,  -16, 7116, 7367, 8256, 8000, 7743, 7614, 7484, 7482
     c, 7608, 7863, 8119, 8376, 8505, 8635, 8637, 8511, 8256, 8127
     c, 8125, 8251, 8506, 8762, 9019, 9148,   64, 8000, 7742, 7612
     c, 7609, 7863,    0,  -16, 7116, 7497, 7484, 7743, 8003,   64
     c, 8134, 6955,   64, 8262, 7083,   64, 7869, 8127, 8384, 8512
     c, 8767, 8765, 8634, 8632, 8759,   64, 8512, 8639, 8637, 8506
     c, 8504, 8759, 9016, 9145, 9404,    0,  -16, 7116, 7369, 8637
     c, 8511, 8256, 8000, 7743, 7614, 7484, 7482, 7608, 7863, 8119
     c, 8376,   64, 8000, 7742, 7612, 7609, 7863,   64, 8768, 7982
     c, 7980, 8107, 8364, 8495, 8503, 8759, 9145, 9404,   64, 8896
     c, 8503, 8242, 7982,    0,  -16, 7116, 7496, 7484, 7743, 8000
     c, 8255, 8253, 7991,   64, 8000, 8127, 8125, 7863,   64, 8253
     c, 8511, 8768, 8896, 8765,   64, 8768, 8765, 8891, 9019, 9276
     c,    0,  -16, 7116, 7752, 7740, 7999, 8129, 8127, 8509, 8635
     c, 8633, 8504, 8247,   64, 8127, 8381, 8507, 8505, 8247,   64
     c, 7736, 7991, 8631, 9017, 9276,    0,  -16, 7116, 7748, 7740
     c, 7999, 8259,   64, 8652, 7866, 7864, 8119, 8376, 8505, 8764
     c,   64, 8780, 7994, 7992, 8119,   64, 8004, 8772,    0,  -16
     c, 7116, 7369, 7744, 7482, 7480, 7735, 7863, 8120, 8378, 8637
     c,   64, 7872, 7610, 7608, 7735,   64, 8768, 8506, 8504, 8759
     c, 9016, 9145, 9404,   64, 8896, 8634, 8632, 8759,    0,  -16
     c, 7116, 7368, 7744, 7614, 7483, 7480, 7735, 7863, 8248, 8506
     c, 8637, 8640,   64, 7872, 7742, 7611, 7608, 7735,   64, 8640
     c, 8764, 8891, 9019, 9276,    0,  -16, 7116, 6987, 7488, 7230
     c, 7099, 7096, 7351, 7479, 7736, 7994,   64, 7616, 7358, 7227
     c, 7224, 7351,   64, 8256, 7994, 7992, 8247, 8375, 8632, 8890
     c, 9021, 9024,   64, 8384, 8122, 8120, 8247,   64, 9024, 9148
     c, 9275, 9403, 9660,    0,  -16, 7116, 7240, 7228, 7487, 7744
     c, 8000, 8127, 8125, 7994, 7864, 7607, 7479, 7352, 7353, 7481
     c, 7352,   64, 8895, 8766, 8894, 8895, 8768, 8640, 8383, 8253
     c, 8122, 8120, 8247, 8631, 9017, 9276,   64, 8127, 8253,   64
     c, 8383, 8125,   64, 7994, 8120,   64, 8122, 7864,    0,  -16
     c, 7116, 7369, 7744, 7482, 7480, 7735, 7863, 8120, 8378, 8637
     c,   64, 7872, 7610, 7608, 7735,   64, 8768, 7982,   64, 8896
     c, 8503, 8242, 7982, 7852, 7595, 7468, 7470, 7601, 7859, 8245
     c, 8759, 9145, 9404,    0,  -16, 7116, 7495, 7484, 7743, 8000
     c, 8256, 8511, 8508, 8378, 7992, 7735,   64, 8256, 8383, 8380
     c, 8250, 7992,   64, 7735, 7990, 8116, 8113, 7982, 7724, 7467
     c, 7340, 7342, 7473, 7860, 8246, 8761, 9148,   64, 7735, 7862
     c, 7988, 7985, 7854, 7724,    0,  -16, 7116, 8256,   64,    0
     c,  -16, 7116, 7748,   64,    0,  -16, 7116, 7240,   64,    0
     c,  -16, 7116, 6986, 8140, 7755, 7496, 7363, 7360, 7483, 7736
     c, 8119, 8375, 8760, 9019, 9152, 9155, 9032, 8779, 8396, 8140
     c,   64, 7883, 7624, 7491, 7488, 7611, 7864,   64, 7737, 8120
     c, 8376, 8761,   64, 8632, 8891, 9024, 9027, 8904, 8651,   64
     c, 8778, 8395, 8139, 7754,    0,  -16, 7116, 6986, 7752, 8009
     c, 8396, 8375,   64, 7752, 7751, 8008, 8266, 8247, 8375,    0
     c,  -16, 7116, 6986, 7495, 7496, 7626, 7755, 8012, 8524, 8779
     c, 8906, 9032, 9030, 8900, 8641, 7479,   64, 7495, 7623, 7624
     c, 7754, 8011, 8523, 8778, 8904, 8902, 8772, 8513, 7351,   64
     c, 7480, 9144, 9143,   64, 7351, 9143,    0,  -16, 7116, 6986
     c, 7628, 9036, 8131,   64, 7628, 7627, 8907,   64, 8908, 8003
     c,   64, 8132, 8388, 8771, 9025, 9150, 9149, 9018, 8760, 8375
     c, 7991, 7608, 7481, 7355, 7483,   64, 8003, 8387, 8770, 9023
     c,   64, 8515, 8897, 9022, 9021, 8890, 8504,   64, 9020, 8761
     c, 8376, 7992, 7609, 7483,   64, 7864, 7482,    0,  -16, 7116
     c, 6986, 8649, 8631, 8759,   64, 8780, 8759,   64, 8780, 7356
     c, 9276,   64, 8649, 7484,   64, 7485, 9277, 9276,    0,  -16
     c, 7116, 6986, 7628, 7491,   64, 7755, 7620,   64, 7628, 8908
     c, 8907,   64, 7755, 8907,   64, 7620, 8005, 8389, 8772, 9026
     c, 9151, 9149, 9018, 8760, 8375, 7991, 7608, 7481, 7355, 7483
     c,   64, 7491, 7619, 7876, 8388, 8771, 9024,   64, 8516, 8898
     c, 9023, 9021, 8890, 8504,   64, 9020, 8761, 8376, 7992, 7609
     c, 7483,   64, 7864, 7482,    0,  -16, 7116, 6986, 8779, 8905
     c, 9033, 8907, 8524, 8268, 7883, 7624, 7491, 7486, 7610, 7864
     c, 8247, 8375, 8760, 9018, 9149, 9150, 9025, 8771, 8388, 8260
     c, 7875, 7617,   64, 8906, 8523, 8267, 7882,   64, 8011, 7752
     c, 7619, 7614, 7738, 8120,   64, 7612, 7865, 8248, 8376, 8761
     c, 9020,   64, 8504, 8890, 9021, 9022, 8897, 8515,   64, 9023
     c, 8770, 8387, 8259, 7874, 7615,   64, 8131, 7745, 7614,    0
     c,  -16, 7116, 6986, 7372, 9164, 7863,   64, 7372, 7371, 9035
     c,   64, 9036, 7735, 7863,    0,  -16, 7116, 6986, 8012, 7627
     c, 7497, 7495, 7621, 7748, 8003, 8514, 8769, 8896, 9022, 9019
     c, 8889, 8504, 7992, 7609, 7483, 7486, 7616, 7745, 8002, 8515
     c, 8772, 8901, 9031, 9033, 8907, 8524, 8012,   64, 7755, 7625
     c, 7623, 7749, 8004, 8515, 8770, 9024, 9150, 9147, 9017, 8888
     c, 8503, 7991, 7608, 7481, 7355, 7358, 7488, 7746, 8003, 8516
     c, 8773, 8903, 8905, 8779,   64, 8906, 8523, 8011, 7626,   64
     c, 7482, 7864,   64, 8632, 9018,    0,  -16, 7116, 6986, 8898
     c, 8640, 8255, 8127, 7744, 7490, 7365, 7366, 7497, 7755, 8140
     c, 8268, 8651, 8905, 9029, 9024, 8891, 8632, 8247, 7991, 7608
     c, 7482, 7610, 7736,   64, 8901, 8770, 8384,   64, 8900, 8641
     c, 8256, 8128, 7745, 7492,   64, 8000, 7618, 7493, 7494, 7625
     c, 8011,   64, 7495, 7754, 8139, 8267, 8650, 8903,   64, 8395
     c, 8777, 8901, 8896, 8763, 8504,   64, 8633, 8248, 7992, 7609
     c,    0,  -16, 7116, 7622, 8250, 8121, 8120, 8247, 8375, 8504
     c, 8505, 8378, 8250,   64, 8249, 8248, 8376, 8377, 8249,    0
     c,  -16, 7116, 7622, 8504, 8375, 8247, 8120, 8121, 8250, 8378
     c, 8505, 8502, 8372, 8115,   64, 8249, 8248, 8376, 8377, 8249
     c,   64, 8375, 8502,   64, 8504, 8372,    0,  -16, 7116, 7622
     c, 8261, 8132, 8131, 8258, 8386, 8515, 8516, 8389, 8261,   64
     c, 8260, 8259, 8387, 8388, 8260,   64, 8250, 8121, 8120, 8247
     c, 8375, 8504, 8505, 8378, 8250,   64, 8249, 8248, 8376, 8377
     c, 8249,    0,  -16, 7116, 7622, 8261, 8132, 8131, 8258, 8386
     c, 8515, 8516, 8389, 8261,   64, 8260, 8259, 8387, 8388, 8260
     c,   64, 8504, 8375, 8247, 8120, 8121, 8250, 8378, 8505, 8502
     c, 8372, 8115,   64, 8249, 8248, 8376, 8377, 8249,   64, 8375
     c, 8502,   64, 8504, 8372,    0,  -16, 7116, 7622, 8268, 8254
     c, 8382,   64, 8268, 8396, 8382,   64, 8250, 8121, 8120, 8247
     c, 8375, 8504, 8505, 8378, 8250,   64, 8249, 8248, 8376, 8377
     c, 8249,    0,  -16, 7116, 7114, 7495, 7496, 7626, 7755, 8140
     c, 8524, 8907, 9034, 9160, 9158, 9028, 8899, 8642, 8257,   64
     c, 7495, 7623, 7624, 7754, 8139, 8523, 8906, 9032, 9030, 8900
     c, 8643, 8258,   64, 7625, 8011,   64, 8651, 9033,   64, 9029
     c, 8514,   64, 8258, 8254, 8382, 8386,   64, 8250, 8121, 8120
     c, 8247, 8375, 8504, 8505, 8378, 8250,   64, 8249, 8248, 8376
     c, 8377, 8249,    0,  -16, 7116, 7622, 8524, 8267, 8137, 8134
     c, 8261, 8389, 8518, 8519, 8392, 8264, 8135,   64, 8263, 8262
     c, 8390, 8391, 8263,   64, 8267, 8135,   64, 8137, 8264,    0
     c,  -16, 7116, 7622, 8522, 8393, 8265, 8138, 8139, 8268, 8396
     c, 8523, 8520, 8390, 8133,   64, 8267, 8266, 8394, 8395, 8267
     c,   64, 8393, 8520,   64, 8522, 8390,    0,  -16, 7116, 6733
     c, 9541, 9285, 9028, 8898, 8636, 8506, 8377, 8120, 7608, 7353
     c, 7227, 7229, 7359, 7488, 8131, 8389, 8519, 8521, 8395, 8140
     c, 8012, 7755, 7625, 7623, 7748, 8001, 8635, 9016, 9271, 9527
     c,   64, 9541, 9540, 9284, 9027,   64, 9156, 9026, 8764, 8634
     c, 8376, 8119, 7607, 7352, 7225, 7099, 7101, 7231, 7489, 8132
     c, 8261, 8391, 8393, 8267,   64, 8394, 8139, 8011, 7754,   64
     c, 7883, 7753, 7751, 7876, 8129, 8763, 9017, 9272, 9528, 9527
     c,    0,  -16, 7116, 7114, 8272, 8243, 8371,   64, 8272, 8400
     c, 8371,   64, 8905, 9161, 8907, 8524, 8140, 7755, 7497, 7495
     c, 7621, 7748, 8768, 8895, 9021, 9019, 8889, 8504, 8120, 7865
     c, 7738,   64, 8905, 8778, 8523, 8139, 7754, 7625, 7623, 7749
     c, 8769, 9023, 9149, 9147, 9017, 8888, 8503, 8119, 7736, 7482
     c, 7738,   64, 9018, 8632,    0,  -16, 7116, 6860, 9424, 7088
     c, 7216,   64, 9424, 9552, 7216,    0,  -16, 7116, 7367, 8656
     c, 8398, 8139, 7879, 7746, 7742, 7865, 8117, 8370, 8624, 8752
     c,   64, 8656, 8784, 8526, 8267, 8007, 7874, 7870, 7993, 8245
     c, 8498, 8752,    0,  -16, 7116, 7367, 7760, 8014, 8267, 8519
     c, 8642, 8638, 8505, 8245, 7986, 7728, 7856,   64, 7760, 7888
     c, 8142, 8395, 8647, 8770, 8766, 8633, 8373, 8114, 7856,    0
     c,  -16, 7116, 7240, 8268, 8139, 8385, 8256,   64, 8268, 8256
     c,   64, 8268, 8395, 8129, 8256,   64, 7625, 7753, 8771, 8899
     c,   64, 7625, 8899,   64, 7625, 7624, 8900, 8899,   64, 8905
     c, 8777, 7747, 7619,   64, 8905, 7619,   64, 8905, 8904, 7620
     c, 7619,    0,  -16, 7116, 6733, 7233, 9409, 9408,   64, 7233
     c, 7232, 9408,    0,  -16, 7116, 6733, 8265, 8248, 8376,   64
     c, 8265, 8393, 8376,   64, 7233, 9409, 9408,   64, 7233, 7232
     c, 9408,    0,  -16, 7116, 6733, 7237, 9413, 9412,   64, 7237
     c, 7236, 9412,   64, 7229, 9405, 9404,   64, 7229, 7228, 9404
     c,    0,  -16, 7116, 7749, 8396, 8267, 8261,   64, 8395, 8261
     c,   64, 8396, 8523, 8261,    0,  -16, 7116, 7113, 7756, 7627
     c, 7621,   64, 7755, 7621,   64, 7756, 7883, 7621,   64, 8908
     c, 8779, 8773,   64, 8907, 8773,   64, 8908, 9035, 8773,    0
     c,  -16, 7116, 7367, 8140, 7883, 7753, 7751, 7877, 8132, 8388
     c, 8645, 8775, 8777, 8651, 8396, 8140,   64, 8140, 7753, 7877
     c, 8388, 8775, 8651, 8140,   64, 8396, 7883, 7751, 8132, 8645
     c, 8777, 8396,    0,  -16, 7116, 8256,   64,    0,  -16, 7116
     c, 7748,   64,    0,  -16, 7116, 7240,   64,    0,  -16, 7116
     c, 6987, 8524, 8139, 7881, 7622, 7491, 7359, 7356, 7481, 7608
     c, 7863, 8119, 8504, 8762, 9021, 9152, 9284, 9287, 9162, 9035
     c, 8780, 8524,   64, 8524, 8267, 8009, 7750, 7619, 7487, 7484
     c, 7609, 7863,   64, 8119, 8376, 8634, 8893, 9024, 9156, 9159
     c, 9034, 8780,    0,  -16, 7116, 6987, 8520, 7863,   64, 8780
     c, 7991,   64, 8780, 8393, 8007, 7750,   64, 8649, 8135, 7750
     c,    0,  -16, 7116, 6987, 7880, 8007, 7878, 7751, 7752, 7882
     c, 8011, 8396, 8780, 9163, 9289, 9287, 9157, 8899, 8513, 7999
     c, 7613, 7355, 7095,   64, 8780, 9035, 9161, 9159, 9029, 8771
     c, 7999,   64, 7225, 7354, 7610, 8248, 8632, 8889, 9019,   64
     c, 7610, 8247, 8631, 8888, 9019,    0,  -16, 7116, 6987, 7880
     c, 8007, 7878, 7751, 7752, 7882, 8011, 8396, 8780, 9163, 9289
     c, 9287, 9157, 8771, 8386,   64, 8780, 9035, 9161, 9159, 9029
     c, 8771,   64, 8130, 8386, 8769, 8896, 9022, 9019, 8889, 8760
     c, 8375, 7863, 7480, 7353, 7227, 7228, 7357, 7484, 7355,   64
     c, 8386, 8641, 8768, 8894, 8891, 8761, 8632, 8375,    0,  -16
     c, 7116, 6987, 9035, 8247,   64, 9164, 8375,   64, 9164, 7229
     c, 9277,    0,  -16, 7116, 6987, 8140, 7490,   64, 8140, 9420
     c,   64, 8139, 8779, 9420,   64, 7490, 7619, 8004, 8388, 8771
     c, 8898, 9024, 9021, 8890, 8632, 8247, 7863, 7480, 7353, 7227
     c, 7228, 7357, 7484, 7355,   64, 8388, 8643, 8770, 8896, 8893
     c, 8762, 8504, 8247,    0,  -16, 7116, 6987, 9161, 9032, 9159
     c, 9288, 9289, 9163, 8908, 8524, 8139, 7881, 7622, 7491, 7359
     c, 7355, 7481, 7608, 7863, 8247, 8632, 8890, 9020, 9023, 8897
     c, 8770, 8515, 8131, 7874, 7616, 7486,   64, 8524, 8267, 8009
     c, 7750, 7619, 7487, 7482, 7608,   64, 8247, 8504, 8762, 8892
     c, 8896, 8770,    0,  -16, 7116, 6987, 7756, 7494,   64, 9420
     c, 9289, 9030, 8384, 8125, 7995, 7863,   64, 9030, 8256, 7997
     c, 7867, 7735,   64, 7625, 8012, 8268, 8905,   64, 7754, 8011
     c, 8267, 8905, 9161, 9290, 9420,    0,  -16, 7116, 6987, 8396
     c, 8011, 7882, 7752, 7749, 7875, 8130, 8514, 9027, 9156, 9286
     c, 9289, 9163, 8780, 8396,   64, 8396, 8139, 8010, 7880, 7877
     c, 8003, 8130,   64, 8514, 8899, 9028, 9158, 9161, 9035, 8780
     c,   64, 8130, 7617, 7359, 7229, 7226, 7352, 7735, 8247, 8760
     c, 8889, 9019, 9022, 8896, 8769, 8514,   64, 8130, 7745, 7487
     c, 7357, 7354, 7480, 7735,   64, 8247, 8632, 8761, 8891, 8895
     c, 8769,    0,  -16, 7116, 6987, 9157, 9027, 8769, 8512, 8128
     c, 7873, 7746, 7620, 7623, 7753, 8011, 8396, 8780, 9035, 9162
     c, 9288, 9284, 9152, 9021, 8762, 8504, 8119, 7735, 7480, 7354
     c, 7355, 7484, 7611, 7482,   64, 7873, 7747, 7751, 7881, 8139
     c, 8396,   64, 9035, 9161, 9156, 9024, 8893, 8634, 8376, 8119
     c,    0,  -16, 7116, 7622, 7993, 7864, 7991, 8120, 7993,    0
     c,  -16, 7116, 7622, 7991, 7864, 7993, 8120, 8119, 7989, 7731
     c,    0,  -16, 7116, 7622, 8389, 8260, 8387, 8516, 8389,   64
     c, 7993, 7864, 7991, 8120,    0,  -16, 7116, 7622, 8389, 8260
     c, 8387, 8516, 8389,   64, 7991, 7864, 7993, 8120, 8119, 7989
     c, 7731,    0,  -16, 7116, 7622, 8652, 8523, 8255,   64, 8651
     c, 8255,   64, 8652, 8779, 8255,   64, 7993, 7864, 7991, 8120
     c, 7993,    0,  -16, 7116, 6987, 7880, 8007, 7878, 7751, 7752
     c, 7882, 8011, 8396, 8908, 9291, 9417, 9415, 9285, 9156, 8386
     c, 8129, 8127, 8254, 8510,   64, 8908, 9163, 9289, 9287, 9157
     c, 9028, 8771,   64, 7993, 7864, 7991, 8120, 7993,    0,  -16
     c, 7116, 7622, 8780, 8522, 8392, 8391, 8518, 8647, 8520,    0
     c,  -16, 7116, 7622, 8650, 8523, 8652, 8779, 8778, 8648, 8390
     c,    0,  -16, 7116, 6605, 9540, 9411, 9538, 9667, 9668, 9541
     c, 9413, 9156, 8898, 8250, 7992, 7735, 7351, 6968, 6842, 6844
     c, 6974, 7103, 7360, 8002, 8259, 8517, 8647, 8649, 8523, 8268
     c, 8011, 7881, 7878, 8000, 8125, 8378, 8632, 8887, 9143, 9273
     c, 9274,   64, 7351, 7096, 6970, 6972, 7102, 7231, 8002,   64
     c, 7878, 8001, 8126, 8379, 8633, 8888, 9144, 9273,    0,  -16
     c, 7116, 6987, 8528, 7475,   64, 9168, 8115,   64, 9288, 9159
     c, 9286, 9415, 9416, 9290, 9163, 8780, 8268, 7883, 7625, 7623
     c, 7749, 7876, 8768, 9022,   64, 7623, 7877, 8769, 8896, 9022
     c, 9019, 8889, 8760, 8375, 7863, 7480, 7353, 7227, 7228, 7357
     c, 7484, 7355,    0,  -16, 7116, 6859, 9936, 6576,    0,  -16
     c, 7116, 7368, 9296, 8781, 8394, 8135, 7875, 7742, 7738, 7861
     c, 7986, 8112,   64, 8781, 8393, 8133, 8002, 7869, 7864, 7987
     c, 8112,    0,  -16, 7116, 7239, 8400, 8526, 8651, 8774, 8770
     c, 8637, 8377, 8118, 7731, 7216,   64, 8400, 8525, 8648, 8643
     c, 8510, 8379, 8119, 7731,    0,  -16, 7116, 7241, 8524, 8512
     c,   64, 7881, 9155,   64, 9161, 7875,    0,  -16, 7116, 6605
     c, 7104, 9408,    0,  -16, 7116, 6605, 8265, 8247,   64, 7104
     c, 9408,    0,  -16, 7116, 6605, 7107, 9411,   64, 7101, 9405
     c,    0,  -16, 7116, 7749, 8652, 8389,   64, 8780, 8389,    0
     c,  -16, 7116, 7113, 8012, 7749,   64, 8140, 7749,   64, 9164
     c, 8901,   64, 9292, 8901,    0,  -16, 7116, 7368, 8396, 8139
     c, 8009, 8007, 8133, 8388, 8644, 8901, 9031, 9033, 8907, 8652
     c, 8396,    0,  -16, 7116, 6986, 8268, 7351,   64, 8268, 9143
     c,   64, 8265, 9015,   64, 7613, 8765,   64, 7095, 7863,   64
     c, 8631, 9399,    0,  -16, 7116, 6859, 7500, 7479,   64, 7628
     c, 7607,   64, 7116, 9164, 9158, 9036,   64, 7618, 8642, 9025
     c, 9152, 9278, 9275, 9145, 9016, 8631, 7095,   64, 8642, 8897
     c, 9024, 9150, 9147, 9017, 8888, 8631,    0,  -16, 7116, 6859
     c, 7500, 7479,   64, 7628, 7607,   64, 7116, 8652, 9035, 9162
     c, 9288, 9286, 9156, 9027, 8642,   64, 8652, 8907, 9034, 9160
     c, 9158, 9028, 8899, 8642,   64, 7618, 8642, 9025, 9152, 9278
     c, 9275, 9145, 9016, 8631, 7095,   64, 8642, 8897, 9024, 9150
     c, 9147, 9017, 8888, 8631,    0,  -16, 7116, 7113, 7756, 7735
     c,   64, 7884, 7863,   64, 7372, 9292, 9286, 9164,   64, 7351
     c, 8247,    0,  -16, 7116, 6732, 7756, 7750, 7614, 7482, 7352
     c, 7223,   64, 9036, 9015,   64, 9164, 9143,   64, 7372, 9548
     c,   64, 6839, 9527,   64, 6839, 6832,   64, 6967, 6832,   64
     c, 9399, 9520,   64, 9527, 9520,    0,  -16, 7116, 6858, 7500
     c, 7479,   64, 7628, 7607,   64, 8390, 8382,   64, 7116, 9164
     c, 9158, 9036,   64, 7618, 8386,   64, 7095, 9143, 9149, 9015
     c,    0,  -16, 7116, 6352, 8268, 8247,   64, 8396, 8375,   64
     c, 7884, 8780,   64, 6859, 6986, 6857, 6730, 6731, 6860, 6988
     c, 7115, 7241, 7365, 7491, 7746, 8898, 9155, 9285, 9417, 9547
     c, 9676, 9804, 9931, 9930, 9801, 9674, 9803,   64, 7746, 7489
     c, 7359, 7226, 7096, 6967,   64, 7746, 7617, 7487, 7354, 7224
     c, 7095, 6839, 6712, 6586,   64, 8898, 9153, 9279, 9402, 9528
     c, 9655,   64, 8898, 9025, 9151, 9274, 9400, 9527, 9783, 9912
     c,10042,   64, 7863, 8759,    0,  -16, 7116, 6986, 7497, 7372
     c, 7366, 7497, 7755, 8012, 8524, 8907, 9033, 9030, 8900, 8515
     c, 8131,   64, 8524, 8779, 8905, 8902, 8772, 8515,   64, 8515
     c, 8770, 9024, 9150, 9147, 9017, 8888, 8503, 7863, 7608, 7481
     c, 7355, 7356, 7485, 7612, 7483,   64, 8897, 9022, 9019, 8889
     c, 8760, 8503,    0,  -16, 7116, 6732, 7372, 7351,   64, 7500
     c, 7479,   64, 9036, 9015,   64, 9164, 9143,   64, 6988, 7884
     c,   64, 8652, 9548,   64, 9034, 7481,   64, 6967, 7863,   64
     c, 8631, 9527,    0,  -16, 7116, 6732, 7372, 7351,   64, 7500
     c, 7479,   64, 9036, 9015,   64, 9164, 9143,   64, 6988, 7884
     c,   64, 8652, 9548,   64, 9034, 7481,   64, 6967, 7863,   64
     c, 8631, 9527,   64, 7762, 7763, 7635, 7634, 7760, 8015, 8527
     c, 8784, 8914,    0,  -16, 7116, 6732, 7372, 7351,   64, 7500
     c, 7479,   64, 6988, 7884,   64, 7490, 8386, 8643, 8773, 8905
     c, 9035, 9164, 9292, 9419, 9418, 9289, 9162, 9291,   64, 8386
     c, 8641, 8767, 8890, 9016, 9143,   64, 8386, 8513, 8639, 8762
     c, 8888, 9015, 9271, 9400, 9530,   64, 6967, 7863,    0,  -16
     c, 7116, 6604, 7628, 7622, 7486, 7354, 7224, 7095, 6967, 6840
     c, 6841, 6970, 7097, 6968,   64, 9036, 9015,   64, 9164, 9143
     c,   64, 7244, 9548,   64, 8631, 9527,    0,  -16, 7116, 6733
     c, 7372, 7351,   64, 7500, 8250,   64, 7372, 8247,   64, 9164
     c, 8247,   64, 9164, 9143,   64, 9292, 9271,   64, 6988, 7500
     c,   64, 9164, 9676,   64, 6967, 7735,   64, 8759, 9655,    0
     c,  -16, 7116, 6732, 7372, 7351,   64, 7500, 7479,   64, 9036
     c, 9015,   64, 9164, 9143,   64, 6988, 7884,   64, 8652, 9548
     c,   64, 7490, 9026,   64, 6967, 7863,   64, 8631, 9527,    0
     c,  -16, 7116, 6859, 8140, 7755, 7497, 7367, 7235, 7232, 7356
     c, 7482, 7736, 8119, 8375, 8760, 9018, 9148, 9280, 9283, 9159
     c, 9033, 8779, 8396, 8140,   64, 8140, 7883, 7625, 7495, 7363
     c, 7360, 7484, 7610, 7864, 8119,   64, 8375, 8632, 8890, 9020
     c, 9152, 9155, 9031, 8905, 8651, 8396,    0,  -16, 7116, 6732
     c, 7372, 7351,   64, 7500, 7479,   64, 9036, 9015,   64, 9164
     c, 9143,   64, 6988, 9548,   64, 6967, 7863,   64, 8631, 9527
     c,    0,  -16, 7116, 6859, 7500, 7479,   64, 7628, 7607,   64
     c, 7116, 8652, 9035, 9162, 9288, 9285, 9155, 9026, 8641, 7617
     c,   64, 8652, 8907, 9034, 9160, 9157, 9027, 8898, 8641,   64
     c, 7095, 7991,    0,  -16, 7116, 6858, 9033, 9158, 9164, 9033
     c, 8779, 8396, 8140, 7755, 7497, 7367, 7236, 7231, 7356, 7482
     c, 7736, 8119, 8375, 8760, 9018, 9148,   64, 8140, 7883, 7625
     c, 7495, 7364, 7359, 7484, 7610, 7864, 8119,    0,  -16, 7116
     c, 7114, 8268, 8247,   64, 8396, 8375,   64, 7500, 7366, 7372
     c, 9292, 9286, 9164,   64, 7863, 8759,    0,  -16, 7116, 6987
     c, 7372, 8252,   64, 7500, 8380,   64, 9292, 8380, 8121, 7992
     c, 7735, 7607, 7480, 7481, 7610, 7737, 7608,   64, 7116, 7884
     c,   64, 8780, 9548,    0,  -16, 7116, 6733, 8268, 8247,   64
     c, 8396, 8375,   64, 7884, 8780,   64, 8009, 7496, 7238, 7107
     c, 7104, 7229, 7483, 7994, 8634, 9147, 9405, 9536, 9539, 9414
     c, 9160, 8649, 8009,   64, 8009, 7624, 7366, 7235, 7232, 7357
     c, 7611, 7994,   64, 8634, 9019, 9277, 9408, 9411, 9286, 9032
     c, 8649,   64, 7863, 8759,    0,  -16, 7116, 6986, 7372, 9015
     c,   64, 7500, 9143,   64, 9164, 7351,   64, 7116, 7884,   64
     c, 8652, 9420,   64, 7095, 7863,   64, 8631, 9399,    0,  -16
     c, 7116, 6732, 7372, 7351,   64, 7500, 7479,   64, 9036, 9015
     c,   64, 9164, 9143,   64, 6988, 7884,   64, 8652, 9548,   64
     c, 6967, 9527,   64, 9399, 9520,   64, 9527, 9520,    0,  -16
     c, 7116, 6731, 7372, 7361, 7487, 7870, 8254, 8639, 8897,   64
     c, 7500, 7489, 7615, 7870,   64, 8908, 8887,   64, 9036, 9015
     c,   64, 6988, 7884,   64, 8524, 9420,   64, 8503, 9399,    0
     c,  -16, 7116, 6225, 6860, 6839,   64, 6988, 6967,   64, 8268
     c, 8247,   64, 8396, 8375,   64, 9676, 9655,   64, 9804, 9783
     c,   64, 6476, 7372,   64, 7884, 8780,   64, 9292,10188,   64
     c, 6455,10167,    0,  -16, 7116, 6225, 6860, 6839,   64, 6988
     c, 6967,   64, 8268, 8247,   64, 8396, 8375,   64, 9676, 9655
     c,   64, 9804, 9783,   64, 6476, 7372,   64, 7884, 8780,   64
     c, 9292,10188,   64, 6455,10167,   64,10039,10160,   64,10167
     c,10160,    0,  -16, 7116, 6734, 8012, 7991,   64, 8140, 8119
     c,   64, 7116, 6982, 6988, 8524,   64, 8130, 9026, 9409, 9536
     c, 9662, 9659, 9529, 9400, 9015, 7607,   64, 9026, 9281, 9408
     c, 9534, 9531, 9401, 9272, 9015,    0,  -16, 7116, 6351, 6988
     c, 6967,   64, 7116, 7095,   64, 6604, 7500,   64, 7106, 8002
     c, 8385, 8512, 8638, 8635, 8505, 8376, 7991, 6583,   64, 8002
     c, 8257, 8384, 8510, 8507, 8377, 8248, 7991,   64, 9420, 9399
     c,   64, 9548, 9527,   64, 9036, 9932,   64, 9015, 9911,    0
     c,  -16, 7116, 6987, 7628, 7607,   64, 7756, 7735,   64, 7244
     c, 8140,   64, 7746, 8642, 9025, 9152, 9278, 9275, 9145, 9016
     c, 8631, 7223,   64, 8642, 8897, 9024, 9150, 9147, 9017, 8888
     c, 8631,    0,  -16, 7116, 6987, 7497, 7372, 7366, 7497, 7755
     c, 8140, 8396, 8779, 9033, 9159, 9284, 9279, 9148, 9018, 8760
     c, 8375, 7991, 7608, 7481, 7355, 7356, 7485, 7612, 7483,   64
     c, 8396, 8651, 8905, 9031, 9156, 9151, 9020, 8890, 8632, 8375
     c,   64, 8002, 9154,    0,  -16, 7116, 6352, 6988, 6967,   64
     c, 7116, 7095,   64, 6604, 7500,   64, 6583, 7479,   64, 8780
     c, 8395, 8137, 8007, 7875, 7872, 7996, 8122, 8376, 8759, 9015
     c, 9400, 9658, 9788, 9920, 9923, 9799, 9673, 9419, 9036, 8780
     c,   64, 8780, 8523, 8265, 8135, 8003, 8000, 8124, 8250, 8504
     c, 8759,   64, 9015, 9272, 9530, 9660, 9792, 9795, 9671, 9545
     c, 9291, 9036,   64, 7106, 7874,    0,  -16, 7116, 6859, 8908
     c, 8887,   64, 9036, 9015,   64, 9420, 7884, 7499, 7370, 7240
     c, 7238, 7364, 7491, 7874, 8898,   64, 7884, 7627, 7498, 7368
     c, 7366, 7492, 7619, 7874,   64, 8258, 8001, 7872, 7481, 7352
     c, 7224, 7097,   64, 8001, 7871, 7608, 7479, 7223, 7097, 7098
     c,   64, 8503, 9399,    0,  -16, 7116, 7115, 7747, 7746, 7618
     c, 7619, 7748, 8005, 8517, 8772, 8899, 9025, 9018, 9144, 9271
     c,   64, 8899, 8890, 9016, 9271, 9399,   64, 8897, 8768, 7999
     c, 7614, 7484, 7482, 7608, 7991, 8375, 8632, 8890,   64, 7999
     c, 7742, 7612, 7610, 7736, 7991,    0,  -16, 7116, 6986, 9036
     c, 8907, 8137, 7751, 7492, 7361, 7357, 7482, 7736, 8119, 8375
     c, 8760, 9018, 9149, 9151, 9026, 8772, 8389, 8133, 7748, 7490
     c, 7359,   64, 9036, 8906, 8649, 8136, 7750, 7492,   64, 8133
     c, 7876, 7618, 7487, 7485, 7610, 7864, 8119,   64, 8375, 8632
     c, 8890, 9021, 9023, 8898, 8644, 8389,    0,  -16, 7116, 6986
     c, 7621, 7607,   64, 7749, 7735,   64, 7237, 8645, 9028, 9154
     c, 9153, 9023, 8638,   64, 8645, 8900, 9026, 9025, 8895, 8638
     c,   64, 7742, 8638, 9021, 9147, 9146, 9016, 8631, 7223,   64
     c, 8638, 8893, 9019, 9018, 8888, 8631,    0,  -16, 7116, 6984
     c, 7621, 7607,   64, 7749, 7735,   64, 7237, 9029, 9024, 8901
     c,   64, 7223, 8119,    0,  -16, 7116, 6731, 7749, 7745, 7611
     c, 7480, 7351,   64, 8901, 8887,   64, 9029, 9015,   64, 7365
     c, 9413,   64, 7095, 6962, 6967, 9399, 9394, 9271,    0,  -16
     c, 7116, 6985, 7487, 9023, 9025, 8899, 8772, 8517, 8133, 7748
     c, 7490, 7359, 7357, 7482, 7736, 8119, 8375, 8760, 9018,   64
     c, 8895, 8898, 8772,   64, 8133, 7876, 7618, 7487, 7485, 7610
     c, 7864, 8119,    0,  -16, 7116, 6606, 8261, 8247,   64, 8389
     c, 8375,   64, 7877, 8773,   64, 7236, 7107, 6980, 7109, 7237
     c, 7364, 7616, 7743, 7998, 8638, 8895, 9024, 9284, 9413, 9541
     c, 9668, 9539, 9412,   64, 7998, 7741, 7612, 7352, 7223,   64
     c, 7998, 7740, 7480, 7351, 7095, 6968, 6842,   64, 8638, 8893
     c, 9020, 9272, 9399,   64, 8638, 8892, 9144, 9271, 9527, 9656
     c, 9786,   64, 7863, 8759,    0,  -16, 7116, 7113, 7619, 7493
     c, 7489, 7619, 7748, 8005, 8517, 8900, 9026, 9025, 8895, 8510
     c,   64, 8517, 8772, 8898, 8897, 8767, 8510,   64, 8126, 8510
     c, 8893, 9019, 9018, 8888, 8503, 7991, 7608, 7482, 7483, 7612
     c, 7739, 7610,   64, 8510, 8765, 8891, 8890, 8760, 8503,    0
     c,  -16, 7116, 6859, 7493, 7479,   64, 7621, 7607,   64, 8901
     c, 8887,   64, 9029, 9015,   64, 7109, 8005,   64, 8517, 9413
     c,   64, 7095, 7991,   64, 8503, 9399,   64, 8900, 7608,    0
     c,  -16, 7116, 6859, 7493, 7479,   64, 7621, 7607,   64, 8901
     c, 8887,   64, 9029, 9015,   64, 7109, 8005,   64, 8517, 9413
     c,   64, 7095, 7991,   64, 8503, 9399,   64, 8900, 7608,   64
     c, 7883, 7884, 7756, 7755, 7881, 8136, 8392, 8649, 8779,    0
     c,  -16, 7116, 6986, 7621, 7607,   64, 7749, 7735,   64, 7237
     c, 8133,   64, 7742, 7998, 8383, 8512, 8772, 8901, 9029, 9156
     c, 9027, 8900,   64, 7998, 8381, 8508, 8760, 8887,   64, 7998
     c, 8253, 8380, 8632, 8759, 9015, 9144, 9274,   64, 7223, 8119
     c,    0,  -16, 7116, 6859, 7749, 7745, 7611, 7480, 7351, 7223
     c, 7096, 7225, 7352,   64, 8901, 8887,   64, 9029, 9015,   64
     c, 7365, 9413,   64, 8503, 9399,    0,  -16, 7116, 6860, 7493
     c, 7479,   64, 7493, 8247,   64, 7621, 8249,   64, 9029, 8247
     c,   64, 9029, 9015,   64, 9157, 9143,   64, 7109, 7621,   64
     c, 9029, 9541,   64, 7095, 7863,   64, 8631, 9527,    0,  -16
     c, 7116, 6859, 7493, 7479,   64, 7621, 7607,   64, 8901, 8887
     c,   64, 9029, 9015,   64, 7109, 8005,   64, 8517, 9413,   64
     c, 7614, 8894,   64, 7095, 7991,   64, 8503, 9399,    0,  -16
     c, 7116, 6986, 8133, 7748, 7490, 7359, 7357, 7482, 7736, 8119
     c, 8375, 8760, 9018, 9149, 9151, 9026, 8772, 8389, 8133,   64
     c, 8133, 7876, 7618, 7487, 7485, 7610, 7864, 8119,   64, 8375
     c, 8632, 8890, 9021, 9023, 8898, 8644, 8389,    0,  -16, 7116
     c, 6859, 7493, 7479,   64, 7621, 7607,   64, 8901, 8887,   64
     c, 9029, 9015,   64, 7109, 9413,   64, 7095, 7991,   64, 8503
     c, 9399,    0,  -16, 7116, 6858, 7493, 7472,   64, 7621, 7600
     c,   64, 7618, 7876, 8133, 8389, 8772, 9026, 9151, 9149, 9018
     c, 8760, 8375, 8119, 7864, 7610,   64, 8389, 8644, 8898, 9023
     c, 9021, 8890, 8632, 8375,   64, 7109, 7621,   64, 7088, 7984
     c,    0,  -16, 7116, 6985, 8898, 8769, 8896, 9025, 9026, 8772
     c, 8517, 8133, 7748, 7490, 7359, 7357, 7482, 7736, 8119, 8375
     c, 8760, 9018,   64, 8133, 7876, 7618, 7487, 7485, 7610, 7864
     c, 8119,    0,  -16, 7116, 7114, 8261, 8247,   64, 8389, 8375
     c,   64, 7621, 7488, 7493, 9157, 9152, 9029,   64, 7863, 8759
     c,    0,  -16, 7116, 7113, 7493, 8247,   64, 7621, 8249,   64
     c, 9029, 8247, 7987, 7729, 7472, 7344, 7217, 7346, 7473,   64
     c, 7237, 8005,   64, 8517, 9285,    0,  -16, 7116, 6987, 8268
     c, 8240,   64, 8396, 8368,   64, 7884, 8396,   64, 8258, 8132
     c, 8005, 7749, 7492, 7361, 7355, 7480, 7735, 7991, 8120, 8250
     c,   64, 7749, 7620, 7489, 7483, 7608, 7735,   64, 8901, 9028
     c, 9153, 9147, 9016, 8887,   64, 8386, 8516, 8645, 8901, 9156
     c, 9281, 9275, 9144, 8887, 8631, 8504, 8378,   64, 7856, 8752
     c,    0,  -16, 7116, 6986, 7493, 8887,   64, 7621, 9015,   64
     c, 9029, 7479,   64, 7237, 8005,   64, 8517, 9285,   64, 7223
     c, 7991,   64, 8503, 9271,    0,  -16, 7116, 6859, 7493, 7479
     c,   64, 7621, 7607,   64, 8901, 8887,   64, 9029, 9015,   64
     c, 7109, 8005,   64, 8517, 9413,   64, 7095, 9399, 9394, 9271
     c,    0,  -16, 7116, 6859, 7493, 7486, 7612, 7995, 8251, 8636
     c, 8894,   64, 7621, 7614, 7740, 7995,   64, 8901, 8887,   64
     c, 9029, 9015,   64, 7109, 8005,   64, 8517, 9413,   64, 8503
     c, 9399,    0,  -16, 7116, 6352, 6981, 6967,   64, 7109, 7095
     c,   64, 8261, 8247,   64, 8389, 8375,   64, 9541, 9527,   64
     c, 9669, 9655,   64, 6597, 7493,   64, 7877, 8773,   64, 9157
     c,10053,   64, 6583,10039,    0,  -16, 7116, 6352, 6981, 6967
     c,   64, 7109, 7095,   64, 8261, 8247,   64, 8389, 8375,   64
     c, 9541, 9527,   64, 9669, 9655,   64, 6597, 7493,   64, 7877
     c, 8773,   64, 9157,10053,   64, 6583,10039,10034, 9911,    0
     c,  -16, 7116, 6987, 8133, 8119,   64, 8261, 8247,   64, 7493
     c, 7360, 7365, 8645,   64, 8254, 8766, 9149, 9275, 9274, 9144
     c, 8759, 7735,   64, 8766, 9021, 9147, 9146, 9016, 8759,    0
     c,  -16, 7116, 6605, 7237, 7223,   64, 7365, 7351,   64, 6853
     c, 7749,   64, 7358, 7870, 8253, 8379, 8378, 8248, 7863, 6839
     c,   64, 7870, 8125, 8251, 8250, 8120, 7863,   64, 9157, 9143
     c,   64, 9285, 9271,   64, 8773, 9669,   64, 8759, 9655,    0
     c,  -16, 7116, 7241, 7877, 7863,   64, 8005, 7991,   64, 7493
     c, 8389,   64, 7998, 8510, 8893, 9019, 9018, 8888, 8503, 7479
     c,   64, 8510, 8765, 8891, 8890, 8760, 8503,    0,  -16, 7116
     c, 7114, 7619, 7493, 7489, 7619, 7748, 8005, 8389, 8772, 9026
     c, 9151, 9149, 9018, 8760, 8375, 7991, 7736, 7482, 7483, 7612
     c, 7739, 7610,   64, 8389, 8644, 8898, 9023, 9021, 8890, 8632
     c, 8375,   64, 8254, 9022,    0,  -16, 7116, 6479, 7109, 7095
     c,   64, 7237, 7223,   64, 6725, 7621,   64, 6711, 7607,   64
     c, 8773, 8388, 8130, 7999, 7997, 8122, 8376, 8759, 9015, 9400
     c, 9658, 9789, 9791, 9666, 9412, 9029, 8773,   64, 8773, 8516
     c, 8258, 8127, 8125, 8250, 8504, 8759,   64, 9015, 9272, 9530
     c, 9661, 9663, 9538, 9284, 9029,   64, 7230, 7998,    0,  -16
     c, 7116, 6858, 8773, 8759,   64, 8901, 8887,   64, 9285, 7877
     c, 7492, 7362, 7361, 7487, 7870, 8766,   64, 7877, 7620, 7490
     c, 7489, 7615, 7870,   64, 8510, 8125, 7996, 7736, 7607,   64
     c, 8510, 8253, 8124, 7864, 7735, 7479, 7352, 7226,   64, 8375
     c, 9271,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0
     c,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0/

      NC1 = 1
			NC2 = 3000

      RETURN
      END
